@@ -170,7 +170,8 @@ class API(wsgi.Router):
                     {
                         'name': 'stack_lookup_subpath',
                         'url': '/stacks/{stack_name}/'
-                               '{path:resources|events|template|actions}',
+                               '{path:resources|events|template|actions'
+                               '|environment|files}',
                         'action': 'lookup',
                         'method': 'GET'
                     },
@@ -193,6 +194,18 @@ class API(wsgi.Router):
                         'action': 'template',
                         'method': 'GET'
                     },
+                    {
+                        'name': 'stack_lookup',
+                        'url': '/stacks/{stack_name}/{stack_id}/environment',
+                        'action': 'environment',
+                        'method': 'GET'
+                    },
+                    {
+                        'name': 'stack_lookup',
+                        'url': '/stacks/{stack_name}/{stack_id}/files',
+                        'action': 'files',
+                        'method': 'GET'
+                    },
 
                     # Stack update/delete
                     {
