@@ -26,8 +26,7 @@ from heat.common import exception
 from heat.common.i18n import _
 from heat.common import messaging
 from heat.common import service_utils
-from heat.db import api as db_api
-from heat.db import utils
+from heat.db.sqlalchemy import api as db_api
 from heat.objects import service as service_objects
 from heat.rpc import client as rpc_client
 from heat import version
@@ -131,9 +130,10 @@ def do_migrate():
 
 def purge_deleted():
     """Remove database records that have been previously soft deleted."""
-    utils.purge_deleted(CONF.command.age,
-                        CONF.command.granularity,
-                        CONF.command.project_id)
+    db_api.purge_deleted(CONF.command.age,
+                         CONF.command.granularity,
+                         CONF.command.project_id,
+                         CONF.command.batch_size)
 
 
 def do_crypt_parameters_and_properties():
@@ -141,13 +141,18 @@ def do_crypt_parameters_and_properties():
     ctxt = context.get_admin_context()
     prev_encryption_key = CONF.command.previous_encryption_key
     if CONF.command.crypt_operation == "encrypt":
-        utils.encrypt_parameters_and_properties(
+        db_api.encrypt_parameters_and_properties(
             ctxt, prev_encryption_key, CONF.command.verbose_update_params)
     elif CONF.command.crypt_operation == "decrypt":
-        utils.decrypt_parameters_and_properties(
+        db_api.decrypt_parameters_and_properties(
             ctxt, prev_encryption_key, CONF.command.verbose_update_params)
 
 
+def do_properties_data_migrate():
+    ctxt = context.get_admin_context()
+    db_api.db_properties_data_migrate(ctxt)
+
+
 def add_command_parsers(subparsers):
     # db_version parser
     parser = subparsers.add_parser('db_version')
@@ -159,8 +164,8 @@ def add_command_parsers(subparsers):
     # positional parameter, can be skipped. default=None
     parser.add_argument('version', nargs='?')
 
-    # migrate-stacks parser
-    parser = subparsers.add_parser('migrate-convergence-1')
+    # migrate_convergence_1 parser
+    parser = subparsers.add_parser('migrate_convergence_1')
     parser.set_defaults(func=do_migrate)
     parser.add_argument('stack_id')
 
@@ -179,6 +184,13 @@ def add_command_parsers(subparsers):
     parser.add_argument(
         '-p', '--project-id',
         help=_('Project ID to purge deleted stacks.'))
+    # optional parameter, can be skipped. default='20'
+    parser.add_argument(
+        '-b', '--batch_size', default='20',
+        help=_('Number of stacks to delete at a time (per transaction). '
+               'Note that a single stack may have many db rows '
+               '(events, etc.) associated with it.'))
+
     # update_params parser
     parser = subparsers.add_parser('update_params')
     parser.set_defaults(func=do_crypt_parameters_and_properties)
@@ -208,11 +220,15 @@ def add_command_parsers(subparsers):
     parser.add_argument('stack_id',
                         help=_('Stack id'))
 
+    # migrate properties_data parser
+    parser = subparsers.add_parser('migrate_properties_data')
+    parser.set_defaults(func=do_properties_data_migrate)
+
     ServiceManageCommand.add_service_parsers(subparsers)
 
 command_opt = cfg.SubCommandOpt('command',
                                 title='Commands',
-                                help='Show available commands.',
+                                help=_('Show available commands.'),
                                 handler=add_command_parsers)
 
 
