@@ -28,7 +28,6 @@ import six
 from heat.common import config
 from heat.common import endpoint_utils
 from heat.common import exception
-from heat.common.i18n import _LE
 from heat.common import policy
 from heat.common import wsgi
 from heat.db.sqlalchemy import api as db_api
@@ -146,10 +145,8 @@ class RequestContext(context.RequestContext):
 
     @property
     def keystone_session(self):
-        if self.auth_needs_refresh():
-            self.reload_auth_plugin()
-            self.clients.invalidate_plugins()
-        self._keystone_session.auth = self.auth_plugin
+        if not self._keystone_session.auth:
+            self._keystone_session.auth = self.auth_plugin
         return self._keystone_session
 
     @property
@@ -158,12 +155,6 @@ class RequestContext(context.RequestContext):
             self._clients = clients.Clients(self)
         return self._clients
 
-    def auth_needs_refresh(self):
-        auth_ref = self.auth_plugin.get_auth_ref(self._keystone_session)
-        return (cfg.CONF.reauthentication_auth_method == 'trusts'
-                and auth_ref.will_expire_soon(
-                    cfg.CONF.stale_token_duration))
-
     def to_dict(self):
         user_idt = '{user} {tenant}'.format(user=self.user_id or '-',
                                             tenant=self.tenant_id or '-')
@@ -183,11 +174,12 @@ class RequestContext(context.RequestContext):
                 'is_admin': self.is_admin,
                 'user': self.username,
                 'request_id': self.request_id,
+                'global_request_id': self.global_request_id,
                 'show_deleted': self.show_deleted,
                 'region_name': self.region_name,
                 'user_identity': user_idt,
-                'user_domain_id': self.user_domain,
-                'project_domain_id': self.project_domain}
+                'user_domain': self.user_domain,
+                'project_domain': self.project_domain}
 
     @classmethod
     def from_dict(cls, values):
@@ -235,9 +227,9 @@ class RequestContext(context.RequestContext):
             if auth_uri:
                 return auth_uri
             else:
-                LOG.error(_LE('Keystone API endpoint not provided. Set '
-                              'auth_uri in section [clients_keystone] '
-                              'of the configuration file.'))
+                LOG.error('Keystone API endpoint not provided. Set '
+                          'auth_uri in section [clients_keystone] '
+                          'of the configuration file.')
                 raise exception.AuthorizationFailure()
 
     @property
@@ -247,8 +239,8 @@ class RequestContext(context.RequestContext):
                 cfg.CONF, TRUSTEE_CONF_GROUP, trust_id=self.trust_id)
 
         if not self._trusts_auth_plugin:
-            LOG.error(_LE('Please add the trustee credentials you need '
-                          'to the %s section of your heat.conf file.'),
+            LOG.error('Please add the trustee credentials you need '
+                      'to the %s section of your heat.conf file.',
                       TRUSTEE_CONF_GROUP)
             raise exception.AuthorizationFailure()
 
@@ -261,6 +253,13 @@ class RequestContext(context.RequestContext):
             return access_plugin.AccessInfoPlugin(
                 auth_ref=access_info, auth_url=self.keystone_v3_endpoint)
 
+        if self.password:
+            return generic.Password(username=self.username,
+                                    password=self.password,
+                                    project_id=self.tenant_id,
+                                    user_domain_id=self.user_domain,
+                                    auth_url=self.keystone_v3_endpoint)
+
         if self.auth_token:
             # FIXME(jamielennox): This is broken but consistent. If you
             # only have a token but don't load a service catalog then
@@ -269,15 +268,8 @@ class RequestContext(context.RequestContext):
             return token_endpoint.Token(endpoint=self.keystone_v3_endpoint,
                                         token=self.auth_token)
 
-        if self.password:
-            return generic.Password(username=self.username,
-                                    password=self.password,
-                                    project_id=self.tenant_id,
-                                    user_domain_id=self.user_domain,
-                                    auth_url=self.keystone_v3_endpoint)
-
-        LOG.error(_LE("Keystone API connection failed, no password "
-                      "trust or auth_token!"))
+        LOG.error("Keystone API connection failed, no password "
+                  "trust or auth_token!")
         raise exception.AuthorizationFailure()
 
     def reload_auth_plugin(self):
@@ -297,7 +289,7 @@ class RequestContext(context.RequestContext):
 class StoredContext(RequestContext):
     def _load_keystone_data(self):
         self._keystone_loaded = True
-        auth_ref = self.clients.client('keystone').auth_ref
+        auth_ref = self.auth_plugin.get_access(self.keystone_session)
 
         self.roles = auth_ref.role_names
         self.user_domain = auth_ref.user_domain_id
@@ -360,6 +352,8 @@ class ContextMiddleware(wsgi.Middleware):
         username = None
         password = None
         aws_creds = None
+        user_domain = None
+        project_domain = None
 
         if headers.get('X-Auth-User') is not None:
             username = headers.get('X-Auth-User')
@@ -367,6 +361,12 @@ class ContextMiddleware(wsgi.Middleware):
         elif headers.get('X-Auth-EC2-Creds') is not None:
             aws_creds = headers.get('X-Auth-EC2-Creds')
 
+        if headers.get('X-User-Domain-Id') is not None:
+            user_domain = headers.get('X-User-Domain-Id')
+
+        if headers.get('X-Project-Domain-Id') is not None:
+            project_domain = headers.get('X-Project-Domain-Id')
+
         project_name = headers.get('X-Project-Name')
         region_name = headers.get('X-Region-Name')
         auth_url = headers.get('X-Auth-Url')
@@ -383,6 +383,8 @@ class ContextMiddleware(wsgi.Middleware):
             password=password,
             auth_url=auth_url,
             request_id=req_id,
+            user_domain=user_domain,
+            project_domain=project_domain,
             auth_token_info=token_info,
             region_name=region_name,
             auth_plugin=auth_plugin,
