@@ -24,7 +24,6 @@ from oslo_utils import excutils
 import six
 
 from heat.common.i18n import _
-from heat.common.i18n import _LE
 
 _FATAL_EXCEPTION_FORMAT_ERRORS = False
 
@@ -56,6 +55,8 @@ class HeatException(Exception):
     # YYY - Specific error code for a given exception.
     error_code = None
 
+    safe = True
+
     def __init__(self, **kwargs):
         self.kwargs = kwargs
 
@@ -69,9 +70,9 @@ class HeatException(Exception):
                     reraise=_FATAL_EXCEPTION_FORMAT_ERRORS):
                 # kwargs doesn't match a variable in the message
                 # log the issue and the kwargs
-                LOG.exception(_LE('Exception in string format operation'))
+                LOG.exception('Exception in string format operation')
                 for name, value in six.iteritems(kwargs):
-                    LOG.error(_LE("%(name)s: %(value)s"),
+                    LOG.error("%(name)s: %(value)s",
                               {'name': name, 'value': value})  # noqa
 
         if self.error_code:
@@ -162,6 +163,11 @@ class TemplateOutputError(HeatException):
     msg_fmt = _('Error in %(resource)s output %(attribute)s: %(message)s')
 
 
+class InvalidEncryptionKey(HeatException):
+    msg_fmt = _('Can not decrypt data with the auth_encryption_key'
+                ' in heat config.')
+
+
 class InvalidExternalResourceDependency(HeatException):
     msg_fmt = _("Invalid dependency with external %(resource_type)s "
                 "resource: %(external_id)s")
@@ -228,18 +234,35 @@ class HeatExceptionWithPath(HeatException):
             message=self.error_message
         )
 
-    def error(self):
-        return self.error
-
-    def path(self):
-        return self.path
-
-    def error_message(self):
-        return self.error_message
-
 
 class StackValidationFailed(HeatExceptionWithPath):
-    pass
+    def __init__(self, error=None, path=None, message=None,
+                 resource=None):
+        if path is None:
+            path = []
+        elif isinstance(path, six.string_types):
+            path = [path]
+
+        if resource is not None and not path:
+            path = [resource.stack.t.get_section_name(
+                resource.stack.t.RESOURCES), resource.name]
+        if isinstance(error, Exception):
+            if isinstance(error, StackValidationFailed):
+                str_error = error.error
+                message = error.error_message
+                path = path + error.path
+                # This is a hack to avoid the py3 (chained exception)
+                # json serialization circular reference error from
+                # oslo.messaging.
+                self.args = error.args
+            else:
+                str_error = six.text_type(type(error).__name__)
+                message = six.text_type(error)
+        else:
+            str_error = error
+
+        super(StackValidationFailed, self).__init__(error=str_error, path=path,
+                                                    message=message)
 
 
 class InvalidSchemaError(HeatException):
@@ -281,11 +304,6 @@ class ClientNotAvailable(HeatException):
     msg_fmt = _("The client (%(client_name)s) is not available.")
 
 
-class WatchRuleNotFound(EntityNotFound):
-    """Keep this for AWS compatibility."""
-    msg_fmt = _("The Watch Rule (%(watch_name)s) could not be found.")
-
-
 class ResourceFailure(HeatExceptionWithPath):
     def __init__(self, exception_or_error, resource, action=None):
         self.resource = resource
@@ -427,6 +445,8 @@ class HTTPExceptionDisguise(Exception):
     They can be handled by the webob fault application in the wsgi pipeline.
     """
 
+    safe = True
+
     def __init__(self, exception):
         self.exc = exception
         self.tb = sys.exc_info()[2]
@@ -531,3 +551,7 @@ class InvalidServiceVersion(HeatException):
 class InvalidTemplateVersions(HeatException):
     msg_fmt = _('A template version alias %(version)s was added for a '
                 'template class that has no official YYYY-MM-DD version.')
+
+
+class UnableToAutoAllocateNetwork(HeatException):
+    msg_fmt = _('Unable to automatically allocate a network: %(message)s')
