@@ -14,14 +14,13 @@
 from oslo_log import log as logging
 import six
 
-from heat.common.i18n import _LW
 
 LOG = logging.getLogger(__name__)
 
 
 def log_fail_msg(manager, entrypoint, exception):
-    LOG.warning(_LW('Encountered exception while loading %(module_name)s: '
-                '"%(message)s". Not using %(name)s.'),
+    LOG.warning('Encountered exception while loading %(module_name)s: '
+                '"%(message)s". Not using %(name)s.',
                 {'module_name': entrypoint.module_name,
                  'message': getattr(exception, 'message',
                                     six.text_type(exception)),
