@@ -175,9 +175,14 @@ def raw_template_files_get(context, files_id):
     return result
 
 
-def resource_get(context, resource_id, refresh=False, refresh_data=False):
-    result = context.session.query(models.Resource).get(resource_id)
+def resource_get(context, resource_id, refresh=False, refresh_data=False,
+                 eager=True):
+    query = context.session.query(models.Resource)
+    query = query.options(orm.joinedload("data"))
+    if eager:
+        query = query.options(orm.joinedload("rsrc_prop_data"))
 
+    result = query.get(resource_id)
     if not result:
         raise exception.NotFound(_("resource with id %s not found") %
                                  resource_id)
@@ -249,6 +254,8 @@ def _add_atomic_key_to_values(values, atomic_key):
         values['atomic_key'] = atomic_key + 1
 
 
+@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
+                           retry_interval=0.5, inc_retry_interval=True)
 def resource_update(context, resource_id, values, atomic_key,
                     expected_engine_id=None):
     session = context.session
@@ -439,12 +446,38 @@ def resource_create(context, values):
     return resource_ref
 
 
+def resource_create_replacement(context,
+                                existing_res_id, existing_res_values,
+                                new_res_values,
+                                atomic_key, expected_engine_id=None):
+    session = context.session
+    try:
+        with session.begin(subtransactions=True):
+            new_res = resource_create(context, new_res_values)
+            update_data = {'replaced_by': new_res.id}
+            update_data.update(existing_res_values)
+            if not resource_update(context,
+                                   existing_res_id, update_data,
+                                   atomic_key,
+                                   expected_engine_id=expected_engine_id):
+                data = {}
+                if 'name' in new_res_values:
+                    data['resource_name'] = new_res_values['name']
+                raise exception.UpdateInProgress(**data)
+    except db_exception.DBReferenceError as exc:
+        # New template_id no longer exists
+        LOG.debug('Not creating replacement resource: %s', exc)
+        return None
+    else:
+        return new_res
+
+
 def resource_get_all_by_stack(context, stack_id, filters=None):
     query = context.session.query(
         models.Resource
     ).filter_by(
         stack_id=stack_id
-    ).options(orm.joinedload("data"))
+    ).options(orm.joinedload("data")).options(orm.joinedload("rsrc_prop_data"))
 
     query = db_filters.exact_filter(query, models.Resource, filters)
     results = query.all()
@@ -464,12 +497,19 @@ def resource_get_all_active_by_stack(context, stack_id):
     return dict((res.id, res) for res in results)
 
 
-def resource_get_all_by_root_stack(context, stack_id, filters=None):
+def resource_get_all_by_root_stack(context, stack_id, filters=None,
+                                   stack_id_only=False):
     query = context.session.query(
         models.Resource
     ).filter_by(
         root_stack_id=stack_id
-    ).options(orm.joinedload("data"))
+    )
+
+    if stack_id_only:
+        query = query.options(orm.load_only("id", "stack_id"))
+    else:
+        query = query.options(orm.joinedload("data")).options(
+            orm.joinedload("rsrc_prop_data"))
 
     query = db_filters.exact_filter(query, models.Resource, filters)
     results = query.all()
@@ -987,31 +1027,33 @@ def _delete_event_rows(context, stack_id, limit):
     # So we must manually supply the IN() values.
     # pgsql SHOULD work with the pure DELETE/JOIN below but that must be
     # confirmed via integration tests.
+    query = _query_all_by_stack(context, stack_id)
     session = context.session
-    with session.begin():
-        query = _query_all_by_stack(context, stack_id)
-        query = query.order_by(models.Event.id).limit(limit)
-        id_pairs = [(e.id, e.rsrc_prop_data_id) for e in query.all()]
-        if not id_pairs:
-            return 0
-        (ids, rsrc_prop_ids) = zip(*id_pairs)
-        max_id = ids[-1]
-        # delete the events
-        retval = session.query(models.Event.id).filter(
-            models.Event.id <= max_id).filter(
-                models.Event.stack_id == stack_id).delete()
+    id_pairs = [(e.id, e.rsrc_prop_data_id) for e in query.order_by(
+        models.Event.id).limit(limit).all()]
+    if id_pairs is None:
+        return 0
+    (ids, rsrc_prop_ids) = zip(*id_pairs)
+    max_id = ids[-1]
+    # delete the events
+    retval = session.query(models.Event.id).filter(
+        models.Event.id <= max_id).filter(
+            models.Event.stack_id == stack_id).delete()
 
-        # delete unreferenced resource_properties_data
-        if rsrc_prop_ids:
-            ev_ref_ids = set(e.rsrc_prop_data_id for e
-                             in _query_all_by_stack(context, stack_id).all())
-            rsrc_ref_ids = set(r.rsrc_prop_data_id for r
-                               in session.query(models.Resource).filter_by(
-                                   stack_id=stack_id).all())
-            clr_prop_ids = set(rsrc_prop_ids) - ev_ref_ids - rsrc_ref_ids
-            q_rpd = session.query(models.ResourcePropertiesData.id).filter(
-                models.ResourcePropertiesData.id.in_(clr_prop_ids))
-            q_rpd.delete(synchronize_session=False)
+    # delete unreferenced resource_properties_data
+    rsrc_prop_ids = set(rsrc_prop_ids)
+    if rsrc_prop_ids:
+        still_ref_ids_from_events = [e.rsrc_prop_data_id for e
+                                     in _query_all_by_stack(
+                                         context, stack_id).all()]
+        still_ref_ids_from_rsrcs = [r.rsrc_prop_data_id for r
+                                    in context.session.query(models.Resource).
+                                    filter_by(stack_id=stack_id).all()]
+        rsrc_prop_ids = rsrc_prop_ids - set(still_ref_ids_from_events) \
+            - set(still_ref_ids_from_rsrcs)
+        q_rpd = session.query(models.ResourcePropertiesData.id).filter(
+            models.ResourcePropertiesData.id.in_(rsrc_prop_ids))
+        q_rpd.delete(synchronize_session=False)
     return retval
 
 
@@ -1024,11 +1066,8 @@ def event_create(context, values):
             (event_count_all_by_stack(context, values['stack_id']) >=
              cfg.CONF.max_events_per_stack)):
             # prune
-            try:
-                _delete_event_rows(context, values['stack_id'],
-                                   cfg.CONF.event_purge_batch_size)
-            except db_exception.DBError as exc:
-                LOG.error('Failed to purge events: %s', six.text_type(exc))
+            _delete_event_rows(
+                context, values['stack_id'], cfg.CONF.event_purge_batch_size)
     event_ref = models.Event()
     event_ref.update(values)
     event_ref.save(context.session)
@@ -1291,12 +1330,14 @@ def service_get_all_by_args(context, host, binary, hostname):
 def purge_deleted(age, granularity='days', project_id=None, batch_size=20):
     def _validate_positive_integer(val, argname):
         try:
-            return int(val)
+            val = int(val)
         except ValueError:
             raise exception.Error(_("%s should be an integer") % argname)
+
         if val < 0:
             raise exception.Error(_("%s should be a positive integer")
                                   % argname)
+        return val
 
     age = _validate_positive_integer(age, 'age')
     batch_size = _validate_positive_integer(batch_size, 'batch_size')
