@@ -93,10 +93,15 @@ class CheckResource(object):
             # Another concurrent update has taken over. But there is a
             # possibility for that update to be waiting for this rsrc to
             # complete, hence retrigger current rsrc for latest traversal.
-            traversal = stack.current_traversal
-            latest_stack = parser.Stack.load(cnxt, stack_id=stack.id,
+            self._retrigger_new_traversal(cnxt, stack.current_traversal,
+                                          is_update,
+                                          stack.id, rsrc_id)
+
+    def _retrigger_new_traversal(self, cnxt, current_traversal, is_update,
+                                 stack_id, rsrc_id):
+            latest_stack = parser.Stack.load(cnxt, stack_id=stack_id,
                                              force_reload=True)
-            if traversal != latest_stack.current_traversal:
+            if current_traversal != latest_stack.current_traversal:
                 self.retrigger_check_resource(cnxt, is_update, rsrc_id,
                                               latest_stack)
 
@@ -104,6 +109,30 @@ class CheckResource(object):
         failure_reason = u'Timed out'
         self._handle_failure(cnxt, stack, failure_reason)
 
+    def _handle_resource_replacement(self, cnxt,
+                                     current_traversal, new_tmpl_id,
+                                     rsrc, stack, adopt_stack_data):
+        """Create a replacement resource and trigger a check on it."""
+        try:
+            new_res_id = rsrc.make_replacement(new_tmpl_id)
+        except exception.UpdateInProgress:
+            LOG.info("No replacement created - "
+                     "resource already locked by new traversal")
+            return
+        if new_res_id is None:
+            LOG.info("No replacement created - "
+                     "new traversal already in progress")
+            self._retrigger_new_traversal(cnxt, current_traversal, True,
+                                          stack.id, rsrc.id)
+            return
+        LOG.info("Replacing resource with new id %s", new_res_id)
+        rpc_data = sync_point.serialize_input_data(self.input_data)
+        self._rpc_client.check_resource(cnxt,
+                                        new_res_id,
+                                        current_traversal,
+                                        rpc_data, True,
+                                        adopt_stack_data)
+
     def _do_check_resource(self, cnxt, current_traversal, tmpl, resource_data,
                            is_update, rsrc, stack, adopt_stack_data):
         try:
@@ -113,15 +142,10 @@ class CheckResource(object):
                                           self.engine_id,
                                           stack, self.msg_queue)
                 except resource.UpdateReplace:
-                    new_res_id = rsrc.make_replacement(tmpl.id)
-                    LOG.info("Replacing resource with new id %s",
-                             new_res_id)
-                    rpc_data = sync_point.serialize_input_data(self.input_data)
-                    self._rpc_client.check_resource(cnxt,
-                                                    new_res_id,
-                                                    current_traversal,
-                                                    rpc_data, is_update,
-                                                    adopt_stack_data)
+                    self._handle_resource_replacement(cnxt, current_traversal,
+                                                      tmpl.id,
+                                                      rsrc, stack,
+                                                      adopt_stack_data)
                     return False
 
             else:
@@ -151,13 +175,13 @@ class CheckResource(object):
             self._handle_resource_failure(cnxt, is_update, rsrc.id,
                                           stack, reason)
         except scheduler.Timeout:
-            # reload the stack to verify current traversal
-            stack = parser.Stack.load(cnxt, stack_id=stack.id)
-            if stack.current_traversal != current_traversal:
-                return
-            self._handle_stack_timeout(cnxt, stack)
-        except CancelOperation:
-            pass
+            self._handle_resource_failure(cnxt, is_update, rsrc.id,
+                                          stack, u'Timed out')
+        except CancelOperation as ex:
+            # Stack is already marked FAILED, so we just need to retrigger
+            # in case a new traversal has started and is waiting on us.
+            self._retrigger_new_traversal(cnxt, current_traversal, is_update,
+                                          stack.id, rsrc.id)
 
         return False
 
@@ -195,7 +219,7 @@ class CheckResource(object):
                                      stack):
         deps = stack.convergence_dependencies
         graph = deps.graph()
-        graph_key = (resource_id, is_update)
+        graph_key = parser.ConvergenceNode(resource_id, is_update)
 
         if graph_key not in graph and rsrc.replaces is not None:
             # If we are a replacement, impersonate the replaced resource for
@@ -204,10 +228,10 @@ class CheckResource(object):
             # graph. Our real resource ID is sent in the input_data, so the
             # dependencies will get updated to point to this resource in time
             # for the next traversal.
-            graph_key = (rsrc.replaces, is_update)
+            graph_key = parser.ConvergenceNode(rsrc.replaces, is_update)
 
-        def _get_input_data(req, fwd, input_forward_data=None):
-            if fwd:
+        def _get_input_data(req_node, input_forward_data=None):
+            if req_node.is_update:
                 if input_forward_data is None:
                     return rsrc.node_data().as_dict()
                 else:
@@ -216,7 +240,7 @@ class CheckResource(object):
             else:
                 # Don't send data if initiating clean-up for self i.e.
                 # initiating delete of a replaced resource
-                if req not in graph_key:
+                if req_node.rsrc_id != graph_key.rsrc_id:
                     # send replaced resource as needed_by if it exists
                     return (rsrc.replaced_by
                             if rsrc.replaced_by is not None
@@ -225,13 +249,15 @@ class CheckResource(object):
 
         try:
             input_forward_data = None
-            for req, fwd in deps.required_by(graph_key):
-                input_data = _get_input_data(req, fwd, input_forward_data)
-                if fwd:
+            for req_node in sorted(deps.required_by(graph_key),
+                                   key=lambda n: n.is_update):
+                input_data = _get_input_data(req_node, input_forward_data)
+                if req_node.is_update:
                     input_forward_data = input_data
                 propagate_check_resource(
-                    cnxt, self._rpc_client, req, current_traversal,
-                    set(graph[(req, fwd)]), graph_key, input_data, fwd,
+                    cnxt, self._rpc_client, req_node.rsrc_id,
+                    current_traversal, set(graph[req_node]),
+                    graph_key, input_data, req_node.is_update,
                     stack.adopt_stack_data)
             if is_update:
                 if input_forward_data is None:
@@ -241,7 +267,7 @@ class CheckResource(object):
                 else:
                     rsrc.store_attributes()
             check_stack_complete(cnxt, stack, current_traversal,
-                                 graph_key[0], deps, graph_key[1])
+                                 graph_key.rsrc_id, deps, graph_key.is_update)
         except exception.EntityNotFound as e:
             if e.entity == "Sync Point":
                 # Reload the stack to determine the current traversal, and
@@ -279,6 +305,9 @@ class CheckResource(object):
         if is_update:
             if (rsrc.replaced_by is not None and
                     rsrc.current_template_id != tmpl.id):
+                LOG.debug('Resource %s with id %s already replaced by %s; '
+                          'not checking',
+                          rsrc.name, resource_id, rsrc.replaced_by)
                 return
 
         try:
