@@ -21,7 +21,6 @@ from stevedore import enabled
 
 from heat.common import exception
 from heat.common.i18n import _
-from heat.common.i18n import _LW
 from heat.common import pluginutils
 
 LOG = logging.getLogger(__name__)
@@ -52,11 +51,6 @@ class OpenStackClients(object):
         assert ctxt is not None, "Need a reference to the context"
         return ctxt
 
-    def invalidate_plugins(self):
-        """Used to force plugins to clear any cached client."""
-        for name in self._client_plugins:
-            self._client_plugins[name].invalidate()
-
     def client_plugin(self, name):
         global _mgr
         if name in self._client_plugins:
@@ -83,7 +77,7 @@ class OpenStackClients(object):
             client = getattr(self, method_name)()
             self._clients[name] = client
             return client
-        LOG.warning(_LW('Requested client "%s" not found'), name)
+        LOG.warning('Requested client "%s" not found', name)
 
 
 class ClientBackend(object):
