@@ -393,22 +393,15 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
             attachments.append((jsonutils.dumps(metadata),
                                 'cfn-init-data', 'x-cfninitdata'))
 
-        heat_client_plugin = self.context.clients.client_plugin('heat')
-        watch_url = cfg.CONF.heat_watch_server_url
-        if not watch_url:
-            watch_url = heat_client_plugin.get_watch_server_url()
-
-        attachments.append((watch_url, 'cfn-watch-server', 'x-cfninitdata'))
-
         if is_cfntools:
+            heat_client_plugin = self.context.clients.client_plugin('heat')
             cfn_md_url = heat_client_plugin.get_cfn_metadata_server_url()
             attachments.append((cfn_md_url,
                                 'cfn-metadata-server', 'x-cfninitdata'))
 
             # Create a boto config which the cfntools on the host use to know
-            # where the cfn and cw API's are to be accessed
+            # where the cfn API is to be accessed
             cfn_url = urlparse.urlparse(cfn_md_url)
-            cw_url = urlparse.urlparse(watch_url)
             is_secure = cfg.CONF.instance_connection_is_secure
             vcerts = cfg.CONF.instance_connection_https_validate_certificates
             boto_cfg = "\n".join(["[Boto]",
@@ -417,10 +410,7 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
                                   "https_validate_certificates = %s" % vcerts,
                                   "cfn_region_name = heat",
                                   "cfn_region_endpoint = %s" %
-                                  cfn_url.hostname,
-                                  "cloudwatch_region_name = heat",
-                                  "cloudwatch_region_endpoint = %s" %
-                                  cw_url.hostname])
+                                  cfn_url.hostname])
             attachments.append((boto_cfg,
                                 'cfn-boto-cfg', 'x-cfninitdata'))
 
@@ -444,9 +434,12 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
             return False
 
         status = self.get_status(server)
-        if status in ("DELETED", "SOFT_DELETED"):
+        if status == 'DELETED':
             return True
-        if status == 'ERROR':
+
+        if status == 'SOFT_DELETED':
+            self.client().servers.force_delete(server_id)
+        elif status == 'ERROR':
             fault = getattr(server, 'fault', {})
             message = fault.get('message', 'Unknown')
             code = fault.get('code')
@@ -702,7 +695,7 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
     @tenacity.retry(
         stop=tenacity.stop_after_attempt(
             cfg.CONF.max_interface_check_attempts),
-        wait=tenacity.wait_fixed(0.5),
+        wait=tenacity.wait_exponential(multiplier=0.5, max=12.0),
         retry=tenacity.retry_if_result(client_plugin.retry_if_result_is_false))
     def check_interface_detach(self, server_id, port_id):
         with self.ignore_not_found:
