@@ -204,15 +204,14 @@ class Schema(collections.Mapping):
 
         return value
 
-    def validate_constraints(self, value, context=None, skipped=None,
-                             template=None):
+    def validate_constraints(self, value, context=None, skipped=None):
         if not skipped:
             skipped = []
 
         try:
             for constraint in self.constraints:
                 if type(constraint) not in skipped:
-                    constraint.validate(value, self, context, template)
+                    constraint.validate(value, self, context)
         except ValueError as ex:
             raise exception.StackValidationFailed(message=six.text_type(ex))
 
@@ -296,8 +295,8 @@ class Constraint(collections.Mapping):
 
         return '\n'.join(desc())
 
-    def validate(self, value, schema=None, context=None, template=None):
-        if not self._is_valid(value, schema, context, template):
+    def validate(self, value, schema=None, context=None):
+        if not self._is_valid(value, schema, context):
             if self.description:
                 err_msg = self.description
             else:
@@ -374,7 +373,7 @@ class Range(Constraint):
                                                           self.min,
                                                           self.max)
 
-    def _is_valid(self, value, schema, context, template):
+    def _is_valid(self, value, schema, context):
         value = Schema.str_to_num(value)
 
         if self.min is not None:
@@ -437,9 +436,8 @@ class Length(Range):
                                                                    self.min,
                                                                    self.max)
 
-    def _is_valid(self, value, schema, context, template):
-        return super(Length, self)._is_valid(len(value), schema, context,
-                                             template)
+    def _is_valid(self, value, schema, context):
+        return super(Length, self)._is_valid(len(value), schema, context)
 
 
 class Modulo(Constraint):
@@ -503,7 +501,7 @@ class Modulo(Constraint):
         return '%s is not a multiple of %s with an offset of %s)' % (
             value, self.step, self.offset)
 
-    def _is_valid(self, value, schema, context, template):
+    def _is_valid(self, value, schema, context):
         value = Schema.str_to_num(value)
 
         if value % self.step != self.offset:
@@ -551,7 +549,7 @@ class AllowedValues(Constraint):
         allowed = '[%s]' % ', '.join(str(a) for a in self.allowed)
         return '"%s" is not an allowed value %s' % (value, allowed)
 
-    def _is_valid(self, value, schema, context, template):
+    def _is_valid(self, value, schema, context):
         # For list values, check if all elements of the list are contained
         # in allowed list.
         if isinstance(value, list):
@@ -594,7 +592,7 @@ class AllowedPattern(Constraint):
     def _err_msg(self, value):
         return '"%s" does not match pattern "%s"' % (value, self.pattern)
 
-    def _is_valid(self, value, schema, context, template):
+    def _is_valid(self, value, schema, context):
         match = self.match(value)
         return match is not None and match.end() == len(value)
 
@@ -645,19 +643,11 @@ class CustomConstraint(Constraint):
         return _('"%(value)s" does not validate %(name)s') % {
             "value": value, "name": self.name}
 
-    def _is_valid(self, value, schema, context, template):
+    def _is_valid(self, value, schema, context):
         constraint = self.custom_constraint
         if not constraint:
             return False
-
-        try:
-            result = constraint.validate(value, context,
-                                         template=template)
-        except TypeError:
-            # for backwards compatibility with older service constraints
-            result = constraint.validate(value, context)
-
-        return result
+        return constraint.validate(value, context)
 
 
 class BaseCustomConstraint(object):
@@ -679,7 +669,7 @@ class BaseCustomConstraint(object):
         return _("Error validating value '%(value)s': %(message)s") % {
             "value": value, "message": self._error_message}
 
-    def validate(self, value, context, template=None):
+    def validate(self, value, context):
 
         @MEMOIZE
         def check_cache_or_validate_value(cache_value_prefix,
