@@ -15,9 +15,10 @@ import collections
 import copy
 import functools
 import itertools
-
 import six
 
+from oslo_log import log as logging
+
 from heat.common import exception
 from heat.common import grouputils
 from heat.common.i18n import _
@@ -34,6 +35,8 @@ from heat.engine import support
 from heat.scaling import rolling_update
 from heat.scaling import template as scl_template
 
+LOG = logging.getLogger(__name__)
+
 
 class ResourceGroup(stack_resource.StackResource):
     """Creates one or more identically configured nested resources.
@@ -76,8 +79,10 @@ class ResourceGroup(stack_resource.StackResource):
 
     PROPERTIES = (
         COUNT, INDEX_VAR, RESOURCE_DEF, REMOVAL_POLICIES,
+        REMOVAL_POLICIES_MODE,
     ) = (
         'count', 'index_var', 'resource_def', 'removal_policies',
+        'removal_policies_mode'
     )
 
     _RESOURCE_DEF_KEYS = (
@@ -92,6 +97,12 @@ class ResourceGroup(stack_resource.StackResource):
         'resource_list',
     )
 
+    _REMOVAL_POLICY_MODES = (
+        REMOVAL_POLICY_APPEND, REMOVAL_POLICY_UPDATE
+    ) = (
+        'append', 'update'
+    )
+
     _ROLLING_UPDATES_SCHEMA_KEYS = (
         MIN_IN_SERVICE, MAX_BATCH_SIZE, PAUSE_TIME,
     ) = (
@@ -195,6 +206,18 @@ class ResourceGroup(stack_resource.StackResource):
             default=[],
             support_status=support.SupportStatus(version='2015.1')
         ),
+        REMOVAL_POLICIES_MODE: properties.Schema(
+            properties.Schema.STRING,
+            _('How to handle changes to removal_policies on update. '
+              'The default "append" mode appends to the internal list, '
+              '"update" replaces it on update.'),
+            default=REMOVAL_POLICY_APPEND,
+            constraints=[
+                constraints.AllowedValues(_REMOVAL_POLICY_MODES)
+            ],
+            update_allowed=True,
+            support_status=support.SupportStatus(version='10.0.0')
+        ),
     }
 
     attributes_schema = {
@@ -280,7 +303,7 @@ class ResourceGroup(stack_resource.StackResource):
         if not self.get_size():
             return
 
-        first_name = next(self._resource_names(update_rsrc_data=False))
+        first_name = next(self._resource_names())
         test_tmpl = self._assemble_nested([first_name],
                                           include_all=True)
         res_def = next(six.itervalues(test_tmpl.resource_definitions(None)))
@@ -307,38 +330,69 @@ class ResourceGroup(stack_resource.StackResource):
         else:
             return []
 
-    def _name_blacklist(self, update_rsrc_data=True):
-        """Resolve the remove_policies to names for removal."""
-
-        nested = self.nested()
-
-        # To avoid reusing names after removal, we store a comma-separated
-        # blacklist in the resource data
-        current_blacklist = self._current_blacklist()
+    def _get_new_blacklist_entries(self, properties, current_blacklist):
+        insp = grouputils.GroupInspector.from_parent_resource(self)
 
         # Now we iterate over the removal policies, and update the blacklist
         # with any additional names
-        rsrc_names = set(current_blacklist)
-
-        for r in self.properties[self.REMOVAL_POLICIES]:
+        for r in properties.get(self.REMOVAL_POLICIES, []):
             if self.REMOVAL_RSRC_LIST in r:
                 # Tolerate string or int list values
                 for n in r[self.REMOVAL_RSRC_LIST]:
                     str_n = six.text_type(n)
-                    if not nested or str_n in nested:
-                        rsrc_names.add(str_n)
-                        continue
-                    rsrc = nested.resource_by_refid(str_n)
-                    if rsrc:
-                        rsrc_names.add(rsrc.name)
+                    if (str_n in current_blacklist or
+                            self.resource_id is None or
+                            str_n in insp.member_names(include_failed=True)):
+                        yield str_n
+                    elif isinstance(n, six.string_types):
+                        try:
+                            refids = self.get_output(self.REFS_MAP)
+                        except (exception.NotFound,
+                                exception.TemplateOutputError) as op_err:
+                            LOG.debug('Falling back to resource_by_refid() '
+                                      ' due to %s', op_err)
+                            rsrc = self.nested().resource_by_refid(n)
+                            if rsrc is not None:
+                                yield rsrc.name
+                        else:
+                            if refids is not None:
+                                for name, refid in refids.items():
+                                    if refid == n:
+                                        yield name
+                                        break
+
+        # Clear output cache from prior to stack update, so we don't get
+        # outdated values after stack update.
+        self._outputs = None
+
+    def _update_name_blacklist(self, properties):
+        """Resolve the remove_policies to names for removal."""
+        # To avoid reusing names after removal, we store a comma-separated
+        # blacklist in the resource data - in cases where you want to
+        # overwrite the stored data, removal_policies_mode: update can be used
+        curr_bl = set(self._current_blacklist())
+        p_mode = properties.get(self.REMOVAL_POLICIES_MODE,
+                                self.REMOVAL_POLICY_APPEND)
+        if p_mode == self.REMOVAL_POLICY_UPDATE:
+            init_bl = set()
+        else:
+            init_bl = curr_bl
+        updated_bl = init_bl | set(self._get_new_blacklist_entries(properties,
+                                                                   curr_bl))
 
         # If the blacklist has changed, update the resource data
-        if update_rsrc_data and rsrc_names != set(current_blacklist):
-            self.data_set('name_blacklist', ','.join(rsrc_names))
-        return rsrc_names
+        if updated_bl != curr_bl:
+            self.data_set('name_blacklist', ','.join(sorted(updated_bl)))
 
-    def _resource_names(self, size=None, update_rsrc_data=True):
-        name_blacklist = self._name_blacklist(update_rsrc_data)
+    def _name_blacklist(self):
+        """Get the list of resource names to blacklist."""
+        bl = set(self._current_blacklist())
+        if self.resource_id is None:
+            bl |= set(self._get_new_blacklist_entries(self.properties, bl))
+        return bl
+
+    def _resource_names(self, size=None):
+        name_blacklist = self._name_blacklist()
         if size is None:
             size = self.get_size()
 
@@ -351,13 +405,13 @@ class ResourceGroup(stack_resource.StackResource):
                                                       candidates),
                                 size)
 
-    def _count_black_listed(self):
+    def _count_black_listed(self, existing_members):
         """Return the number of current resource names that are blacklisted."""
-        existing_members = grouputils.get_member_names(self)
         return len(self._name_blacklist() & set(existing_members))
 
     def handle_create(self):
-        if self.update_policy.get(self.BATCH_CREATE):
+        self._update_name_blacklist(self.properties)
+        if self.update_policy.get(self.BATCH_CREATE) and self.get_size():
             batch_create = self.update_policy[self.BATCH_CREATE]
             max_batch_size = batch_create[self.MAX_BATCH_SIZE]
             pause_sec = batch_create[self.PAUSE_TIME]
@@ -415,6 +469,7 @@ class ResourceGroup(stack_resource.StackResource):
         checkers = []
         self.properties = json_snippet.properties(self.properties_schema,
                                                   self.context)
+        self._update_name_blacklist(self.properties)
         if prop_diff and self.res_def_changed(prop_diff):
             updaters = self._try_rolling_update()
             if updaters:
@@ -430,7 +485,46 @@ class ResourceGroup(stack_resource.StackResource):
         checkers[0].start()
         return checkers
 
+    def _attribute_output_name(self, *attr_path):
+        if attr_path[0] == self.REFS:
+            return self.REFS
+        return ', '.join(six.text_type(a) for a in attr_path)
+
     def get_attribute(self, key, *path):
+        if key == self.REMOVED_RSRC_LIST:
+            return self._current_blacklist()
+        if key == self.ATTR_ATTRIBUTES and not path:
+            raise exception.InvalidTemplateAttribute(resource=self.name,
+                                                     key=key)
+
+        is_resource_ref = (key.startswith("resource.") and
+                           not path and (len(key.split('.', 2)) == 2))
+        if is_resource_ref:
+            output_name = self.REFS_MAP
+        else:
+            output_name = self._attribute_output_name(key, *path)
+
+        if self.resource_id is not None:
+            try:
+                output = self.get_output(output_name)
+            except (exception.NotFound,
+                    exception.TemplateOutputError) as op_err:
+                LOG.debug('Falling back to grouputils due to %s', op_err)
+            else:
+                if is_resource_ref:
+                    try:
+                        target = key.split('.', 2)[1]
+                        return output[target]
+                    except KeyError:
+                        raise exception.NotFound(_("Member '%(mem)s' not "
+                                                   "found in group resource "
+                                                   "'%(grp)s'.") %
+                                                 {'mem': target,
+                                                  'grp': self.name})
+                if key == self.REFS:
+                    return attributes.select_from_attribute(output, path)
+                return output
+
         if key.startswith("resource."):
             return grouputils.get_nested_attrs(self, key, False, *path)
 
@@ -442,12 +536,7 @@ class ResourceGroup(stack_resource.StackResource):
             refs_map = {n: grouputils.get_rsrc_id(self, key, False, n)
                         for n in names}
             return refs_map
-        if key == self.REMOVED_RSRC_LIST:
-            return self._current_blacklist()
         if key == self.ATTR_ATTRIBUTES:
-            if not path:
-                raise exception.InvalidTemplateAttribute(
-                    resource=self.name, key=key)
             return dict((n, grouputils.get_rsrc_attr(
                 self, key, False, n, *path)) for n in names)
 
@@ -455,31 +544,38 @@ class ResourceGroup(stack_resource.StackResource):
         return [grouputils.get_rsrc_attr(self, key, False, n, *path)
                 for n in names]
 
-    def _nested_output_defns(self, resource_names, get_attr_fn):
+    def _nested_output_defns(self, resource_names, get_attr_fn, get_res_fn):
         for attr in self.referenced_attrs():
             if isinstance(attr, six.string_types):
                 key, path = attr, []
-                output_name = attr
             else:
                 key, path = attr[0], list(attr[1:])
-                output_name = ', '.join(six.text_type(a) for a in attr)
+            output_name = self._attribute_output_name(key, *path)
+            value = None
 
             if key.startswith("resource."):
                 keycomponents = key.split('.', 2)
                 res_name = keycomponents[1]
-                attr_name = keycomponents[2:]
-                if attr_name and (res_name in resource_names):
-                    value = get_attr_fn([res_name] + attr_name + path)
-                    yield output.OutputDefinition(output_name, value)
-
+                attr_path = keycomponents[2:] + path
+                if attr_path:
+                    if res_name in resource_names:
+                        value = get_attr_fn([res_name] + attr_path)
+                else:
+                    output_name = key = self.REFS_MAP
             elif key == self.ATTR_ATTRIBUTES and path:
                 value = {r: get_attr_fn([r] + path) for r in resource_names}
-                yield output.OutputDefinition(output_name, value)
-
             elif key not in self.ATTRIBUTES:
                 value = [get_attr_fn([r, key] + path) for r in resource_names]
+
+            if key == self.REFS:
+                value = [get_res_fn(r) for r in resource_names]
+
+            if value is not None:
                 yield output.OutputDefinition(output_name, value)
 
+        value = {r: get_res_fn(r) for r in resource_names}
+        yield output.OutputDefinition(self.REFS_MAP, value)
+
     def build_resource_definition(self, res_name, res_defn):
         res_def = copy.deepcopy(res_defn)
 
@@ -551,6 +647,15 @@ class ResourceGroup(stack_resource.StackResource):
             return [recurse(v) for v in val]
         return val
 
+    def _add_output_defns_to_template(self, tmpl, resource_names):
+        att_func = 'get_attr'
+        get_attr = functools.partial(tmpl.functions[att_func], None, att_func)
+        res_func = 'get_resource'
+        get_res = functools.partial(tmpl.functions[res_func], None, res_func)
+        for odefn in self._nested_output_defns(resource_names,
+                                               get_attr, get_res):
+            tmpl.add_output(odefn)
+
     def _assemble_nested(self, names, include_all=False,
                          template_version=('heat_template_version',
                                            '2015-04-30')):
@@ -560,13 +665,7 @@ class ResourceGroup(stack_resource.StackResource):
                        for k in names]
         tmpl = scl_template.make_template(definitions,
                                           version=template_version)
-
-        att_func = 'get_attr'
-        get_attr = functools.partial(tmpl.functions[att_func], None, att_func)
-        for odefn in self._nested_output_defns([k for k, d in definitions],
-                                               get_attr):
-            tmpl.add_output(odefn)
-
+        self._add_output_defns_to_template(tmpl, [k for k, d in definitions])
         return tmpl
 
     def _assemble_for_rolling_update(self, total_capacity, max_updates,
@@ -608,8 +707,10 @@ class ResourceGroup(stack_resource.StackResource):
             max_updates,
             lambda: next(new_names),
             self.build_resource_definition)
-        return scl_template.make_template(definitions,
+        tmpl = scl_template.make_template(definitions,
                                           version=template_version)
+        self._add_output_defns_to_template(tmpl, names)
+        return tmpl
 
     def _try_rolling_update(self):
         if self.update_policy[self.ROLLING_UPDATE]:
@@ -653,11 +754,12 @@ class ResourceGroup(stack_resource.StackResource):
             while not duration.expired():
                 yield
 
-        # blacklist count existing
-        num_blacklist = self._count_black_listed()
-
         # current capacity not including existing blacklisted
-        curr_cap = len(self.nested()) - num_blacklist if self.nested() else 0
+        inspector = grouputils.GroupInspector.from_parent_resource(self)
+        num_blacklist = self._count_black_listed(
+            inspector.member_names(include_failed=False))
+        num_resources = inspector.size(include_failed=True)
+        curr_cap = num_resources - num_blacklist
 
         batches = list(self._get_batches(self.get_size(), curr_cap, batch_size,
                                          min_in_service))
