@@ -701,6 +701,9 @@ class SoftwareDeploymentGroup(resource_group.ResourceGroup):
     def res_def_changed(self, prop_diff):
         return True
 
+    def _update_name_blacklist(self, properties):
+        pass
+
     def _name_blacklist(self):
         return set()
 
@@ -735,11 +738,12 @@ class SoftwareDeploymentGroup(resource_group.ResourceGroup):
         rg_attr = rg.get_attribute(rg.ATTR_ATTRIBUTES, n_attr)
         return attributes.select_from_attribute(rg_attr, path)
 
-    def _nested_output_defns(self, resource_names, get_attr_fn):
+    def _nested_output_defns(self, resource_names, get_attr_fn, get_res_fn):
         for attr in self.referenced_attrs():
             key = attr if isinstance(attr, six.string_types) else attr[0]
             n_attr = self._member_attribute_name(key)
-            output_name = '%s, %s' % (self.ATTR_ATTRIBUTES, n_attr)
+            output_name = self._attribute_output_name(self.ATTR_ATTRIBUTES,
+                                                      n_attr)
             value = {r: get_attr_fn([r, n_attr])
                      for r in resource_names}
             yield output.OutputDefinition(output_name, value)
