@@ -45,12 +45,12 @@ class TestResource(resource.Resource):
         VALUE, UPDATE_REPLACE, FAIL,
         CLIENT_NAME, ENTITY_NAME,
         WAIT_SECS, ACTION_WAIT_SECS, ATTR_WAIT_SECS,
-        CONSTRAINT_PROP_SECS,
+        CONSTRAINT_PROP_SECS, UPDATE_REPLACE_VALUE,
     ) = (
         'value', 'update_replace', 'fail',
         'client_name', 'entity_name',
         'wait_secs', 'action_wait_secs', 'attr_wait_secs',
-        'constraint_prop_secs',
+        'constraint_prop_secs', 'update_replace_value',
     )
 
     ATTRIBUTES = (
@@ -83,6 +83,11 @@ class TestResource(resource.Resource):
             default='test_string',
             update_allowed=True
         ),
+        UPDATE_REPLACE_VALUE: properties.Schema(
+            properties.Schema.STRING,
+            _('Some value that can be stored but can not be updated.'),
+            support_status=support.SupportStatus(version='7.0.0')
+        ),
         FAIL: properties.Schema(
             properties.Schema.BOOLEAN,
             _('Value which can be set to fail the resource operation '
@@ -156,8 +161,10 @@ class TestResource(resource.Resource):
             secs = self.properties[self.ACTION_WAIT_SECS][self.action.lower()]
         if secs is None:
             secs = self.properties[self.WAIT_SECS]
-        LOG.info('%s wait_secs:%s, action:%s' % (self.name, secs,
-                                                 self.action.lower()))
+        LOG.info('%(name)s wait_secs:%(wait)s, action:%(action)s',
+                 {'name': self.name,
+                  'wait': secs,
+                  'action': self.action.lower()})
         return secs
 
     def handle_create(self):
@@ -221,8 +228,10 @@ class TestResource(resource.Resource):
 
         started_at = timeutils.normalize_time(started_at)
         waited = timeutils.utcnow() - started_at
-        LOG.info("Resource %s waited %s/%s seconds",
-                 self.name, waited, wait_secs)
+        LOG.info("Resource %(name)s waited %(waited)s/%(sec)s seconds",
+                 {'name': self.name,
+                  'waited': waited,
+                  'sec': wait_secs})
 
         # wait_secs < 0 is an infinite wait time.
         if wait_secs >= 0 and waited > datetime.timedelta(seconds=wait_secs):
