@@ -11,10 +11,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import uuid
+
 from oslo_serialization import jsonutils
 
 from heat.common.i18n import _
-from heat.common import password_gen
 from heat.engine import attributes
 from heat.engine import constraints
 from heat.engine import properties
@@ -134,7 +135,7 @@ class HeatWaitConditionHandle(wc_base.BaseWaitConditionHandle):
             self.SIGNAL_TRANSPORT) == self.TOKEN_SIGNAL
 
     def handle_create(self):
-        self.password = password_gen.generate_openstack_password()
+        self.password = uuid.uuid4().hex
         super(HeatWaitConditionHandle, self).handle_create()
         if self._signal_transport_token():
             # FIXME(shardy): The assumption here is that token expiry > timeout
@@ -221,8 +222,8 @@ class HeatWaitConditionHandle(wc_base.BaseWaitConditionHandle):
 class UpdateWaitConditionHandle(aws_wch.WaitConditionHandle):
     """WaitConditionHandle that clears signals and changes handle on update.
 
-    This works identically to a regular WaitConditionHandle, except that
-    on update it clears all signals received and changes the handle. Using
+    This works similarly to an AWS::CloudFormation::WaitConditionHandle, except
+    that on update it clears all signals received and changes the handle. Using
     this handle means that you must setup the signal senders to send their
     signals again any time the update handle changes. This allows us to roll
     out new configurations and be confident that they are rolled out once
