@@ -65,10 +65,8 @@ class ExtraRoute(neutron.NeutronResource):
             # depend on any RouterInterface in this template with the same
             # router_id as this router_id
             if resource.has_interface('OS::Neutron::RouterInterface'):
-                dep_router_id = self.client_plugin().resolve_router({
-                    'router': resource.properties.get('router'),
-                    'router_id': None}, 'router', 'router_id')
                 router_id = self.properties[self.ROUTER_ID]
+                dep_router_id = resource.properties['router']
                 if dep_router_id == router_id:
                     deps += (self, resource)
             # depend on any RouterGateway in this template with the same
