@@ -34,13 +34,16 @@ class FloatingIP(neutron.NeutronResource):
     user to have a "static" IP address that can be reassigned when an instance
     is upgraded or moved.
     """
+
+    entity = 'floatingip'
+
     PROPERTIES = (
-        FLOATING_NETWORK_ID, FLOATING_NETWORK, VALUE_SPECS,
-        PORT_ID, FIXED_IP_ADDRESS, FLOATING_IP_ADDRESS,
+        FLOATING_NETWORK_ID, FLOATING_NETWORK,  FLOATING_SUBNET,
+        VALUE_SPECS, PORT_ID, FIXED_IP_ADDRESS, FLOATING_IP_ADDRESS,
         DNS_NAME, DNS_DOMAIN,
     ) = (
-        'floating_network_id', 'floating_network', 'value_specs',
-        'port_id', 'fixed_ip_address', 'floating_ip_address',
+        'floating_network_id', 'floating_network', 'floating_subnet',
+        'value_specs', 'port_id', 'fixed_ip_address', 'floating_ip_address',
         'dns_name', 'dns_domain',
     )
 
@@ -77,6 +80,14 @@ class FloatingIP(neutron.NeutronResource):
                 constraints.CustomConstraint('neutron.network')
             ],
         ),
+        FLOATING_SUBNET: properties.Schema(
+            properties.Schema.STRING,
+            _('Subnet to allocate floating IP from.'),
+            support_status=support.SupportStatus(version='9.0.0'),
+            constraints=[
+                constraints.CustomConstraint('neutron.subnet')
+            ],
+        ),
         VALUE_SPECS: properties.Schema(
             properties.Schema.MAP,
             _('Extra parameters to include in the "floatingip" object in the '
@@ -176,9 +187,56 @@ class FloatingIP(neutron.NeutronResource):
                 client_plugin=self.client_plugin(),
                 finder='find_resourceid_by_name_or_id',
                 entity='network'
+            ),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.FLOATING_SUBNET],
+                client_plugin=self.client_plugin(),
+                finder='find_resourceid_by_name_or_id',
+                entity='subnet',
             )
         ]
 
+    def _add_router_interface_dependencies(self, deps, resource):
+        def port_on_subnet(resource, subnet):
+            if not resource.has_interface('OS::Neutron::Port'):
+                return False
+
+            fixed_ips = resource.properties.get(port.Port.FIXED_IPS)
+            if not fixed_ips:
+                # During create we have only unresolved value for
+                # functions, so can not use None value for building
+                # correct dependencies. Depend on all RouterInterfaces
+                # when the port has no fixed IP specified, since we
+                # can't safely assume that any are in different
+                # networks.
+                if subnet is None:
+                    return True
+
+                p_net = (resource.properties.get(port.Port.NETWORK) or
+                         resource.properties.get(port.Port.NETWORK_ID))
+                if p_net:
+                    network = self.client().show_network(p_net)['network']
+                    return subnet in network['subnets']
+            else:
+                for fixed_ip in resource.properties.get(
+                        port.Port.FIXED_IPS):
+
+                    port_subnet = (fixed_ip.get(port.Port.FIXED_IP_SUBNET) or
+                                   fixed_ip.get(port.Port.FIXED_IP_SUBNET_ID))
+                    if subnet == port_subnet:
+                        return True
+            return False
+
+        interface_subnet = (
+            resource.properties.get(router.RouterInterface.SUBNET) or
+            resource.properties.get(router.RouterInterface.SUBNET_ID))
+        for d in deps.graph()[self]:
+            if port_on_subnet(d, interface_subnet):
+                deps += (self, resource)
+                break
+
     def add_dependencies(self, deps):
         super(FloatingIP, self).add_dependencies(deps)
 
@@ -197,46 +255,7 @@ class FloatingIP(neutron.NeutronResource):
             # with the same subnet that this floating IP's port is assigned
             # to
             elif resource.has_interface('OS::Neutron::RouterInterface'):
-
-                def port_on_subnet(resource, subnet):
-                    if not resource.has_interface('OS::Neutron::Port'):
-                        return False
-
-                    fixed_ips = resource.properties.get(port.Port.FIXED_IPS)
-                    if not fixed_ips:
-                        # During create we have only unresolved value for
-                        # functions, so can not use None value for building
-                        # correct dependencies. Depend on all RouterInterfaces
-                        # when the port has no fixed IP specified, since we
-                        # can't safely assume that any are in different
-                        # networks.
-                        if subnet is None:
-                            return True
-
-                        p_net = (resource.properties.get(port.Port.NETWORK) or
-                                 resource.properties.get(port.Port.NETWORK_ID))
-                        if p_net:
-                            subnets = self.client().show_network(p_net)[
-                                'network']['subnets']
-                            return subnet in subnets
-                    else:
-                        for fixed_ip in resource.properties.get(
-                                port.Port.FIXED_IPS):
-
-                            port_subnet = (
-                                fixed_ip.get(port.Port.FIXED_IP_SUBNET)
-                                or fixed_ip.get(port.Port.FIXED_IP_SUBNET_ID))
-                            if subnet == port_subnet:
-                                return True
-                    return False
-
-                interface_subnet = (
-                    resource.properties.get(router.RouterInterface.SUBNET) or
-                    resource.properties.get(router.RouterInterface.SUBNET_ID))
-                for d in deps.graph()[self]:
-                    if port_on_subnet(d, interface_subnet):
-                        deps += (self, resource)
-                        break
+                self._add_router_interface_dependencies(deps, resource)
             # depend on Router with EXTERNAL_GATEWAY_NETWORK property
             # this template with the same network_id as this
             # floating_network_id
@@ -263,13 +282,12 @@ class FloatingIP(neutron.NeutronResource):
             self.properties,
             self.physical_resource_name())
         props['floating_network_id'] = props.pop(self.FLOATING_NETWORK)
+        if self.FLOATING_SUBNET in props:
+            props['subnet_id'] = props.pop(self.FLOATING_SUBNET)
         fip = self.client().create_floatingip({
             'floatingip': props})['floatingip']
         self.resource_id_set(fip['id'])
 
-    def _show_resource(self):
-        return self.client().show_floatingip(self.resource_id)['floatingip']
-
     def handle_delete(self):
         with self.client_plugin().ignore_not_found:
             self.client().delete_floatingip(self.resource_id)
