@@ -33,6 +33,10 @@ class HealthMonitor(neutron.NeutronResource):
 
     required_service_extension = 'lbaasv2'
 
+    entity = 'lbaas_healthmonitor'
+
+    res_info_key = 'healthmonitor'
+
     # Properties inputs for the resources create/update.
     PROPERTIES = (
         ADMIN_STATE_UP, DELAY, EXPECTED_CODES, HTTP_METHOD,
@@ -148,8 +152,7 @@ class HealthMonitor(neutron.NeutronResource):
         if self._lb_id is None:
             pool_id = self.client_plugin().find_resourceid_by_name_or_id(
                 self.POOL,
-                self.properties[self.POOL],
-                cmd_resource='lbaas_pool')
+                self.properties[self.POOL])
             pool = self.client().show_lbaas_pool(pool_id)['pool']
 
             listener_id = pool['listeners'][0]['id']
@@ -184,10 +187,6 @@ class HealthMonitor(neutron.NeutronResource):
 
         return self._check_lb_status()
 
-    def _show_resource(self):
-        return self.client().show_lbaas_healthmonitor(
-            self.resource_id)['healthmonitor']
-
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         self._update_called = False
         return prop_diff
