@@ -21,6 +21,7 @@ from heat.engine import constraints
 from heat.engine import properties
 from heat.engine.resources.openstack.neutron import neutron
 from heat.engine import support
+from heat.engine import translation
 
 
 class Pool(neutron.NeutronResource):
@@ -35,14 +36,18 @@ class Pool(neutron.NeutronResource):
 
     required_service_extension = 'lbaasv2'
 
+    entity = 'lbaas_pool'
+
+    res_info_key = 'pool'
+
     PROPERTIES = (
         ADMIN_STATE_UP, DESCRIPTION, SESSION_PERSISTENCE, NAME,
-        LB_ALGORITHM, LISTENER, PROTOCOL, SESSION_PERSISTENCE_TYPE,
-        SESSION_PERSISTENCE_COOKIE_NAME,
+        LB_ALGORITHM, LISTENER, LOADBALANCER, PROTOCOL,
+        SESSION_PERSISTENCE_TYPE, SESSION_PERSISTENCE_COOKIE_NAME,
     ) = (
         'admin_state_up', 'description', 'session_persistence', 'name',
-        'lb_algorithm', 'listener', 'protocol', 'type',
-        'cookie_name'
+        'lb_algorithm', 'listener', 'loadbalancer', 'protocol',
+        'type', 'cookie_name'
     )
 
     SESSION_PERSISTENCE_TYPES = (
@@ -108,12 +113,20 @@ class Pool(neutron.NeutronResource):
         ),
         LISTENER: properties.Schema(
             properties.Schema.STRING,
-            _('Listner name or ID to be associated with this pool.'),
-            required=True,
+            _('Listener name or ID to be associated with this pool.'),
             constraints=[
                 constraints.CustomConstraint('neutron.lbaas.listener')
             ]
         ),
+        LOADBALANCER: properties.Schema(
+            properties.Schema.STRING,
+            _('Loadbalancer name or ID to be associated with this pool. '
+              'Requires shared_pools service extension.'),
+            constraints=[
+                constraints.CustomConstraint('neutron.lbaas.loadbalancer')
+            ],
+            support_status=support.SupportStatus(version='9.0.0')
+        ),
         PROTOCOL: properties.Schema(
             properties.Schema.STRING,
             _('Protocol of the pool.'),
@@ -135,21 +148,44 @@ class Pool(neutron.NeutronResource):
         ),
         MEMBERS_ATTR: attributes.Schema(
             _('Members associated with this pool.'),
+            cache_mode=attributes.Schema.CACHE_NONE,
             type=attributes.Schema.LIST
         ),
     }
 
+    def translation_rules(self, props):
+        return [
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.LISTENER],
+                client_plugin=self.client_plugin(),
+                finder='find_resourceid_by_name_or_id',
+                entity='listener'
+            ),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.LOADBALANCER],
+                client_plugin=self.client_plugin(),
+                finder='find_resourceid_by_name_or_id',
+                entity='loadbalancer'
+            ),
+        ]
+
     def __init__(self, name, definition, stack):
         super(Pool, self).__init__(name, definition, stack)
         self._lb_id = None
 
     @property
     def lb_id(self):
-        if self._lb_id is None:
-            listener_id = self.client_plugin().find_resourceid_by_name_or_id(
-                'listener', self.properties[self.LISTENER])
-            listener = self.client().show_listener(listener_id)['listener']
+        if self._lb_id:
+            return self._lb_id
 
+        self._lb_id = self.properties[self.LOADBALANCER]
+        if self._lb_id is None:
+            listener_id = self.properties[self.LISTENER]
+            listener = self.client().show_listener(listener_id)['listener']
             self._lb_id = listener['loadbalancers'][0]['id']
         return self._lb_id
 
@@ -158,6 +194,11 @@ class Pool(neutron.NeutronResource):
         if res:
             return res
 
+        if (self.properties[self.LISTENER] is None and
+                self.properties[self.LOADBALANCER] is None):
+                raise exception.PropertyUnspecifiedError(self.LISTENER,
+                                                         self.LOADBALANCER)
+
         if self.properties[self.SESSION_PERSISTENCE] is not None:
             session_p = self.properties[self.SESSION_PERSISTENCE]
             persistence_type = session_p[self.SESSION_PERSISTENCE_TYPE]
@@ -186,8 +227,10 @@ class Pool(neutron.NeutronResource):
             self.properties,
             self.physical_resource_name())
 
-        self.client_plugin().resolve_listener(
-            properties, self.LISTENER, 'listener_id')
+        if self.LISTENER in properties:
+            properties['listener_id'] = properties.pop(self.LISTENER)
+        if self.LOADBALANCER in properties:
+            properties['loadbalancer_id'] = properties.pop(self.LOADBALANCER)
 
         session_p = properties.get(self.SESSION_PERSISTENCE)
         if session_p is not None:
@@ -209,9 +252,6 @@ class Pool(neutron.NeutronResource):
 
         return self._check_lb_status()
 
-    def _show_resource(self):
-        return self.client().show_lbaas_pool(self.resource_id)['pool']
-
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         self._update_called = False
         return prop_diff
