@@ -15,7 +15,6 @@ from oslo_log import log as logging
 
 from heat.common import exception
 from heat.common.i18n import _
-from heat.common.i18n import _LW
 from heat.engine import resource
 
 LOG = logging.getLogger(__name__)
@@ -27,6 +26,18 @@ class NeutronResource(resource.Resource):
 
     res_info_key = None
 
+    def get_resource_plural(self):
+        """Return the plural of resource type.
+
+        The default implementation is to return self.entity + 's',
+        the rule is not appropriate for some special resources,
+        e.g. qos_policy, this method should be overridden by the
+        special resources if needed.
+        """
+        if not self.entity:
+            return
+        return self.entity + 's'
+
     def validate(self):
         """Validate any of the provided params."""
         res = super(NeutronResource, self).validate()
@@ -70,6 +81,16 @@ class NeutronResource(resource.Resource):
             NeutronResource.merge_value_specs(props)
         return props
 
+    def _store_config_default_properties(self, attrs):
+        """A method for storing properties, which defaults stored in config.
+
+        A method allows to store properties default values, which cannot be
+        defined in schema in case of specifying in config file.
+        """
+        if 'port_security_enabled' in attrs:
+            self.data_set('port_security_enabled',
+                          attrs['port_security_enabled'])
+
     @staticmethod
     def merge_value_specs(props):
         value_spec_props = props.pop('value_specs')
@@ -93,7 +114,7 @@ class NeutronResource(resource.Resource):
             return False
         if status in ('ACTIVE', 'DOWN'):
             return True
-        elif status == 'ERROR':
+        elif status in ('ERROR', 'DEGRADED'):
             raise exception.ResourceInError(
                 resource_status=status)
         else:
@@ -113,8 +134,7 @@ class NeutronResource(resource.Resource):
             key = self.res_info_key if self.res_info_key else self.entity
             return res_info[key]
         except AttributeError as ex:
-            LOG.warning(_LW("Resolving 'show' attribute has failed : %s"),
-                        ex)
+            LOG.warning("Resolving 'show' attribute has failed : %s", ex)
 
     def _resolve_attribute(self, name):
         if self.resource_id is None:
@@ -156,3 +176,33 @@ class NeutronResource(resource.Resource):
             raise exception.PhysicalResourceExists(
                 name=self.physical_resource_name_or_FnGetRefId())
         return True
+
+    def set_tags(self, tags):
+        resource_plural = self.get_resource_plural()
+        if resource_plural:
+            tags = tags or []
+            body = {'tags': tags}
+            self.client().replace_tag(resource_plural,
+                                      self.resource_id,
+                                      body)
+
+    def parse_live_resource_data(self, resource_properties, resource_data):
+        result = super(NeutronResource, self).parse_live_resource_data(
+            resource_properties, resource_data)
+
+        if 'value_specs' in self.properties.keys():
+            result.update({self.VALUE_SPECS: {}})
+            for key in self.properties.get(self.VALUE_SPECS):
+                if key in resource_data:
+                    result[self.VALUE_SPECS][key] = resource_data.get(key)
+
+        # We already get real `port_security_enabled` from
+        # super().parse_live_resource_data above, so just check and remove
+        # if that's same value as old port value.
+        if 'port_security_enabled' in self.properties.keys():
+            old_port = bool(self.data().get(self.PORT_SECURITY_ENABLED))
+            new_port = resource_data.get(self.PORT_SECURITY_ENABLED)
+            if old_port == new_port:
+                del result[self.PORT_SECURITY_ENABLED]
+
+        return result
