@@ -25,6 +25,9 @@ class ProviderNet(net.Net):
 
     Provider networks specify details of physical realisation of the existing
     network.
+
+    The default policy usage of this resource is limited to
+    administrators only.
     """
 
     required_service_extension = 'provider'
@@ -85,7 +88,6 @@ class ProviderNet(net.Net):
               'provides the default value for the attribute of the ports '
               'created on this network.'),
             update_allowed=True,
-            default=True,
             support_status=support.SupportStatus(version='8.0.0')
         ),
         ROUTER_EXTERNAL: properties.Schema(
@@ -168,6 +170,24 @@ class ProviderNet(net.Net):
             self.client().update_network(self.resource_id,
                                          {'network': prop_diff})
 
+    def parse_live_resource_data(self, resource_properties, resource_data):
+        # this resource should not have super in case of we don't need to
+        # parse Net resource properties.
+        result = {}
+        provider_keys = [self.PROVIDER_NETWORK_TYPE,
+                         self.PROVIDER_PHYSICAL_NETWORK,
+                         self.PROVIDER_SEGMENTATION_ID]
+        for key in provider_keys:
+            result[key] = resource_data.get('provider:%s' % key)
+        result[self.ROUTER_EXTERNAL] = resource_data.get('router:external')
+        provider_keys.append(self.ROUTER_EXTERNAL)
+
+        provider_keys.append(self.SHARED)
+        for key in set(self.PROPERTIES) - set(provider_keys):
+            if key in resource_data:
+                result[key] = resource_data.get(key)
+        return result
+
 
 def resource_mapping():
     return {
