@@ -32,16 +32,18 @@ class Subnet(neutron.NeutronResource):
     CIDR, or from "allocation pools" that can be specified by the user.
     """
 
+    entity = 'subnet'
+
     PROPERTIES = (
         NETWORK_ID, NETWORK, SUBNETPOOL, PREFIXLEN, CIDR,
         VALUE_SPECS, NAME, IP_VERSION, DNS_NAMESERVERS, GATEWAY_IP,
         ENABLE_DHCP, ALLOCATION_POOLS, TENANT_ID, HOST_ROUTES,
-        IPV6_RA_MODE, IPV6_ADDRESS_MODE,
+        IPV6_RA_MODE, IPV6_ADDRESS_MODE, SEGMENT, TAGS,
     ) = (
         'network_id', 'network', 'subnetpool', 'prefixlen', 'cidr',
         'value_specs', 'name', 'ip_version', 'dns_nameservers', 'gateway_ip',
         'enable_dhcp', 'allocation_pools', 'tenant_id', 'host_routes',
-        'ipv6_ra_mode', 'ipv6_address_mode',
+        'ipv6_ra_mode', 'ipv6_address_mode', 'segment', 'tags',
     )
 
     _ALLOCATION_POOL_KEYS = (
@@ -234,6 +236,21 @@ class Subnet(neutron.NeutronResource):
             ],
             support_status=support.SupportStatus(version='2015.1')
         ),
+        SEGMENT: properties.Schema(
+            properties.Schema.STRING,
+            _('The name/ID of the segment to associate.'),
+            constraints=[
+                constraints.CustomConstraint('neutron.segment')
+            ],
+            support_status=support.SupportStatus(version='9.0.0')
+        ),
+        TAGS: properties.Schema(
+            properties.Schema.LIST,
+            _('The tags to be added to the subnet.'),
+            schema=properties.Schema(properties.Schema.STRING),
+            update_allowed=True,
+            support_status=support.SupportStatus(version='9.0.0')
+        ),
     }
 
     attributes_schema = {
@@ -301,6 +318,13 @@ class Subnet(neutron.NeutronResource):
                 client_plugin=self.client_plugin(),
                 finder='find_resourceid_by_name_or_id',
                 entity='subnetpool'
+            ),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.SEGMENT],
+                client_plugin=self.client_plugin('openstack'),
+                finder='find_network_segment'
             )
         ]
 
@@ -355,12 +379,21 @@ class Subnet(neutron.NeutronResource):
             self.properties,
             self.physical_resource_name())
         props['network_id'] = props.pop(self.NETWORK)
+        if self.SEGMENT in props and props[self.SEGMENT]:
+            props['segment_id'] = props.pop(self.SEGMENT)
+
+        tags = props.pop(self.TAGS, [])
+
         if self.SUBNETPOOL in props and props[self.SUBNETPOOL]:
             props['subnetpool_id'] = props.pop('subnetpool')
         self._null_gateway_ip(props)
+
         subnet = self.client().create_subnet({'subnet': props})['subnet']
         self.resource_id_set(subnet['id'])
 
+        if tags:
+            self.set_tags(tags)
+
     def handle_delete(self):
         try:
             self.client().delete_subnet(self.resource_id)
@@ -369,9 +402,6 @@ class Subnet(neutron.NeutronResource):
         else:
             return True
 
-    def _show_resource(self):
-        return self.client().show_subnet(self.resource_id)['subnet']
-
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         if prop_diff:
             self.prepare_update_properties(prop_diff)
@@ -381,7 +411,9 @@ class Subnet(neutron.NeutronResource):
 
             # If the new value is '', set to None
             self._null_gateway_ip(prop_diff)
-
+            if self.TAGS in prop_diff:
+                tags = prop_diff.pop(self.TAGS)
+                self.set_tags(tags)
             self.client().update_subnet(
                 self.resource_id, {'subnet': prop_diff})
 
