@@ -121,11 +121,20 @@ class HostAggregate(resource.Resource):
         if self.resource_id is None:
             return
 
-        aggregate = self.client().aggregates.get(self.resource_id)
-        for host in aggregate.hosts:
-            aggregate.remove_host(host)
+        with self.client_plugin().ignore_not_found:
+            aggregate = self.client().aggregates.get(self.resource_id)
+            for host in aggregate.hosts:
+                aggregate.remove_host(host)
         super(HostAggregate, self).handle_delete()
 
+    def parse_live_resource_data(self, resource_properties, resource_data):
+        aggregate_reality = {}
+
+        for key in self.PROPERTIES:
+            aggregate_reality.update({key: resource_data.get(key)})
+
+        return aggregate_reality
+
 
 def resource_mapping():
     return {
