@@ -21,7 +21,6 @@ import tenacity
 
 from heat.common import exception
 from heat.common.i18n import _
-from heat.common.i18n import _LI
 from heat.engine import resource
 from heat.engine.resources.openstack.neutron import port as neutron_port
 
@@ -36,20 +35,27 @@ class ServerNetworkMixin(object):
         subnet = network.get(self.NETWORK_SUBNET)
         fixed_ip = network.get(self.NETWORK_FIXED_IP)
         floating_ip = network.get(self.NETWORK_FLOATING_IP)
+        str_network = network.get(self.ALLOCATE_NETWORK)
 
-        if net_id is None and port is None and subnet is None:
-            msg = _('One of the properties "%(id)s", "%(port_id)s" '
-                    'or "%(subnet)s" should be set for the '
+        if (net_id is None and
+                port is None and
+                subnet is None and
+                not str_network):
+            msg = _('One of the properties "%(id)s", "%(port_id)s", '
+                    '"%(str_network)s" or "%(subnet)s" should be set for the '
                     'specified network of server "%(server)s".'
                     '') % dict(id=self.NETWORK_ID,
                                port_id=self.NETWORK_PORT,
                                subnet=self.NETWORK_SUBNET,
+                               str_network=self.ALLOCATE_NETWORK,
                                server=self.name)
             raise exception.StackValidationFailed(message=msg)
-
-        if port is not None and not self.is_using_neutron():
-            msg = _('Property "%s" is supported only for '
-                    'Neutron.') % self.NETWORK_PORT
+        # can not specify str_network with other keys of networks
+        # at the same time
+        has_value_keys = [k for k, v in network.items() if v is not None]
+        if str_network and len(has_value_keys) != 1:
+            msg = _('Can not specify "%s" with other keys of networks '
+                    'at the same time.') % self.ALLOCATE_NETWORK
             raise exception.StackValidationFailed(message=msg)
 
         # Nova doesn't allow specify ip and port at the same time
@@ -61,7 +67,7 @@ class ServerNetworkMixin(object):
         # if user only specifies network and floating ip, floating ip
         # can't be associated as the the neutron port isn't created/managed
         # by heat
-        if floating_ip is not None and self.is_using_neutron():
+        if floating_ip is not None:
             if net_id is not None and port is None and subnet is None:
                 msg = _('Property "%(fip)s" is not supported if only '
                         '"%(net)s" is specified, because the corresponding '
@@ -71,7 +77,7 @@ class ServerNetworkMixin(object):
                 raise exception.StackValidationFailed(message=msg)
 
     def _validate_belonging_subnet_to_net(self, network):
-        if network.get(self.NETWORK_PORT) is None and self.is_using_neutron():
+        if network.get(self.NETWORK_PORT) is None:
             net = self._get_network_id(network)
             # check if there are subnet and network both specified that
             # subnet belongs to specified network
@@ -184,9 +190,6 @@ class ServerNetworkMixin(object):
         creating. We need to store information about that ports, so store
         their IDs to data with key `external_ports`.
         """
-        if not self.is_using_neutron():
-            return
-
         # check if os-attach-interfaces extension is available on this cloud.
         # If it's not, then novaclient's interface_list method cannot be used
         # to get the list of interfaces.
@@ -215,6 +218,11 @@ class ServerNetworkMixin(object):
     def _build_nics(self, networks, security_groups=None):
         if not networks:
             return None
+
+        str_network = self._str_network(networks)
+        if str_network:
+            return str_network
+
         nics = []
 
         for idx, net in enumerate(networks):
@@ -222,7 +230,7 @@ class ServerNetworkMixin(object):
             nic_info = {'net-id': self._get_network_id(net)}
             if net.get(self.NETWORK_PORT):
                 nic_info['port-id'] = net[self.NETWORK_PORT]
-            elif self.is_using_neutron() and net.get(self.NETWORK_SUBNET):
+            elif net.get(self.NETWORK_SUBNET):
                 nic_info['port-id'] = self._create_internal_port(
                     net, idx, security_groups)
 
@@ -244,48 +252,67 @@ class ServerNetworkMixin(object):
                 self._floating_ip_neutron_associate(
                     net.get(self.NETWORK_FLOATING_IP), floating_ip_data)
 
+            if net.get(self.NIC_TAG):
+                nic_info[self.NIC_TAG] = net.get(self.NIC_TAG)
+
             nics.append(nic_info)
         return nics
 
     def _floating_ip_neutron_associate(self, floating_ip, floating_ip_data):
-        if self.is_using_neutron():
-            self.client('neutron').update_floatingip(
-                floating_ip, {'floatingip': floating_ip_data})
-
-    def _floating_ip_nova_associate(self, floating_ip):
-        fl_ip = self.client().floating_ips.get(floating_ip)
-        if fl_ip and self.resource_id:
-            self.client().servers.add_floating_ip(self.resource_id, fl_ip.ip)
-
-    def _floating_ips_disassociate(self):
-        networks = self.properties[self.NETWORKS] or []
-        for network in networks:
-            floating_ip = network.get(self.NETWORK_FLOATING_IP)
-            if floating_ip is not None:
-                self._floating_ip_disassociate(floating_ip)
+        self.client('neutron').update_floatingip(
+            floating_ip, {'floatingip': floating_ip_data})
 
     def _floating_ip_disassociate(self, floating_ip):
-        if self.is_using_neutron():
-            with self.client_plugin('neutron').ignore_not_found:
-                self.client('neutron').update_floatingip(
-                    floating_ip, {'floatingip': {'port_id': None}})
-        else:
-            with self.client_plugin().ignore_conflict_and_not_found:
-                fl_ip = self.client().floating_ips.get(floating_ip)
-                self.client().servers.remove_floating_ip(self.resource_id,
-                                                         fl_ip.ip)
+        with self.client_plugin('neutron').ignore_not_found:
+            self.client('neutron').update_floatingip(
+                floating_ip, {'floatingip': {'port_id': None}})
 
-    def _exclude_not_updated_networks(self, old_nets, new_nets):
-        # make networks similar by adding None vlues for not used keys
+    def _find_best_match(self, existing_interfaces, specified_net):
+        specified_net_items = set(specified_net.items())
+        if specified_net.get(self.NETWORK_PORT) is not None:
+            for iface in existing_interfaces:
+                if (iface[self.NETWORK_PORT] ==
+                        specified_net[self.NETWORK_PORT] and
+                        specified_net_items.issubset(set(iface.items()))):
+                    return iface
+        elif specified_net.get(self.NETWORK_FIXED_IP) is not None:
+            for iface in existing_interfaces:
+                if (iface[self.NETWORK_FIXED_IP] ==
+                        specified_net[self.NETWORK_FIXED_IP] and
+                        specified_net_items.issubset(set(iface.items()))):
+                    return iface
+        else:
+            # Best subset intersection
+            best, matches, num = None, 0, 0
+            for iface in existing_interfaces:
+                iface_items = set(iface.items())
+                if specified_net_items.issubset(iface_items):
+                    num = len(specified_net_items.intersection(iface_items))
+                if num > matches:
+                    best, matches = iface, num
+            return best
+
+    def _exclude_not_updated_networks(self, old_nets, new_nets, interfaces):
+        not_updated_nets = []
+
+        # Update old_nets to match interfaces
+        self.update_networks_matching_iface_port(old_nets, interfaces)
+        # make networks similar by adding None values for not used keys
         for key in self._NETWORK_KEYS:
             # if _net.get(key) is '', convert to None
             for _net in itertools.chain(new_nets, old_nets):
                 _net[key] = _net.get(key) or None
-        # find matches and remove them from old and new networks
-        not_updated_nets = [net for net in old_nets if net in new_nets]
-        for net in not_updated_nets:
-            old_nets.remove(net)
-            new_nets.remove(net)
+
+        for new_net in list(new_nets):
+            new_net_reduced = {k: v for k, v in new_net.items()
+                               if k not in self._IFACE_MANAGED_KEYS or
+                               v is not None}
+            match = self._find_best_match(old_nets, new_net_reduced)
+            if match is not None:
+                not_updated_nets.append(match)
+                new_nets.remove(new_net)
+                old_nets.remove(match)
+
         return not_updated_nets
 
     def _get_network_id(self, net):
@@ -296,49 +323,95 @@ class ServerNetworkMixin(object):
                 'neutron').network_id_from_subnet_id(subnet)
         return net_id
 
-    def update_networks_matching_iface_port(self, nets, interfaces):
+    def update_networks_matching_iface_port(self, old_nets, interfaces):
 
-        def find_equal(port, net_id, ip, nets):
-            for net in nets:
-                if (net.get('port') == port or
-                        (net.get('fixed_ip') == ip and
-                         self._get_network_id(net) == net_id)):
-                    return net
+        def get_iface_props(iface):
+            ipaddr = None
+            subnet = None
+            if len(iface.fixed_ips) > 0:
+                ipaddr = iface.fixed_ips[0]['ip_address']
+                subnet = iface.fixed_ips[0]['subnet_id']
+            return {self.NETWORK_PORT: iface.port_id,
+                    self.NETWORK_ID: iface.net_id,
+                    self.NETWORK_FIXED_IP: ipaddr,
+                    self.NETWORK_SUBNET: subnet}
 
-        def find_poor_net(net_id, nets):
-            for net in nets:
-                if (not net.get('port') and not net.get('fixed_ip') and
-                        self._get_network_id(net) == net_id):
-                    return net
+        interfaces_net_props = [get_iface_props(iface) for iface in interfaces]
+        for old_net in old_nets:
+            if old_net[self.NETWORK_PORT] is None:
+                old_net[self.NETWORK_ID] = self._get_network_id(old_net)
+            old_net_reduced = {k: v for k, v in old_net.items()
+                               if k in self._IFACE_MANAGED_KEYS and
+                               v is not None}
+            match = self._find_best_match(interfaces_net_props,
+                                          old_net_reduced)
+            if match is not None:
+                old_net.update(match)
+                interfaces_net_props.remove(match)
 
-        for iface in interfaces:
-            # get interface properties
-            props = {'port': iface.port_id,
-                     'net_id': iface.net_id,
-                     'ip': iface.fixed_ips[0]['ip_address'],
-                     'nets': nets}
-            # try to match by port or network_id with fixed_ip
-            net = find_equal(**props)
-            if net is not None:
-                net['port'] = props['port']
-                continue
-            # find poor net that has only network_id
-            net = find_poor_net(props['net_id'], nets)
-            if net is not None:
-                net['port'] = props['port']
+    def _get_available_networks(self):
+        # first we get the private networks owned by the tenant
+        search_opts = {'tenant_id': self.context.tenant_id, 'shared': False,
+                       'admin_state_up': True, }
+        nc = self.client('neutron')
+        nets = nc.list_networks(**search_opts).get('networks', [])
+        # second we get the public shared networks
+        search_opts = {'shared': True}
+        nets += nc.list_networks(**search_opts).get('networks', [])
 
-    def calculate_networks(self, old_nets, new_nets, ifaces,
-                           security_groups=None):
+        ids = [net['id'] for net in nets]
+
+        return ids
+
+    def _auto_allocate_network(self):
+        topology = self.client('neutron').get_auto_allocated_topology(
+            self.context.tenant_id)['auto_allocated_topology']
+
+        return topology['id']
+
+    def _calculate_using_str_network(self, ifaces, str_net,
+                                     security_groups=None):
+        add_nets = []
+        remove_ports = [iface.port_id for iface in ifaces or []]
+        if str_net == self.NETWORK_AUTO:
+            nets = self._get_available_networks()
+            if not nets:
+                nets = [self._auto_allocate_network()]
+            if len(nets) > 1:
+                msg = 'Multiple possible networks found.'
+                raise exception.UnableToAutoAllocateNetwork(message=msg)
+            handle_args = {'port_id': None, 'net_id': nets[0], 'fip': None}
+            if security_groups:
+                sg_ids = self.client_plugin(
+                    'neutron').get_secgroup_uuids(security_groups)
+                handle_args['security_groups'] = sg_ids
+            add_nets.append(handle_args)
+        return remove_ports, add_nets
+
+    def _calculate_using_list_networks(self, old_nets, new_nets, ifaces,
+                                       security_groups):
         remove_ports = []
         add_nets = []
-        attach_first_free_port = False
-        if not new_nets:
-            new_nets = []
-            attach_first_free_port = True
+        # if update networks between None and empty, no need to
+        # detach and attach, the server got first free port already.
+        if not new_nets and not old_nets:
+            return remove_ports, add_nets
 
-        # if old nets is None, it means that the server got first
+        new_nets = new_nets or []
+        old_nets = old_nets or []
+        remove_ports, not_updated_nets = self._calculate_remove_ports(
+            old_nets, new_nets, ifaces)
+        add_nets = self._calculate_add_nets(new_nets, not_updated_nets,
+                                            security_groups)
+
+        return remove_ports, add_nets
+
+    def _calculate_remove_ports(self, old_nets, new_nets, ifaces):
+        remove_ports = []
+        not_updated_nets = []
+        # if old nets is empty, it means that the server got first
         # free port. so we should detach this interface.
-        if old_nets is None:
+        if not old_nets:
             for iface in ifaces:
                 remove_ports.append(iface.port_id)
 
@@ -349,75 +422,117 @@ class ServerNetworkMixin(object):
         # 3. detach unmatched networks, which were present in old_nets
         # 4. attach unmatched networks, which were present in new_nets
         else:
-            # remove not updated networks from old and new networks lists,
-            # also get list these networks
-            not_updated_nets = self._exclude_not_updated_networks(old_nets,
-                                                                  new_nets)
+            # if old net is string net, remove the interfaces
+            if self._str_network(old_nets):
+                remove_ports = [iface.port_id for iface in ifaces or []]
+            else:
+                # remove not updated networks from old and new networks lists,
+                # also get list these networks
+                not_updated_nets = self._exclude_not_updated_networks(
+                    old_nets, new_nets, ifaces)
 
-            self.update_networks_matching_iface_port(
-                old_nets + not_updated_nets, ifaces)
+                # according to nova interface-detach command detached port
+                # will be deleted
+                inter_port_data = self._data_get_ports()
+                inter_port_ids = [p['id'] for p in inter_port_data]
+                for net in old_nets:
+                    port_id = net.get(self.NETWORK_PORT)
+                    # we can't match the port for some user case, like:
+                    # the internal port was detached in nova first, then
+                    # user update template to detach this nic. The internal
+                    # port will remains till we delete the server resource.
+                    if port_id:
+                        remove_ports.append(port_id)
+                        if port_id in inter_port_ids:
+                            # if we have internal port with such id, remove it
+                            # instantly.
+                            self._delete_internal_port(port_id)
+                    if net.get(self.NETWORK_FLOATING_IP):
+                        self._floating_ip_disassociate(
+                            net.get(self.NETWORK_FLOATING_IP))
+        return remove_ports, not_updated_nets
 
-            # according to nova interface-detach command detached port
-            # will be deleted
-            inter_port_data = self._data_get_ports()
-            inter_port_ids = [p['id'] for p in inter_port_data]
-            for net in old_nets:
-                port_id = net.get(self.NETWORK_PORT)
-                # we can't match the port for some user case, like:
-                # the internal port was detached in nova first, then
-                # user update template to detach this nic. The internal
-                # port will remains till we delete the server resource.
-                if port_id:
-                    remove_ports.append(port_id)
-                    if port_id in inter_port_ids:
-                        # if we have internal port with such id, remove it
-                        # instantly.
-                        self._delete_internal_port(port_id)
-                if net.get(self.NETWORK_FLOATING_IP):
-                    self._floating_ip_disassociate(
-                        net.get(self.NETWORK_FLOATING_IP))
+    def _calculate_add_nets(self, new_nets, not_updated_nets,
+                            security_groups):
+        add_nets = []
 
-        handler_kwargs = {'port_id': None, 'net_id': None, 'fip': None}
-        # if new_nets is None, we should attach first free port,
-        # according to similar behavior during instance creation
-        if attach_first_free_port:
+        # if new_nets is empty (including the non_updated_nets), we should
+        # attach first free port, similar to the behavior for instance
+        # creation
+        if not new_nets and not not_updated_nets:
+            handler_kwargs = {'port_id': None, 'net_id': None, 'fip': None}
+            if security_groups:
+                sec_uuids = self.client_plugin(
+                    'neutron').get_secgroup_uuids(security_groups)
+                handler_kwargs['security_groups'] = sec_uuids
             add_nets.append(handler_kwargs)
-        # attach section similar for both variants that
-        # were mentioned above
-        for idx, net in enumerate(new_nets):
-            handler_kwargs = {'port_id': None,
-                              'net_id': None,
-                              'fip': None}
+        else:
+            # attach section similar for both variants that
+            # were mentioned above
+            for idx, net in enumerate(new_nets):
+                handler_kwargs = {'port_id': None,
+                                  'net_id': None,
+                                  'fip': None}
 
-            if net.get(self.NETWORK_PORT):
-                handler_kwargs['port_id'] = net.get(self.NETWORK_PORT)
-            elif self.is_using_neutron() and net.get(self.NETWORK_SUBNET):
-                handler_kwargs['port_id'] = self._create_internal_port(
-                    net, idx, security_groups)
+                if net.get(self.NETWORK_PORT):
+                    handler_kwargs['port_id'] = net.get(self.NETWORK_PORT)
+                elif net.get(self.NETWORK_SUBNET):
+                    handler_kwargs['port_id'] = self._create_internal_port(
+                        net, idx, security_groups)
 
-            if not handler_kwargs['port_id']:
-                handler_kwargs['net_id'] = self._get_network_id(net)
-            if handler_kwargs['net_id']:
-                handler_kwargs['fip'] = net.get('fixed_ip')
+                if not handler_kwargs['port_id']:
+                    handler_kwargs['net_id'] = self._get_network_id(net)
+                    if security_groups:
+                        sec_uuids = self.client_plugin(
+                            'neutron').get_secgroup_uuids(security_groups)
+                        handler_kwargs['security_groups'] = sec_uuids
+                if handler_kwargs['net_id']:
+                    handler_kwargs['fip'] = net.get('fixed_ip')
 
-            floating_ip = net.get(self.NETWORK_FLOATING_IP)
-            if floating_ip:
-                flip_associate = {'port_id': handler_kwargs.get('port_id')}
-                if net.get('fixed_ip'):
-                    flip_associate['fixed_ip_address'] = net.get('fixed_ip')
+                floating_ip = net.get(self.NETWORK_FLOATING_IP)
+                if floating_ip:
+                    flip_associate = {'port_id': handler_kwargs.get('port_id')}
+                    if net.get('fixed_ip'):
+                        flip_associate['fixed_ip_address'] = net.get(
+                            'fixed_ip')
 
-                self.update_floating_ip_association(floating_ip,
-                                                    flip_associate)
+                    self.update_floating_ip_association(floating_ip,
+                                                        flip_associate)
 
-            add_nets.append(handler_kwargs)
+                add_nets.append(handler_kwargs)
 
-        return remove_ports, add_nets
+        return add_nets
+
+    def _str_network(self, networks):
+        # if user specify 'allocate_network', return it
+        # otherwise we return None
+        for net in networks or []:
+            str_net = net.get(self.ALLOCATE_NETWORK)
+            if str_net:
+                return str_net
+
+    def _is_nic_tagged(self, networks):
+        # if user specify 'tag', return True
+        # otherwise return False
+        for net in networks or []:
+            if net.get(self.NIC_TAG):
+                return True
+
+        return False
+
+    def calculate_networks(self, old_nets, new_nets, ifaces,
+                           security_groups=None):
+        new_str_net = self._str_network(new_nets)
+        if new_str_net:
+            return self._calculate_using_str_network(ifaces, new_str_net,
+                                                     security_groups)
+        else:
+            return self._calculate_using_list_networks(
+                old_nets, new_nets, ifaces, security_groups)
 
     def update_floating_ip_association(self, floating_ip, flip_associate):
-        if self.is_using_neutron() and flip_associate.get('port_id'):
+        if flip_associate.get('port_id'):
             self._floating_ip_neutron_associate(floating_ip, flip_associate)
-        elif not self.is_using_neutron():
-            self._floating_ip_nova_associate(floating_ip)
 
     @staticmethod
     def get_all_ports(server):
@@ -438,10 +553,10 @@ class ServerNetworkMixin(object):
             try:
                 if self.client_plugin().check_interface_detach(
                         existing_server_id, port['id']):
-                    LOG.info(_LI('Detach interface %(port)s successful from '
-                                 'server %(server)s.')
-                             % {'port': port['id'],
-                                'server': existing_server_id})
+                    LOG.info('Detach interface %(port)s successful from '
+                             'server %(server)s.',
+                             {'port': port['id'],
+                              'server': existing_server_id})
             except tenacity.RetryError:
                 raise exception.InterfaceDetachFailed(
                     port=port['id'], server=existing_server_id)
@@ -455,46 +570,31 @@ class ServerNetworkMixin(object):
             try:
                 if self.client_plugin().check_interface_attach(
                         prev_server_id, port['id']):
-                    LOG.info(_LI('Attach interface %(port)s successful to '
-                                 'server %(server)s')
-                             % {'port': port['id'],
-                                'server': prev_server_id})
+                    LOG.info('Attach interface %(port)s successful to '
+                             'server %(server)s',
+                             {'port': port['id'],
+                              'server': prev_server_id})
             except tenacity.RetryError:
                 raise exception.InterfaceAttachFailed(
                     port=port['id'], server=prev_server_id)
 
     def prepare_ports_for_replace(self):
-        if not self.is_using_neutron():
-            return
-
-        data = {'external_ports': [],
-                'internal_ports': []}
-        port_data = list(itertools.chain(
-            [('internal_ports', port) for port in self._data_get_ports()],
-            [('external_ports', port)
-             for port in self._data_get_ports('external_ports')]))
-        for port_type, port in port_data:
-            data[port_type].append({'id': port['id']})
-
         self.detach_ports(self)
 
     def restore_ports_after_rollback(self, convergence):
-        if not self.is_using_neutron():
-            return
-
         # In case of convergence, during rollback, the previous rsrc is
         # already selected and is being acted upon.
-        backup_stack = self.stack._backup_stack()
-        backup_res = backup_stack.resources.get(self.name)
-        prev_server = self if convergence else backup_res
-
         if convergence:
+            prev_server = self
             rsrc, rsrc_owning_stack, stack = resource.Resource.load(
-                prev_server.context, prev_server.replaced_by, True,
-                prev_server.stack.cache_data
+                prev_server.context, prev_server.replaced_by,
+                prev_server.stack.current_traversal, True,
+                prev_server.stack.defn._resource_data
             )
             existing_server = rsrc
         else:
+            backup_stack = self.stack._backup_stack()
+            prev_server = backup_stack.resources.get(self.name)
             existing_server = self
 
         # Wait until server will move to active state. We can't
