@@ -11,29 +11,30 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
+
 from heat.common.i18n import _
 from heat.engine import attributes
 from heat.engine import constraints
 from heat.engine import properties
-from heat.engine import resource
+from heat.engine.resources.openstack.senlin import res_base
 from heat.engine import support
+from heat.engine import translation
 
 
-class Node(resource.Resource):
+class Node(res_base.BaseSenlinResource):
     """A resource that creates a Senlin Node.
 
     Node is an object that belongs to at most one Cluster, it can be created
     based on a profile.
     """
 
-    support_status = support.SupportStatus(version='6.0.0')
-
-    default_client_name = 'senlin'
+    entity = 'node'
 
     PROPERTIES = (
-        NAME, METADATA, PROFILE,
+        NAME, METADATA, PROFILE, CLUSTER
     ) = (
-        'name', 'metadata', 'profile',
+        'name', 'metadata', 'profile', 'cluster'
     )
 
     _NODE_STATUS = (
@@ -69,6 +70,15 @@ class Node(resource.Resource):
                 constraints.CustomConstraint('senlin.profile')
             ]
         ),
+        CLUSTER: properties.Schema(
+            properties.Schema.STRING,
+            _('The name of senlin cluster to attach to.'),
+            update_allowed=True,
+            constraints=[
+                constraints.CustomConstraint('senlin.cluster')
+            ],
+            support_status=support.SupportStatus(version='8.0.0'),
+        ),
     }
 
     attributes_schema = {
@@ -82,12 +92,30 @@ class Node(resource.Resource):
         ),
     }
 
+    def translation_rules(self, props):
+        rules = [
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                translation_path=[self.PROFILE],
+                client_plugin=self.client_plugin(),
+                finder='get_profile_id'),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                translation_path=[self.CLUSTER],
+                client_plugin=self.client_plugin(),
+                finder='get_cluster_id'),
+        ]
+        return rules
+
     def handle_create(self):
         params = {
             'name': (self.properties[self.NAME] or
                      self.physical_resource_name()),
             'metadata': self.properties[self.METADATA],
             'profile_id': self.properties[self.PROFILE],
+            'cluster_id': self.properties[self.CLUSTER],
         }
 
         node = self.client().create_node(**params)
@@ -115,30 +143,77 @@ class Node(resource.Resource):
             return True
         return False
 
-    def _show_resource(self):
-        node = self.client().get_node(self.resource_id)
-        return node.to_dict()
-
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        action_id = None
+        actions = []
         if prop_diff:
+            old_cluster = None
+            new_cluster = None
             if self.PROFILE in prop_diff:
                 prop_diff['profile_id'] = prop_diff.pop(self.PROFILE)
-            node = self.client().update_node(
-                self.resource_id, **prop_diff)
-            action_id = node.location.split('/')[-1]
+            if self.CLUSTER in prop_diff:
+                old_cluster = self.properties[self.CLUSTER]
+                new_cluster = prop_diff.pop(self.CLUSTER)
+            if old_cluster:
+                params = {
+                    'cluster': old_cluster,
+                    'nodes': [self.resource_id],
+                }
+                action = {
+                    'func': 'cluster_del_nodes',
+                    'action_id': None,
+                    'params': params,
+                    'done': False,
+                }
+                actions.append(action)
+            if prop_diff:
+                node = self.client().get_node(self.resource_id)
+                params = copy.deepcopy(prop_diff)
+                params['node'] = node
+                action = {
+                    'func': 'update_node',
+                    'action_id': None,
+                    'params': params,
+                    'done': False,
+                }
+                actions.append(action)
+            if new_cluster:
+                params = {
+                    'cluster': new_cluster,
+                    'nodes': [self.resource_id],
+                }
+                action = {
+                    'func': 'cluster_add_nodes',
+                    'action_id': None,
+                    'params': params,
+                    'done': False,
+                }
+                actions.append(action)
 
-        return action_id
+        return actions
 
-    def check_update_complete(self, action_id):
-        if action_id is None:
-            return True
-        return self.client_plugin().check_action_status(action_id)
+    def check_update_complete(self, actions):
+        return self.client_plugin().execute_actions(actions)
 
     def _resolve_attribute(self, name):
+        if self.resource_id is None:
+            return
         node = self.client().get_node(self.resource_id, details=True)
         return getattr(node, name, None)
 
+    def parse_live_resource_data(self, resource_properties, resource_data):
+        reality = {}
+
+        for key in self._update_allowed_properties:
+            if key == self.PROFILE:
+                value = resource_data.get('profile_id')
+            elif key == self.CLUSTER:
+                value = resource_data.get('cluster_id')
+            else:
+                value = resource_data.get(key)
+            reality.update({key: value})
+
+        return reality
+
 
 def resource_mapping():
     return {
