@@ -18,20 +18,18 @@ from heat.common import exception
 from heat.common.i18n import _
 from heat.engine import constraints
 from heat.engine import properties
-from heat.engine import resource
-from heat.engine import support
+from heat.engine.resources.openstack.senlin import res_base
+from heat.engine import translation
 
 
-class Policy(resource.Resource):
+class Policy(res_base.BaseSenlinResource):
     """A resource that creates a Senlin Policy.
 
     A policy is a set of rules that can be checked and/or enforced when
     an action is performed on a Cluster.
     """
 
-    support_status = support.SupportStatus(version='6.0.0')
-
-    default_client_name = 'senlin'
+    entity = 'policy'
 
     PROPERTIES = (
         NAME, TYPE, POLICY_PROPS, BINDINGS,
@@ -95,6 +93,17 @@ class Policy(resource.Resource):
         )
     }
 
+    def translation_rules(self, props):
+        rules = [
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                translation_path=[self.BINDINGS, self.BD_CLUSTER],
+                client_plugin=self.client_plugin(),
+                finder='get_cluster_id'),
+        ]
+        return rules
+
     def remove_bindings(self, bindings):
         for bd in bindings:
             try:
@@ -175,7 +184,8 @@ class Policy(resource.Resource):
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         if self.NAME in prop_diff:
             param = {'name': prop_diff[self.NAME]}
-            self.client().update_policy(self.resource_id, **param)
+            policy_obj = self.client().get_policy(self.resource_id)
+            self.client().update_policy(policy_obj, **param)
         actions = dict()
         if self.BINDINGS in prop_diff:
             old = self.properties[self.BINDINGS] or []
@@ -196,10 +206,6 @@ class Policy(resource.Resource):
             ret = self.check_action_done(actions['add'])
         return ret
 
-    def _show_resource(self):
-        policy = self.client().get_policy(self.resource_id)
-        return policy.to_dict()
-
 
 def resource_mapping():
     return {
