@@ -29,6 +29,7 @@ from heat.engine import environment
 from heat.engine import resource
 from heat.engine import scheduler
 from heat.engine import stack as parser
+from heat.engine import stk_defn
 from heat.engine import template
 from heat.objects import raw_template
 from heat.objects import stack as stack_object
@@ -221,6 +222,7 @@ class StackResource(resource.Resource):
 
         parsed_template = self._child_parsed_template(child_template,
                                                       child_env)
+        self._validate_nested_resources(parsed_template)
 
         # Note we disable rollback for nested stacks, since they
         # should be rolled back by the parent stack on failure
@@ -247,7 +249,6 @@ class StackResource(resource.Resource):
 
     def _child_parsed_template(self, child_template, child_env):
         parsed_template = self._parse_child_template(child_template, child_env)
-        self._validate_nested_resources(parsed_template)
 
         # Don't overwrite the attributes_schema for subclasses that
         # define their own attributes_schema.
@@ -259,12 +260,16 @@ class StackResource(resource.Resource):
     def _validate_nested_resources(self, templ):
         if cfg.CONF.max_resources_per_stack == -1:
             return
+
         total_resources = (len(templ[templ.RESOURCES]) +
                            self.stack.total_resources(self.root_stack_id))
 
-        if self.nested():
-            # It's an update and these resources will be deleted
-            total_resources -= len(self.nested().resources)
+        identity = self.nested_identifier()
+        if identity is not None:
+            existing = self.rpc_client().list_stack_resources(self.context,
+                                                              identity)
+            # Don't double-count existing resources during an update
+            total_resources -= len(existing)
 
         if (total_resources > cfg.CONF.max_resources_per_stack):
             message = exception.StackResourceLimitExceeded.msg_fmt
@@ -316,12 +321,15 @@ class StackResource(resource.Resource):
 
         self.resource_id_set(result['stack_id'])
 
-    def _stack_kwargs(self, user_params, child_template, adopt_data=None):
-
+    def child_definition(self, child_template=None, user_params=None,
+                         nested_identifier=None):
         if user_params is None:
             user_params = self.child_params()
         if child_template is None:
             child_template = self.child_template()
+        if nested_identifier is None:
+            nested_identifier = self.nested_identifier()
+
         child_env = environment.get_child_environment(
             self.stack.env,
             user_params,
@@ -330,6 +338,14 @@ class StackResource(resource.Resource):
 
         parsed_template = self._child_parsed_template(child_template,
                                                       child_env)
+        return stk_defn.StackDefinition(self.context, parsed_template,
+                                        nested_identifier,
+                                        None)
+
+    def _stack_kwargs(self, user_params, child_template, adopt_data=None):
+        defn = self.child_definition(child_template, user_params)
+        parsed_template = defn.t
+
         if adopt_data is None:
             template_id = parsed_template.store(self.context)
             return {
@@ -341,7 +357,7 @@ class StackResource(resource.Resource):
         else:
             return {
                 'template': parsed_template.t,
-                'params': child_env.user_env_as_dict(),
+                'params': defn.env.user_env_as_dict(),
                 'files': parsed_template.files,
             }
 
@@ -594,10 +610,7 @@ class StackResource(resource.Resource):
     def get_output(self, op):
         """Return the specified Output value from the nested stack.
 
-        If the output key does not exist, raise an InvalidTemplateAttribute
-        exception. (Note that TemplateResource.get_attribute() relies on this
-        particular exception, not KeyError, being raised if the key does not
-        exist.)
+        If the output key does not exist, raise a NotFound exception.
         """
         if (self._outputs is None or
                 (op in self._outputs and
@@ -614,8 +627,8 @@ class StackResource(resource.Resource):
             self._outputs = {o[rpc_api.OUTPUT_KEY]: o for o in outputs}
 
         if op not in self._outputs:
-            raise exception.InvalidTemplateAttribute(resource=self.name,
-                                                     key=op)
+            raise exception.NotFound(_('Specified output key %s not '
+                                       'found.') % op)
 
         output_data = self._outputs[op]
         if rpc_api.OUTPUT_ERROR in output_data:
@@ -627,4 +640,8 @@ class StackResource(resource.Resource):
         return output_data[rpc_api.OUTPUT_VALUE]
 
     def _resolve_attribute(self, name):
-        return self.get_output(name)
+        try:
+            return self.get_output(name)
+        except exception.NotFound:
+            raise exception.InvalidTemplateAttribute(resource=self.name,
+                                                     key=name)
