@@ -11,10 +11,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_config import cfg
+
 from heat.common import exception
 from heat.common.i18n import _
 from heat.engine.clients import progress
 from heat.engine import resource
+from heat.engine import rsrc_defn
 
 
 class BaseVolume(resource.Resource):
@@ -22,20 +25,20 @@ class BaseVolume(resource.Resource):
 
     default_client_name = 'cinder'
 
+    def _create_arguments(self):
+        return {}
+
     def handle_create(self):
         backup_id = self.properties.get(self.BACKUP_ID)
         cinder = self.client()
         if backup_id is not None:
             vol_id = cinder.restores.restore(backup_id).volume_id
-
             vol = cinder.volumes.get(vol_id)
-            kwargs = self._fetch_name_and_description(
-                cinder.volume_api_version)
+            kwargs = self._fetch_name_and_description()
             cinder.volumes.update(vol_id, **kwargs)
         else:
             kwargs = self._create_arguments()
-            kwargs.update(self._fetch_name_and_description(
-                cinder.volume_api_version))
+            kwargs.update(self._fetch_name_and_description())
             vol = cinder.volumes.create(**kwargs)
         self.resource_id_set(vol.id)
 
@@ -62,14 +65,10 @@ class BaseVolume(resource.Resource):
     def _description(self):
         return self.physical_resource_name()
 
-    def _fetch_name_and_description(self, api_version, name=None,
+    def _fetch_name_and_description(self, name=None,
                                     description=None):
-        if api_version == 1:
-            return {'display_name': name or self._name(),
-                    'display_description': description or self._description()}
-        else:
-            return {'name': name or self._name(),
-                    'description': description or self._description()}
+        return {'name': name or self._name(),
+                'description': description or self._description()}
 
     def handle_check(self):
         vol = self.client().volumes.get(self.resource_id)
@@ -159,6 +158,17 @@ class BaseVolume(resource.Resource):
                 return False
         return True
 
+    @classmethod
+    def validate_deletion_policy(cls, policy):
+        res = super(BaseVolume, cls).validate_deletion_policy(policy)
+        if res:
+            return res
+        if (policy == rsrc_defn.ResourceDefinition.SNAPSHOT and
+                not cfg.CONF.volumes.backups_enabled):
+            msg = _('"%s" deletion policy not supported - '
+                    'volume backup service is not enabled.') % policy
+            raise exception.StackValidationFailed(message=msg)
+
 
 class BaseVolumeAttachment(resource.Resource):
     """Base Volume Attachment Manager."""
