@@ -57,6 +57,10 @@ from heat.rpc import worker_client as rpc_worker_client
 LOG = logging.getLogger(__name__)
 
 
+ConvergenceNode = collections.namedtuple('ConvergenceNode',
+                                         ['rsrc_id', 'is_update'])
+
+
 class ForcedCancel(Exception):
     """Exception raised to cancel task execution."""
 
@@ -385,7 +389,10 @@ class Stack(collections.Mapping):
         elif stk_def_cache and tid in stk_def_cache:
             stk_def = stk_def_cache[tid]
         else:
-            t = tmpl.Template.load(self.context, tid)
+            try:
+                t = tmpl.Template.load(self.context, tid)
+            except exception.NotFound:
+                return None
             stk_def = self.defn.clone_with_new_template(t,
                                                         self.identifier())
             if stk_def_cache is not None:
@@ -396,7 +403,9 @@ class Stack(collections.Mapping):
         except KeyError:
             return None
 
-        return resource.Resource(db_res.name, defn, self)
+        res = resource.Resource(db_res.name, defn, self)
+        res._load_data(db_res)
+        return res
 
     def resource_get(self, name):
         """Return a stack resource, even if not in the current template."""
@@ -898,13 +907,15 @@ class Stack(collections.Mapping):
 
         for op_name, output in six.iteritems(self.outputs):
             try:
-                path = '.'.join([self.t.OUTPUTS, op_name,
-                                 self.t.OUTPUT_VALUE])
-                output.validate(path)
-            except exception.StackValidationFailed:
-                raise
-            except AssertionError:
-                raise
+                output.validate()
+            except exception.StackValidationFailed as ex:
+                path = [self.t.OUTPUTS, op_name,
+                        self.t.get_section_name(ex.path[0])]
+                path.extend(ex.path[1:])
+                raise exception.StackValidationFailed(
+                    error=ex.error,
+                    path=path,
+                    message=ex.error_message)
 
     def requires_deferred_auth(self):
         """Determine whether to perform API requests with deferred auth.
@@ -918,7 +929,7 @@ class Stack(collections.Mapping):
     def _add_event(self, action, status, reason):
         """Add a state change event to the database."""
         ev = event.Event(self.context, self, action, status, reason,
-                         self.id, None,
+                         self.id, None, None,
                          self.name, 'OS::Heat::Stack')
 
         ev.store()
@@ -958,13 +969,11 @@ class Stack(collections.Mapping):
             # delete, stack lock is not used, hence persist state
             updated = self._persist_state()
             if not updated:
-                # Possibly failed concurrent update
-                LOG.warning("Failed to set state of stack %(name)s with"
-                            " traversal ID %(trvsl_id)s, to"
-                            " %(action)s_%(status)s",
-                            {'name': self.name,
-                             'trvsl_id': self.current_traversal,
-                             'action': action, 'status': status})
+                LOG.info("Stack %(name)s traversal %(trvsl_id)s no longer "
+                         "active; not setting state to %(action)s_%(status)s",
+                         {'name': self.name,
+                          'trvsl_id': self.current_traversal,
+                          'action': action, 'status': status})
             return updated
 
         # Persist state to db only if status == IN_PROGRESS
@@ -1337,10 +1346,11 @@ class Stack(collections.Mapping):
             self.context, self.id, self.current_traversal, True, self.id)
 
         leaves = set(self.convergence_dependencies.leaves())
-        if not any(leaves):
+        if not leaves:
             self.mark_complete()
         else:
-            for rsrc_id, is_update in self.convergence_dependencies.leaves():
+            for rsrc_id, is_update in sorted(leaves,
+                                             key=lambda n: n.is_update):
                 if is_update:
                     LOG.info("Triggering resource %s for update", rsrc_id)
                 else:
@@ -1377,7 +1387,7 @@ class Stack(collections.Mapping):
     def _get_best_existing_rsrc_db(self, rsrc_name):
         candidate = None
         if self.ext_rsrcs_db:
-            for id, ext_rsrc in self.ext_rsrcs_db.items():
+            for ext_rsrc in self.ext_rsrcs_db.values():
                 if ext_rsrc.name != rsrc_name:
                     continue
                 if ext_rsrc.current_template_id == self.t.id:
@@ -1440,27 +1450,32 @@ class Stack(collections.Mapping):
     def _compute_convg_dependencies(self, existing_resources,
                                     current_template_deps, current_resources):
         def make_graph_key(rsrc):
-            return current_resources[rsrc.name].id, True
+            return ConvergenceNode(current_resources[rsrc.name].id, True)
+
         dep = current_template_deps.translate(make_graph_key)
         if existing_resources:
             for rsrc_id, rsrc in existing_resources.items():
-                dep += (rsrc_id, False), None
+                dep += ConvergenceNode(rsrc_id, False), None
 
                 for requirement in rsrc.requires:
                     if requirement in existing_resources:
-                        dep += (requirement, False), (rsrc_id, False)
+                        dep += (ConvergenceNode(requirement, False),
+                                ConvergenceNode(rsrc_id, False))
                 if rsrc.replaces in existing_resources:
-                    dep += (rsrc.replaces, False), (rsrc_id, False)
+                    dep += (ConvergenceNode(rsrc.replaces, False),
+                            ConvergenceNode(rsrc_id, False))
 
-                if (rsrc.id, True) in dep:
-                    dep += (rsrc_id, False), (rsrc_id, True)
+                if ConvergenceNode(rsrc.id, True) in dep:
+                    dep += (ConvergenceNode(rsrc_id, False),
+                            ConvergenceNode(rsrc_id, True))
 
         self._convg_deps = dep
 
     @property
     def convergence_dependencies(self):
         if self._convg_deps is None:
-            current_deps = ([tuple(i), (tuple(j) if j is not None else None)]
+            current_deps = ((ConvergenceNode(*i),
+                             ConvergenceNode(*j) if j is not None else None)
                             for i, j in self.current_deps['edges'])
             self._convg_deps = dependencies.Dependencies(edges=current_deps)
 
@@ -1973,44 +1988,6 @@ class Stack(collections.Mapping):
                                        action=self.RESTORE)
         updater()
 
-    def restart_resource(self, resource_name):
-        """Restart the resource specified by resource_name.
-
-        stop resource_name and all that depend on it
-        start resource_name and all that depend on it
-        """
-        warnings.warn("Stack.restart_resource() is horribly broken and will "
-                      "never be fixed. If you're using it in a resource type "
-                      "other than HARestarter, don't. And don't use "
-                      "HARestarter either.",
-                      DeprecationWarning)
-
-        deps = self.dependencies[self[resource_name]]
-        failed = False
-
-        for res in reversed(deps):
-            try:
-                scheduler.TaskRunner(res.destroy)()
-            except exception.ResourceFailure as ex:
-                failed = True
-                LOG.info('Resource %(name)s delete failed: %(ex)s',
-                         {'name': res.name, 'ex': ex})
-
-        for res in deps:
-            if not failed:
-                try:
-                    res.state_reset()
-                    scheduler.TaskRunner(res.create)()
-                except exception.ResourceFailure as ex:
-                    failed = True
-                    LOG.info('Resource %(name)s create failed: '
-                             '%(ex)s', {'name': res.name, 'ex': ex})
-            else:
-                res.state_set(res.CREATE, res.FAILED,
-                              'Resource restart aborted')
-        # TODO(asalkeld) if any of this fails we Should
-        # restart the whole stack
-
     def get_availability_zones(self):
         nova = self.clients.client('nova')
         if self._zones is None:
