@@ -18,7 +18,7 @@
 from oslo_versionedobjects import base
 from oslo_versionedobjects import fields
 
-from heat.db import api as db_api
+from heat.db.sqlalchemy import api as db_api
 from heat.objects import base as heat_base
 from heat.objects import fields as heat_fields
 
