@@ -11,11 +11,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_log import log as logging
+import datetime
 
 from heat.common import exception
 from heat.common.i18n import _
 from heat.engine import resource
+from oslo_log import log as logging
 from oslo_utils import timeutils
 import six
 
@@ -25,11 +26,15 @@ LOG = logging.getLogger(__name__)
 class CooldownMixin(object):
     """Utility class to encapsulate Cooldown related logic.
 
-    This class is shared between AutoScalingGroup and ScalingPolicy.
     This logic includes both cooldown timestamp comparing and scaling in
     progress checking.
     """
-    def _check_scaling_allowed(self):
+    def _sanitize_cooldown(self, cooldown):
+        if cooldown is None:
+            return 0
+        return max(0, cooldown)
+
+    def _check_scaling_allowed(self, cooldown):
         metadata = self.metadata_get()
         if metadata.get('scaling_in_progress'):
             LOG.info("Can not perform scaling action: resource %s "
@@ -37,51 +42,58 @@ class CooldownMixin(object):
             reason = _('due to scaling activity')
             raise resource.NoActionRequired(res_name=self.name,
                                             reason=reason)
-        try:
-            # Negative values don't make sense, so they are clamped to zero
-            cooldown = max(0, self.properties[self.COOLDOWN])
-        except TypeError:
-            # If not specified, it will be None, same as cooldown == 0
-            cooldown = 0
+        cooldown = self._sanitize_cooldown(cooldown)
+        # if both cooldown and cooldown_end not in metadata
+        if all(k not in metadata for k in ('cooldown', 'cooldown_end')):
+            # Note: this is for supporting old version cooldown checking
+            metadata.pop('scaling_in_progress', None)
+            if metadata and cooldown != 0:
+                last_adjust = next(six.iterkeys(metadata))
+                if not timeutils.is_older_than(last_adjust, cooldown):
+                    self._log_and_raise_no_action(cooldown)
 
-        if cooldown != 0:
-            try:
-                if 'cooldown' not in metadata:
-                    # Note: this is for supporting old version cooldown logic
-                    if metadata:
-                        last_adjust = next(six.iterkeys(metadata))
-                        self._cooldown_check(cooldown, last_adjust)
-                else:
-                    last_adjust = next(six.iterkeys(metadata['cooldown']))
-                    self._cooldown_check(cooldown, last_adjust)
-            except ValueError:
-                # occurs when metadata has only {scaling_in_progress: False}
-                pass
+        elif 'cooldown_end' in metadata:
+            cooldown_end = next(six.iterkeys(metadata['cooldown_end']))
+            now = timeutils.utcnow().isoformat()
+            if now < cooldown_end:
+                self._log_and_raise_no_action(cooldown)
+
+        elif cooldown != 0:
+            # Note: this is also for supporting old version cooldown checking
+            last_adjust = next(six.iterkeys(metadata['cooldown']))
+            if not timeutils.is_older_than(last_adjust, cooldown):
+                self._log_and_raise_no_action(cooldown)
 
         # Assumes _finished_scaling is called
         # after the scaling operation completes
         metadata['scaling_in_progress'] = True
         self.metadata_set(metadata)
 
-    def _cooldown_check(self, cooldown, last_adjust):
-        if not timeutils.is_older_than(last_adjust, cooldown):
-            LOG.info("Can not perform scaling action: "
-                     "resource %(name)s is in cooldown (%(cooldown)s).",
-                     {'name': self.name,
-                      'cooldown': cooldown})
-            reason = _('due to cooldown, '
-                       'cooldown %s') % cooldown
-            raise resource.NoActionRequired(
-                res_name=self.name, reason=reason)
+    def _log_and_raise_no_action(self, cooldown):
+        LOG.info("Can not perform scaling action: "
+                 "resource %(name)s is in cooldown (%(cooldown)s).",
+                 {'name': self.name,
+                  'cooldown': cooldown})
+        reason = _('due to cooldown, '
+                   'cooldown %s') % cooldown
+        raise resource.NoActionRequired(
+            res_name=self.name, reason=reason)
 
-    def _finished_scaling(self, cooldown_reason, size_changed=True):
+    def _finished_scaling(self, cooldown,
+                          cooldown_reason, size_changed=True):
         # If we wanted to implement the AutoScaling API like AWS does,
         # we could maintain event history here, but since we only need
         # the latest event for cooldown, just store that for now
         metadata = self.metadata_get()
         if size_changed:
-            now = timeutils.utcnow().isoformat()
-            metadata['cooldown'] = {now: cooldown_reason}
+            cooldown = self._sanitize_cooldown(cooldown)
+            cooldown_end = (timeutils.utcnow() + datetime.timedelta(
+                seconds=cooldown)).isoformat()
+            if 'cooldown_end' in metadata:
+                cooldown_end = max(
+                    next(six.iterkeys(metadata['cooldown_end'])),
+                    cooldown_end)
+            metadata['cooldown_end'] = {cooldown_end: cooldown_reason}
         metadata['scaling_in_progress'] = False
         try:
             self.metadata_set(metadata)
