@@ -28,10 +28,11 @@ from heat.tests import common
 
 @mock.patch.object(policy.Enforcer, 'enforce')
 class ResourceControllerTest(tools.ControllerTest, common.HeatTestCase):
-    '''
+    """Tests the API class ResourceController.
+
     Tests the API class which acts as the WSGI controller,
     the endpoint processing API requests after they are routed
-    '''
+    """
 
     def setUp(self):
         super(ResourceControllerTest, self).setUp()
@@ -74,8 +75,9 @@ class ResourceControllerTest(tools.ControllerTest, common.HeatTestCase):
             ('list_stack_resources', {'stack_identity': stack_identity,
                                       'nested_depth': 0,
                                       'with_detail': False,
+                                      'filters': {}
                                       }),
-            version='1.12'
+            version='1.25'
         ).AndReturn(engine_resp)
         self.m.ReplayAll()
 
@@ -107,14 +109,15 @@ class ResourceControllerTest(tools.ControllerTest, common.HeatTestCase):
 
         req = self._get(stack_identity._tenant_path() + '/resources')
 
-        error = heat_exc.StackNotFound(stack_name='a')
+        error = heat_exc.EntityNotFound(entity='Stack', name='a')
         self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
         rpc_client.EngineClient.call(
             req.context,
             ('list_stack_resources', {'stack_identity': stack_identity,
                                       'nested_depth': 0,
-                                      'with_detail': False}),
-            version='1.12'
+                                      'with_detail': False,
+                                      'filters': {}}),
+            version='1.25'
         ).AndRaise(tools.to_remote_error(error))
         self.m.ReplayAll()
 
@@ -126,9 +129,29 @@ class ResourceControllerTest(tools.ControllerTest, common.HeatTestCase):
             stack_id=stack_identity.stack_id)
 
         self.assertEqual(404, resp.json['code'])
-        self.assertEqual('StackNotFound', resp.json['error']['type'])
+        self.assertEqual('EntityNotFound', resp.json['error']['type'])
         self.m.VerifyAll()
 
+    def test_index_invalid_filters(self, mock_enforce):
+        self._mock_enforce_setup(mock_enforce, 'index', True)
+        stack_identity = identifier.HeatIdentifier(self.tenant,
+                                                   'rubbish', '1')
+
+        req = self._get(stack_identity._tenant_path() + '/resources',
+                        {'invalid_key': 'junk'})
+
+        mock_call = self.patchobject(rpc_client.EngineClient, 'call')
+        ex = self.assertRaises(webob.exc.HTTPBadRequest,
+                               self.controller.index, req,
+                               tenant_id=self.tenant,
+                               stack_name=stack_identity.stack_name,
+                               stack_id=stack_identity.stack_id)
+
+        self.assertIn("Invalid filter parameters %s" %
+                      [six.text_type('invalid_key')],
+                      six.text_type(ex))
+        self.assertFalse(mock_call.called)
+
     def test_index_nested_depth(self, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'index', True)
         stack_identity = identifier.HeatIdentifier(self.tenant,
@@ -142,8 +165,9 @@ class ResourceControllerTest(tools.ControllerTest, common.HeatTestCase):
             req.context,
             ('list_stack_resources', {'stack_identity': stack_identity,
                                       'nested_depth': 99,
-                                      'with_detail': False}),
-            version='1.12'
+                                      'with_detail': False,
+                                      'filters': {}}),
+            version='1.25'
         ).AndReturn([])
         self.m.ReplayAll()
 
@@ -237,8 +261,9 @@ class ResourceControllerTest(tools.ControllerTest, common.HeatTestCase):
             req.context,
             ('list_stack_resources', {'stack_identity': stack_identity,
                                       'nested_depth': 0,
-                                      'with_detail': True}),
-            version='1.12'
+                                      'with_detail': True,
+                                      'filters': {}}),
+            version='1.25'
         ).AndReturn(engine_resp)
         self.m.ReplayAll()
 
@@ -393,7 +418,7 @@ class ResourceControllerTest(tools.ControllerTest, common.HeatTestCase):
 
         req = self._get(res_identity._tenant_path())
 
-        error = heat_exc.StackNotFound(stack_name='a')
+        error = heat_exc.EntityNotFound(entity='Stack', name='a')
         self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
         rpc_client.EngineClient.call(
             req.context,
@@ -413,7 +438,7 @@ class ResourceControllerTest(tools.ControllerTest, common.HeatTestCase):
             resource_name=res_name)
 
         self.assertEqual(404, resp.json['code'])
-        self.assertEqual('StackNotFound', resp.json['error']['type'])
+        self.assertEqual('EntityNotFound', resp.json['error']['type'])
         self.m.VerifyAll()
 
     def test_show_with_single_attribute(self, mock_enforce):
@@ -599,7 +624,7 @@ class ResourceControllerTest(tools.ControllerTest, common.HeatTestCase):
 
         req = self._get(res_identity._tenant_path() + '/metadata')
 
-        error = heat_exc.StackNotFound(stack_name='a')
+        error = heat_exc.EntityNotFound(entity='Stack', name='a')
         self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
         rpc_client.EngineClient.call(
             req.context,
@@ -619,7 +644,7 @@ class ResourceControllerTest(tools.ControllerTest, common.HeatTestCase):
             resource_name=res_name)
 
         self.assertEqual(404, resp.json['code'])
-        self.assertEqual('StackNotFound', resp.json['error']['type'])
+        self.assertEqual('EntityNotFound', resp.json['error']['type'])
         self.m.VerifyAll()
 
     def test_metadata_show_nonexist_resource(self, mock_enforce):
@@ -702,3 +727,140 @@ class ResourceControllerTest(tools.ControllerTest, common.HeatTestCase):
 
         self.assertIsNone(result)
         self.m.VerifyAll()
+
+    def test_mark_unhealthy_valid_request(self, mock_enforce):
+        self._mock_enforce_setup(mock_enforce, 'mark_unhealthy', True)
+        res_name = 'WebServer'
+        stack_identity = identifier.HeatIdentifier(self.tenant,
+                                                   'wordpress', '1')
+
+        req = self._get(stack_identity._tenant_path())
+
+        self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
+        body = {'mark_unhealthy': True,
+                rpc_api.RES_STATUS_DATA: 'Anything'}
+        params = {'stack_identity': stack_identity,
+                  'resource_name': res_name}
+        params.update(body)
+
+        rpc_client.EngineClient.call(
+            req.context,
+            ('resource_mark_unhealthy', params),
+            version='1.26')
+        self.m.ReplayAll()
+
+        result = self.controller.mark_unhealthy(
+            req, tenant_id=self.tenant,
+            stack_name=stack_identity.stack_name,
+            stack_id=stack_identity.stack_id,
+            resource_name=res_name,
+            body=body)
+
+        self.assertIsNone(result)
+        self.m.VerifyAll()
+
+    def test_mark_unhealthy_without_reason(self, mock_enforce):
+        self._mock_enforce_setup(mock_enforce, 'mark_unhealthy', True)
+        res_name = 'WebServer'
+        stack_identity = identifier.HeatIdentifier(self.tenant,
+                                                   'wordpress', '1')
+
+        req = self._get(stack_identity._tenant_path())
+
+        self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
+        body = {'mark_unhealthy': True, rpc_api.RES_STATUS_DATA: ''}
+        params = {'stack_identity': stack_identity,
+                  'resource_name': res_name}
+        params.update(body)
+
+        rpc_client.EngineClient.call(
+            req.context,
+            ('resource_mark_unhealthy', params),
+            version='1.26')
+        self.m.ReplayAll()
+
+        del body[rpc_api.RES_STATUS_DATA]
+
+        result = self.controller.mark_unhealthy(
+            req, tenant_id=self.tenant,
+            stack_name=stack_identity.stack_name,
+            stack_id=stack_identity.stack_id,
+            resource_name=res_name,
+            body=body)
+
+        self.assertIsNone(result)
+        self.m.VerifyAll()
+
+    def test_mark_unhealthy_with_invalid_keys(self, mock_enforce):
+        self._mock_enforce_setup(mock_enforce, 'mark_unhealthy', True)
+        res_name = 'WebServer'
+        stack_identity = identifier.HeatIdentifier(self.tenant,
+                                                   'wordpress', '1')
+
+        req = self._get(stack_identity._tenant_path())
+
+        self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
+        body = {'mark_unhealthy': True,
+                rpc_api.RES_STATUS_DATA: 'Any',
+                'invalid_key1': 1, 'invalid_key2': 2}
+        expected = "Invalid keys in resource mark unhealthy"
+        actual = self.assertRaises(webob.exc.HTTPBadRequest,
+                                   self.controller.mark_unhealthy, req,
+                                   tenant_id=self.tenant,
+                                   stack_name=stack_identity.stack_name,
+                                   stack_id=stack_identity.stack_id,
+                                   resource_name=res_name,
+                                   body=body)
+
+        self.assertIn(expected, six.text_type(actual))
+        self.assertIn('invalid_key1', six.text_type(actual))
+        self.assertIn('invalid_key2', six.text_type(actual))
+
+    def test_mark_unhealthy_with_invalid_value(self, mock_enforce):
+        self._mock_enforce_setup(mock_enforce, 'mark_unhealthy', True)
+        res_name = 'WebServer'
+        stack_identity = identifier.HeatIdentifier(self.tenant,
+                                                   'wordpress', '1')
+
+        req = self._get(stack_identity._tenant_path())
+
+        self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
+        body = {'mark_unhealthy': 'XYZ',
+                rpc_api.RES_STATUS_DATA: 'Any'}
+
+        expected = ('Unrecognized value "XYZ" for "mark_unhealthy", '
+                    'acceptable values are: true, false')
+
+        actual = self.assertRaises(webob.exc.HTTPBadRequest,
+                                   self.controller.mark_unhealthy, req,
+                                   tenant_id=self.tenant,
+                                   stack_name=stack_identity.stack_name,
+                                   stack_id=stack_identity.stack_id,
+                                   resource_name=res_name,
+                                   body=body)
+
+        self.assertIn(expected, six.text_type(actual))
+
+    def test_mark_unhealthy_without_mark_unhealthy_key(self, mock_enforce):
+        self._mock_enforce_setup(mock_enforce, 'mark_unhealthy', True)
+        res_name = 'WebServer'
+        stack_identity = identifier.HeatIdentifier(self.tenant,
+                                                   'wordpress', '1')
+
+        req = self._get(stack_identity._tenant_path())
+
+        self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
+        body = {rpc_api.RES_STATUS_DATA: 'Any'}
+
+        expected = ("Missing mandatory (%s) key from "
+                    "mark unhealthy request" % 'mark_unhealthy')
+
+        actual = self.assertRaises(webob.exc.HTTPBadRequest,
+                                   self.controller.mark_unhealthy, req,
+                                   tenant_id=self.tenant,
+                                   stack_name=stack_identity.stack_name,
+                                   stack_id=stack_identity.stack_id,
+                                   resource_name=res_name,
+                                   body=body)
+
+        self.assertIn(expected, six.text_type(actual))
