@@ -11,9 +11,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import datetime
 import json
 
 import mock
+from oslo_utils import timeutils
 import six
 
 from heat.common import exception
@@ -357,6 +359,7 @@ class TestGroupAdjust(common.HeatTestCase):
         self.assertEqual(expected_notifies, notify.call_args_list)
         resize.assert_called_once_with(3)
         finished_scaling.assert_called_once_with(
+            None,
             'PercentChangeInCapacity : 33',
             size_changed=True)
 
@@ -388,6 +391,7 @@ class TestGroupAdjust(common.HeatTestCase):
         self.assertEqual(expected_notifies, notify.call_args_list)
         resize.assert_called_once_with(3)
         finished_scaling.assert_called_once_with(
+            None,
             'PercentChangeInCapacity : -33',
             size_changed=True)
 
@@ -416,7 +420,8 @@ class TestGroupAdjust(common.HeatTestCase):
 
         self.assertEqual(expected_notifies, notify.call_args_list)
         resize.assert_called_once_with(1)
-        finished_scaling.assert_called_once_with('ChangeInCapacity : 1',
+        finished_scaling.assert_called_once_with(None,
+                                                 'ChangeInCapacity : 1',
                                                  size_changed=True)
         grouputils.get_size.assert_called_once_with(self.group)
 
@@ -722,3 +727,121 @@ class RollingUpdatePolicyDiffTest(common.HeatTestCase):
     def test_update_policy_removed(self):
         self.validate_update_policy_diff(asg_tmpl_with_updt_policy(),
                                          inline_templates.as_template)
+
+
+class TestCooldownMixin(common.HeatTestCase):
+    def setUp(self):
+        super(TestCooldownMixin, self).setUp()
+        t = template_format.parse(inline_templates.as_template)
+        self.stack = utils.parse_stack(t, params=inline_templates.as_params)
+        self.stack.store()
+        self.group = self.stack['WebServerGroup']
+        self.group.state_set('CREATE', 'COMPLETE')
+
+    def test_cooldown_is_in_progress_toosoon(self):
+        cooldown_end = timeutils.utcnow() + datetime.timedelta(seconds=60)
+        previous_meta = {'cooldown_end': {
+            cooldown_end.isoformat(): 'ChangeInCapacity : 1'}}
+        self.patchobject(self.group, 'metadata_get',
+                         return_value=previous_meta)
+        self.assertRaises(resource.NoActionRequired,
+                          self.group._check_scaling_allowed,
+                          60)
+
+    def test_cooldown_is_in_progress_toosoon_legacy(self):
+        now = timeutils.utcnow()
+        previous_meta = {'cooldown': {
+            now.isoformat(): 'ChangeInCapacity : 1'}}
+        self.patchobject(self.group, 'metadata_get',
+                         return_value=previous_meta)
+        self.assertRaises(resource.NoActionRequired,
+                          self.group._check_scaling_allowed,
+                          60)
+
+    def test_cooldown_is_in_progress_scaling_unfinished(self):
+        previous_meta = {'scaling_in_progress': True}
+        self.patchobject(self.group, 'metadata_get',
+                         return_value=previous_meta)
+        self.assertRaises(resource.NoActionRequired,
+                          self.group._check_scaling_allowed,
+                          60)
+
+    def test_scaling_not_in_progress_legacy(self):
+        awhile_ago = timeutils.utcnow() - datetime.timedelta(seconds=100)
+        previous_meta = {
+            'cooldown': {
+                awhile_ago.isoformat(): 'ChangeInCapacity : 1'
+            },
+            'scaling_in_progress': False
+        }
+        self.patchobject(self.group, 'metadata_get',
+                         return_value=previous_meta)
+        self.assertIsNone(self.group._check_scaling_allowed(60))
+
+    def test_scaling_not_in_progress(self):
+        awhile_after = timeutils.utcnow() + datetime.timedelta(seconds=60)
+        previous_meta = {
+            'cooldown_end': {
+                awhile_after.isoformat(): 'ChangeInCapacity : 1'
+            },
+            'scaling_in_progress': False
+        }
+        timeutils.set_time_override()
+        timeutils.advance_time_seconds(100)
+        self.patchobject(self.group, 'metadata_get',
+                         return_value=previous_meta)
+        self.assertIsNone(self.group._check_scaling_allowed(60))
+        timeutils.clear_time_override()
+
+    def test_scaling_policy_cooldown_zero(self):
+        now = timeutils.utcnow()
+        previous_meta = {
+            'cooldown_end': {
+                now.isoformat(): 'ChangeInCapacity : 1'
+            },
+            'scaling_in_progress': False
+        }
+        self.patchobject(self.group, 'metadata_get',
+                         return_value=previous_meta)
+        self.assertIsNone(self.group._check_scaling_allowed(60))
+
+    def test_scaling_policy_cooldown_none(self):
+        now = timeutils.utcnow()
+        previous_meta = {
+            'cooldown_end': {
+                now.isoformat(): 'ChangeInCapacity : 1'
+            },
+            'scaling_in_progress': False
+        }
+        self.patchobject(self.group, 'metadata_get',
+                         return_value=previous_meta)
+        self.assertIsNone(self.group._check_scaling_allowed(None))
+
+    def test_metadata_is_written(self):
+        nowish = timeutils.utcnow()
+        reason = 'cool as'
+        meta_set = self.patchobject(self.group, 'metadata_set')
+        self.patchobject(timeutils, 'utcnow', return_value=nowish)
+        self.group._finished_scaling(60, reason)
+        cooldown_end = nowish + datetime.timedelta(seconds=60)
+        meta_set.assert_called_once_with(
+            {'cooldown_end': {cooldown_end.isoformat(): reason},
+             'scaling_in_progress': False})
+
+    def test_metadata_is_written_update(self):
+        nowish = timeutils.utcnow()
+        reason = 'cool as'
+        prev_cooldown_end = nowish + datetime.timedelta(seconds=100)
+        previous_meta = {
+            'cooldown_end': {
+                prev_cooldown_end.isoformat(): 'ChangeInCapacity : 1'
+            }
+        }
+        self.patchobject(self.group, 'metadata_get',
+                         return_value=previous_meta)
+        meta_set = self.patchobject(self.group, 'metadata_set')
+        self.patchobject(timeutils, 'utcnow', return_value=nowish)
+        self.group._finished_scaling(60, reason)
+        meta_set.assert_called_once_with(
+            {'cooldown_end': {prev_cooldown_end.isoformat(): reason},
+             'scaling_in_progress': False})
