@@ -14,20 +14,20 @@
 import copy
 
 import mock
-import mox
 from neutronclient.v2_0 import client as neutronclient
-from novaclient import exceptions as nova_exceptions
 import six
 
 from heat.common import exception
 from heat.common import short_id
 from heat.common import template_format
 from heat.engine.clients.os import nova
+from heat.engine import node_data
 from heat.engine import resource
 from heat.engine.resources.aws.ec2 import eip
 from heat.engine import rsrc_defn
 from heat.engine import scheduler
 from heat.engine import stack as parser
+from heat.engine import stk_defn
 from heat.engine import template as tmpl
 from heat.tests import common
 from heat.tests.openstack.nova import fakes as fakes_nova
@@ -172,7 +172,54 @@ class EIPTest(common.HeatTestCase):
         super(EIPTest, self).setUp()
         self.fc = fakes_nova.FakeClient()
         self.m.StubOutWithMock(nova.NovaClientPlugin, '_create')
+        self.m.StubOutWithMock(neutronclient.Client, 'list_networks')
         self.m.StubOutWithMock(self.fc.servers, 'get')
+        self.m.StubOutWithMock(neutronclient.Client,
+                               'create_floatingip')
+        self.m.StubOutWithMock(neutronclient.Client,
+                               'show_floatingip')
+        self.m.StubOutWithMock(neutronclient.Client,
+                               'update_floatingip')
+        self.m.StubOutWithMock(neutronclient.Client,
+                               'delete_floatingip')
+
+    def mock_create_floatingip(self):
+        neutronclient.Client.list_networks(
+            **{'router:external': True}).AndReturn({'networks': [{
+                'status': 'ACTIVE',
+                'subnets': [],
+                'name': 'nova',
+                'router:external': True,
+                'tenant_id': 'c1210485b2424d48804aad5d39c61b8f',
+                'admin_state_up': True,
+                'shared': True,
+                'id': 'eeee'
+            }]})
+
+        neutronclient.Client.create_floatingip({
+            'floatingip': {'floating_network_id': u'eeee'}
+        }).AndReturn({'floatingip': {
+            "status": "ACTIVE",
+            "id": "fc68ea2c-b60b-4b4f-bd82-94ec81110766",
+            "floating_ip_address": "11.0.0.1"
+        }})
+
+    def mock_show_floatingip(self, refid):
+        neutronclient.Client.show_floatingip(
+            refid,
+        ).AndReturn({'floatingip': {
+            'router_id': None,
+            'tenant_id': 'e936e6cd3e0b48dcb9ff853a8f253257',
+            'floating_network_id': 'eeee',
+            'fixed_ip_address': None,
+            'floating_ip_address': '11.0.0.1',
+            'port_id': None,
+            'id': 'ffff'
+        }})
+
+    def mock_delete_floatingip(self):
+        id = 'fc68ea2c-b60b-4b4f-bd82-94ec81110766'
+        neutronclient.Client.delete_floatingip(id).AndReturn(None)
 
     def create_eip(self, t, stack, resource_name):
         resource_defns = stack.t.resource_definitions(stack)
@@ -182,6 +229,8 @@ class EIPTest(common.HeatTestCase):
         self.assertIsNone(rsrc.validate())
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
+        stk_defn.update_resource_data(stack.defn, resource_name,
+                                      rsrc.node_data())
         return rsrc
 
     def create_association(self, t, stack, resource_name):
@@ -192,6 +241,8 @@ class EIPTest(common.HeatTestCase):
         self.assertIsNone(rsrc.validate())
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
+        stk_defn.update_resource_data(stack.defn, resource_name,
+                                      rsrc.node_data())
         return rsrc
 
     def _mock_server_get(self, server='WebServer', mock_server=None,
@@ -208,7 +259,8 @@ class EIPTest(common.HeatTestCase):
         mock_server = self.fc.servers.list()[0]
         self._mock_server_get(mock_server=mock_server)
         self._mock_server_get(mock_again=True)
-
+        self.mock_create_floatingip()
+        self.mock_delete_floatingip()
         self.m.ReplayAll()
 
         t = template_format.parse(eip_template)
@@ -221,7 +273,8 @@ class EIPTest(common.HeatTestCase):
             rsrc.refid = None
             self.assertEqual('11.0.0.1', rsrc.FnGetRefId())
 
-            self.assertEqual('1', rsrc.FnGetAtt('AllocationId'))
+            self.assertEqual('fc68ea2c-b60b-4b4f-bd82-94ec81110766',
+                             rsrc.FnGetAtt('AllocationId'))
 
             self.assertRaises(exception.InvalidTemplateAttribute,
                               rsrc.FnGetAtt, 'Foo')
@@ -238,7 +291,7 @@ class EIPTest(common.HeatTestCase):
         server_update = self.fc.servers.list()[1]
         self._mock_server_get(server='5678', mock_server=server_update,
                               multiple=True, mock_again=True)
-
+        self.mock_create_floatingip()
         self.m.ReplayAll()
         t = template_format.parse(eip_template)
         stack = utils.parse_stack(t)
@@ -267,6 +320,9 @@ class EIPTest(common.HeatTestCase):
         server = self.fc.servers.list()[0]
         self._mock_server_get(mock_server=server, multiple=True)
 
+        self.mock_create_floatingip()
+        self.mock_delete_floatingip()
+        self.mock_show_floatingip('fc68ea2c-b60b-4b4f-bd82-94ec81110766')
         self.m.ReplayAll()
 
         t = template_format.parse(eip_template_ipassoc)
@@ -294,9 +350,19 @@ class EIPTest(common.HeatTestCase):
         self.m.VerifyAll()
 
     def test_eip_with_exception(self):
-        self.m.StubOutWithMock(self.fc.floating_ips, 'create')
-        nova.NovaClientPlugin._create().AndReturn(self.fc)
-        self.fc.floating_ips.create().AndRaise(fakes_nova.fake_exception())
+        neutronclient.Client.list_networks(
+            **{'router:external': True}).AndReturn({'networks': [{
+                'status': 'ACTIVE',
+                'subnets': [],
+                'name': 'nova',
+                'router:external': True,
+                'tenant_id': 'c1210485b2424d48804aad5d39c61b8f',
+                'admin_state_up': True,
+                'shared': True,
+                'id': 'eeee'
+            }]})
+        self.patchobject(neutronclient.Client, 'create_floatingip',
+                         side_effect=neutronclient.exceptions.NotFound)
         self.m.ReplayAll()
 
         t = template_format.parse(eip_template)
@@ -307,74 +373,10 @@ class EIPTest(common.HeatTestCase):
                              resource_defns[resource_name],
                              stack)
 
-        self.assertRaises(nova_exceptions.NotFound,
+        self.assertRaises(neutronclient.exceptions.NotFound,
                           rsrc.handle_create)
         self.m.VerifyAll()
 
-    def test_delete_eip_with_exception(self):
-        self.m.StubOutWithMock(self.fc.floating_ips, 'delete')
-        nova.NovaClientPlugin._create().AndReturn(self.fc)
-        self.fc.floating_ips.delete(mox.IsA(object)).AndRaise(
-            fakes_nova.fake_exception())
-        self.fc.servers.get(mox.IsA(object)).AndReturn(False)
-        self.m.ReplayAll()
-
-        t = template_format.parse(eip_template)
-        stack = utils.parse_stack(t)
-        resource_name = 'IPAddress'
-        resource_defns = stack.t.resource_definitions(stack)
-        rsrc = eip.ElasticIp(resource_name,
-                             resource_defns[resource_name],
-                             stack)
-        rsrc.resource_id = 'fake_id'
-        rsrc.handle_delete()
-        self.m.VerifyAll()
-
-    def test_delete_eip_successful_if_eip_associate_failed(self):
-        floating_ip = mox.IsA(object)
-        floating_ip.ip = '172.24.4.13'
-        floating_ip.id = '9037272b-6875-42e6-82e9-4342d5925da4'
-
-        self.m.StubOutWithMock(self.fc.floating_ips, 'create')
-        self.fc.floating_ips.create().AndReturn(floating_ip)
-
-        server = self.fc.servers.list()[0]
-        self._mock_server_get(mock_server=server, multiple=True)
-
-        self.m.StubOutWithMock(self.fc.servers, 'add_floating_ip')
-        self.fc.servers.add_floating_ip(
-            server, floating_ip.ip, None
-        ).AndRaise(nova_exceptions.BadRequest(400))
-
-        self.m.StubOutWithMock(self.fc.servers, 'remove_floating_ip')
-        msg = ("ClientException: Floating ip 172.24.4.13 is not associated "
-               "with instance 1234.")
-        self.fc.servers.remove_floating_ip(
-            server, floating_ip.ip
-        ).AndRaise(nova_exceptions.ClientException(422, msg))
-        self.m.StubOutWithMock(self.fc.floating_ips, 'delete')
-        self.fc.floating_ips.delete(mox.IsA(object))
-
-        self.m.ReplayAll()
-
-        t = template_format.parse(eip_template)
-        stack = utils.parse_stack(t)
-        resource_name = 'IPAddress'
-        resource_defns = stack.t.resource_definitions(stack)
-        rsrc = eip.ElasticIp(resource_name,
-                             resource_defns[resource_name],
-                             stack)
-
-        self.assertIsNone(rsrc.validate())
-        self.assertRaises(exception.ResourceFailure,
-                          scheduler.TaskRunner(rsrc.create))
-        self.assertEqual((rsrc.CREATE, rsrc.FAILED), rsrc.state)
-
-        # to delete the eip
-        scheduler.TaskRunner(rsrc.delete)()
-        self.assertEqual((rsrc.DELETE, rsrc.COMPLETE), rsrc.state)
-        self.m.VerifyAll()
-
     @mock.patch.object(eip.ElasticIp, '_ipaddress')
     def test_FnGetRefId_resource_name(self, mock_ipaddr):
         t = template_format.parse(ipassoc_template_validate)
@@ -396,14 +398,14 @@ class EIPTest(common.HeatTestCase):
         template = tmpl.Template(t)
         stack = parser.Stack(utils.dummy_context(), 'test', template,
                              cache_data={
-                                 'eip': {
+                                 'eip': node_data.NodeData.from_dict({
                                      'uuid': mock.ANY,
                                      'id': mock.ANY,
                                      'action': 'CREATE',
                                      'status': 'COMPLETE',
-                                     'reference_id': '1.1.1.1'}})
+                                     'reference_id': '1.1.1.1'})})
 
-        rsrc = stack['eip']
+        rsrc = stack.defn['eip']
         self.assertEqual('1.1.1.1', rsrc.FnGetRefId())
 
 
@@ -478,23 +480,26 @@ class AllocTest(common.HeatTestCase):
             self.fc.servers.get(server).AndReturn(mock_server)
 
     def create_eip(self, t, stack, resource_name):
-        resource_defns = stack.t.resource_definitions(stack)
         rsrc = eip.ElasticIp(resource_name,
-                             resource_defns[resource_name],
+                             stack.defn.resource_definition(resource_name),
                              stack)
         self.assertIsNone(rsrc.validate())
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
+        stk_defn.update_resource_data(stack.defn, resource_name,
+                                      rsrc.node_data())
         return rsrc
 
     def create_association(self, t, stack, resource_name):
-        resource_defns = stack.t.resource_definitions(stack)
+        resource_defn = stack.defn.resource_definition(resource_name)
         rsrc = eip.ElasticIpAssociation(resource_name,
-                                        resource_defns[resource_name],
+                                        resource_defn,
                                         stack)
         self.assertIsNone(rsrc.validate())
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
+        stk_defn.update_resource_data(stack.defn, resource_name,
+                                      rsrc.node_data())
         return rsrc
 
     def mock_update_floatingip(self, port='the_nic'):
@@ -524,7 +529,7 @@ class AllocTest(common.HeatTestCase):
         }).AndReturn({'floatingip': {
             "status": "ACTIVE",
             "id": "fc68ea2c-b60b-4b4f-bd82-94ec81110766",
-            "floating_ip_address": "192.168.9.3"
+            "floating_ip_address": "11.0.0.1"
         }})
 
     def mock_show_floatingip(self, refid):
@@ -535,7 +540,7 @@ class AllocTest(common.HeatTestCase):
             'tenant_id': 'e936e6cd3e0b48dcb9ff853a8f253257',
             'floating_network_id': 'eeee',
             'fixed_ip_address': None,
-            'floating_ip_address': '172.24.4.227',
+            'floating_ip_address': '11.0.0.1',
             'port_id': None,
             'id': 'ffff'
         }})
@@ -606,31 +611,6 @@ class AllocTest(common.HeatTestCase):
             "routers": []
         })
 
-    def test_neutron_eip(self):
-        mock_server = self.fc.servers.list()[0]
-        self._mock_server_get(mock_server=mock_server)
-        self._mock_server_get(mock_again=True)
-        self.m.ReplayAll()
-
-        t = template_format.parse(eip_template)
-        stack = utils.parse_stack(t)
-
-        rsrc = self.create_eip(t, stack, 'IPAddress')
-
-        try:
-            self.assertEqual('11.0.0.1', rsrc.FnGetRefId())
-            rsrc.refid = None
-            self.assertEqual('11.0.0.1', rsrc.FnGetRefId())
-
-            self.assertEqual('1', rsrc.FnGetAtt('AllocationId'))
-
-            self.assertRaises(exception.InvalidTemplateAttribute,
-                              rsrc.FnGetAtt, 'Foo')
-        finally:
-            scheduler.TaskRunner(rsrc.destroy)()
-
-        self.m.VerifyAll()
-
     def test_association_allocationid(self):
         self.mock_create_gateway_attachment()
         self.mock_show_network()
@@ -752,6 +732,8 @@ class AllocTest(common.HeatTestCase):
         self.m.StubOutWithMock(self.fc.servers, 'add_floating_ip')
         self.fc.servers.add_floating_ip(server, '11.0.0.1').AndRaise(
             fakes_nova.fake_exception(400))
+        self.mock_create_floatingip()
+        self.mock_show_floatingip('fc68ea2c-b60b-4b4f-bd82-94ec81110766')
         self.m.ReplayAll()
 
         t = template_format.parse(eip_template_ipassoc)
@@ -782,6 +764,7 @@ class AllocTest(common.HeatTestCase):
                               multiple=True,
                               mock_again=True)
 
+        self.mock_create_floatingip()
         self.m.ReplayAll()
 
         t = template_format.parse(eip_template_ipassoc)
@@ -795,7 +778,7 @@ class AllocTest(common.HeatTestCase):
         update_server_id = '5678'
         props['InstanceId'] = update_server_id
         update_snippet = rsrc_defn.ResourceDefinition(ass.name, ass.type(),
-                                                      stack.t.parse(stack,
+                                                      stack.t.parse(stack.defn,
                                                                     props))
         scheduler.TaskRunner(ass.update, update_snippet)()
         self.assertEqual((ass.UPDATE, ass.COMPLETE), ass.state)
@@ -805,7 +788,7 @@ class AllocTest(common.HeatTestCase):
     def test_update_association_with_EIP(self):
         server = self.fc.servers.list()[0]
         self._mock_server_get(mock_server=server, multiple=True)
-
+        self.mock_create_floatingip()
         self.m.ReplayAll()
 
         t = template_format.parse(eip_template_ipassoc)
@@ -818,7 +801,7 @@ class AllocTest(common.HeatTestCase):
         update_eip = '11.0.0.2'
         props['EIP'] = update_eip
         update_snippet = rsrc_defn.ResourceDefinition(ass.name, ass.type(),
-                                                      stack.t.parse(stack,
+                                                      stack.t.parse(stack.defn,
                                                                     props))
         scheduler.TaskRunner(ass.update, update_snippet)()
         self.assertEqual((ass.UPDATE, ass.COMPLETE), ass.state)
@@ -828,6 +811,7 @@ class AllocTest(common.HeatTestCase):
     def test_update_association_with_AllocationId_or_EIP(self):
         server = self.fc.servers.list()[0]
         self._mock_server_get(mock_server=server, multiple=True)
+        self.mock_create_floatingip()
 
         self.mock_list_instance_ports('WebServer')
         self.mock_show_network()
@@ -850,10 +834,11 @@ class AllocTest(common.HeatTestCase):
         props['AllocationId'] = update_allocationId
         props.pop('EIP')
         update_snippet = rsrc_defn.ResourceDefinition(ass.name, ass.type(),
-                                                      stack.t.parse(stack,
+                                                      stack.t.parse(stack.defn,
                                                                     props))
         scheduler.TaskRunner(ass.update, update_snippet)()
         self.assertEqual((ass.UPDATE, ass.COMPLETE), ass.state)
+        stk_defn.update_resource_data(stack.defn, ass.name, ass.node_data())
 
         # change AllocationId to EIP
         props = copy.deepcopy(ass.properties.data)
@@ -861,7 +846,7 @@ class AllocTest(common.HeatTestCase):
         props['EIP'] = update_eip
         props.pop('AllocationId')
         update_snippet = rsrc_defn.ResourceDefinition(ass.name, ass.type(),
-                                                      stack.t.parse(stack,
+                                                      stack.t.parse(stack.defn,
                                                                     props))
         scheduler.TaskRunner(ass.update, update_snippet)()
         self.assertEqual((ass.UPDATE, ass.COMPLETE), ass.state)
@@ -871,6 +856,8 @@ class AllocTest(common.HeatTestCase):
     def test_update_association_needs_update_InstanceId(self):
         server = self.fc.servers.list()[0]
         self._mock_server_get(mock_server=server, multiple=True)
+        self.mock_create_floatingip()
+        self.mock_show_floatingip('fc68ea2c-b60b-4b4f-bd82-94ec81110766')
 
         server_update = self.fc.servers.list()[1]
         self._mock_server_get(server='5678',
@@ -897,6 +884,8 @@ class AllocTest(common.HeatTestCase):
     def test_update_association_needs_update_InstanceId_EIP(self):
         server = self.fc.servers.list()[0]
         self._mock_server_get(mock_server=server, multiple=True)
+        self.mock_create_floatingip()
+        self.mock_show_floatingip('fc68ea2c-b60b-4b4f-bd82-94ec81110766')
 
         server_update = self.fc.servers.list()[1]
         self._mock_server_get(server='5678',
@@ -951,7 +940,7 @@ class AllocTest(common.HeatTestCase):
         props['NetworkInterfaceId'] = update_networkInterfaceId
 
         update_snippet = rsrc_defn.ResourceDefinition(ass.name, ass.type(),
-                                                      stack.t.parse(stack,
+                                                      stack.t.parse(stack.defn,
                                                                     props))
         scheduler.TaskRunner(ass.update, update_snippet)()
         self.assertEqual((ass.UPDATE, ass.COMPLETE), ass.state)
@@ -963,7 +952,7 @@ class AllocTest(common.HeatTestCase):
         props['InstanceId'] = instance_id
 
         update_snippet = rsrc_defn.ResourceDefinition(ass.name, ass.type(),
-                                                      stack.t.parse(stack,
+                                                      stack.t.parse(stack.defn,
                                                                     props))
         scheduler.TaskRunner(ass.update, update_snippet)()
         self.assertEqual((ass.UPDATE, ass.COMPLETE), ass.state)
@@ -991,13 +980,13 @@ class AllocTest(common.HeatTestCase):
 
     def test_eip_allocation_refid_convergence_cache_data(self):
         t = template_format.parse(eip_template_ipassoc)
-        cache_data = {'IPAssoc': {
+        cache_data = {'IPAssoc': node_data.NodeData.from_dict({
             'uuid': mock.ANY,
             'id': mock.ANY,
             'action': 'CREATE',
             'status': 'COMPLETE',
             'reference_id': 'convg_xyz'
-        }}
+        })}
         stack = utils.parse_stack(t, cache_data=cache_data)
-        rsrc = stack['IPAssoc']
+        rsrc = stack.defn['IPAssoc']
         self.assertEqual('convg_xyz', rsrc.FnGetRefId())
