@@ -27,6 +27,7 @@ from heat.engine.clients.os import neutron
 from heat.engine.clients.os import nova
 from heat.engine.clients import progress
 from heat.engine import environment
+from heat.engine import resource
 from heat.engine.resources.aws.ec2 import instance as instances
 from heat.engine.resources import scheduler_hints as sh
 from heat.engine import scheduler
@@ -124,6 +125,7 @@ class InstancesTest(common.HeatTestCase):
         stack_name = '%s_s' % name
         tmpl, self.stack = self._get_test_template(stack_name, image_id,
                                                    volumes=volumes)
+        self.instance_props = tmpl.t['Resources']['WebServer']['Properties']
         resource_defns = tmpl.resource_definitions(self.stack)
         instance = instances.Instance(name, resource_defns['WebServer'],
                                       self.stack)
@@ -173,7 +175,7 @@ class InstancesTest(common.HeatTestCase):
         instance = self._create_test_instance(return_server,
                                               'in_create')
         # this makes sure the auto increment worked on instance creation
-        self.assertTrue(instance.id > 0)
+        self.assertGreater(instance.id, 0)
 
         expected_ip = return_server.networks['public'][0]
         expected_az = getattr(return_server, 'OS-EXT-AZ:availability_zone')
@@ -195,7 +197,7 @@ class InstancesTest(common.HeatTestCase):
         instance = self._create_test_instance(return_server,
                                               'create_with_bdm')
         # this makes sure the auto increment worked on instance creation
-        self.assertTrue(instance.id > 0)
+        self.assertGreater(instance.id, 0)
 
         expected_ip = return_server.networks['public'][0]
         expected_az = getattr(return_server, 'OS-EXT-AZ:availability_zone')
@@ -373,7 +375,7 @@ class InstancesTest(common.HeatTestCase):
         scheduler.TaskRunner(instance.create)()
 
         # this makes sure the auto increment worked on instance creation
-        self.assertTrue(instance.id > 0)
+        self.assertGreater(instance.id, 0)
 
         expected_ip = return_server.networks['public'][0]
         expected_az = getattr(return_server, 'OS-EXT-AZ:availability_zone')
@@ -398,6 +400,19 @@ class InstancesTest(common.HeatTestCase):
         self.assertEqual(expected_az, actual_az)
         self.m.VerifyAll()
 
+    def test_instance_create_resolve_az_attribute_nova_az_ext_disabled(self):
+        return_server = self.fc.servers.list()[1]
+        delattr(return_server, 'OS-EXT-AZ:availability_zone')
+        instance = self._setup_test_instance(return_server,
+                                             'create_resolve_az_attribute')
+        self.patchobject(self.fc.servers, 'get',
+                         return_value=return_server)
+        self.m.ReplayAll()
+        scheduler.TaskRunner(instance.create)()
+
+        self.assertIsNone(instance._availability_zone())
+        self.m.VerifyAll()
+
     def test_instance_create_image_name_err(self):
         stack_name = 'test_instance_create_image_name_err_stack'
         (tmpl, stack) = self._setup_test_stack(stack_name)
@@ -407,9 +422,10 @@ class InstancesTest(common.HeatTestCase):
         resource_defns = tmpl.resource_definitions(stack)
         instance = instances.Instance('instance_create_image_err',
                                       resource_defns['WebServer'], stack)
-
-        self._mock_get_image_id_fail('Slackware',
-                                     glance.exceptions.NotFound())
+        self._mock_get_image_id_fail(
+            'Slackware',
+            glance.client_exception.EntityMatchNotFound(
+                entity='image', args='Slackware'))
         self.stub_VolumeConstraint_validate()
         self.stub_FlavorConstraint_validate()
         self.stub_KeypairConstraint_validate()
@@ -421,7 +437,7 @@ class InstancesTest(common.HeatTestCase):
         self.assertEqual(
             "StackValidationFailed: resources.instance_create_image_err: "
             "Property error: WebServer.Properties.ImageId: "
-            "Error validating value 'Slackware': Not Found (HTTP 404)",
+            "Error validating value 'Slackware': No image matching Slackware.",
             six.text_type(error))
 
         self.m.VerifyAll()
@@ -437,8 +453,10 @@ class InstancesTest(common.HeatTestCase):
         instance = instances.Instance('instance_create_image_err',
                                       resource_defns['WebServer'], stack)
 
-        self._mock_get_image_id_fail('CentOS 5.2',
-                                     glance.exceptions.NoUniqueMatch())
+        self._mock_get_image_id_fail(
+            'CentOS 5.2',
+            glance.client_exception.EntityUniqueMatchNotFound(
+                entity='image', args='CentOS 5.2'))
 
         self.stub_KeypairConstraint_validate()
         self.stub_SnapshotConstraint_validate()
@@ -451,7 +469,8 @@ class InstancesTest(common.HeatTestCase):
         self.assertEqual(
             "StackValidationFailed: resources.instance_create_image_err: "
             "Property error: WebServer.Properties.ImageId: "
-            "Error validating value 'CentOS 5.2': ",
+            "Error validating value 'CentOS 5.2': No image unique match "
+            "found for CentOS 5.2.",
             six.text_type(error))
 
         self.m.VerifyAll()
@@ -466,7 +485,9 @@ class InstancesTest(common.HeatTestCase):
         instance = instances.Instance('instance_create_image_err',
                                       resource_defns['WebServer'], stack)
 
-        self._mock_get_image_id_fail('1', glance.exceptions.NotFound())
+        self._mock_get_image_id_fail(
+            '1', glance.client_exception.EntityMatchNotFound(entity='image',
+                                                             args='1'))
 
         self.stub_VolumeConstraint_validate()
         self.stub_FlavorConstraint_validate()
@@ -479,7 +500,7 @@ class InstancesTest(common.HeatTestCase):
         self.assertEqual(
             "StackValidationFailed: resources.instance_create_image_err: "
             "Property error: WebServer.Properties.ImageId: "
-            "Error validating value '1': Not Found (HTTP 404)",
+            "Error validating value '1': No image matching 1.",
             six.text_type(error))
 
         self.m.VerifyAll()
@@ -611,7 +632,7 @@ class InstancesTest(common.HeatTestCase):
             scheduler_hints={shm.HEAT_ROOT_STACK_ID: stack.root_stack_id(),
                              shm.HEAT_STACK_ID: stack.id,
                              shm.HEAT_STACK_NAME: stack.name,
-                             shm.HEAT_PATH_IN_STACK: [(None, stack.name)],
+                             shm.HEAT_PATH_IN_STACK: [stack.name],
                              shm.HEAT_RESOURCE_NAME: instance.name,
                              shm.HEAT_RESOURCE_UUID: instance.uuid,
                              'foo': ['spam', 'ham', 'baz'], 'bar': 'eggs'},
@@ -620,7 +641,7 @@ class InstancesTest(common.HeatTestCase):
                 return_server)
         self.m.ReplayAll()
         scheduler.TaskRunner(instance.create)()
-        self.assertTrue(instance.id > 0)
+        self.assertGreater(instance.id, 0)
         self.m.VerifyAll()
 
     def test_instance_validate(self):
@@ -653,7 +674,7 @@ class InstancesTest(common.HeatTestCase):
         instance.status = vm_status
 
         # this makes sure the auto increment worked on instance creation
-        self.assertTrue(instance.id > 0)
+        self.assertGreater(instance.id, 0)
 
         d1 = {'server': self.fc.client.get_servers_detail()[1]['servers'][0]}
         d1['server']['status'] = vm_status
@@ -688,7 +709,7 @@ class InstancesTest(common.HeatTestCase):
         instance.resource_id = '1234'
 
         # this makes sure the auto increment worked on instance creation
-        self.assertTrue(instance.id > 0)
+        self.assertGreater(instance.id, 0)
 
         self.m.StubOutWithMock(self.fc.client, 'delete_servers_1234')
         self.fc.client.delete_servers_1234().AndRaise(
@@ -722,8 +743,9 @@ class InstancesTest(common.HeatTestCase):
         instance = self._create_test_instance(return_server,
                                               'ud_type')
 
-        update_template = copy.deepcopy(instance.t)
-        update_template['Properties']['InstanceType'] = 'm1.small'
+        update_props = self.instance_props.copy()
+        update_props['InstanceType'] = 'm1.small'
+        update_template = instance.t.freeze(properties=update_props)
 
         def side_effect(*args):
             return 2 if args[0] == 'm1.small' else 1
@@ -785,8 +807,9 @@ class InstancesTest(common.HeatTestCase):
         self.patchobject(glance.GlanceClientPlugin, 'find_image_by_name_or_id',
                          return_value=1)
 
-        update_template = copy.deepcopy(instance.t)
-        update_template['Properties']['InstanceType'] = 'm1.small'
+        update_props = self.instance_props.copy()
+        update_props['InstanceType'] = 'm1.small'
+        update_template = instance.t.freeze(properties=update_props)
 
         self.m.StubOutWithMock(self.fc.servers, 'get')
 
@@ -849,9 +872,12 @@ class InstancesTest(common.HeatTestCase):
             {'NetworkInterfaceId': '34b752ec-14de-416a-8722-9531015e04a5',
              'DeviceIndex': '3'}]
 
-        instance.t['Properties']['NetworkInterfaces'] = old_interfaces
-        update_template = copy.deepcopy(instance.t)
-        update_template['Properties']['NetworkInterfaces'] = new_interfaces
+        before_props = self.instance_props.copy()
+        before_props['NetworkInterfaces'] = old_interfaces
+        update_props = self.instance_props.copy()
+        update_props['NetworkInterfaces'] = new_interfaces
+        after = instance.t.freeze(properties=update_props)
+        before = instance.t.freeze(properties=before_props)
 
         self.m.StubOutWithMock(self.fc.servers, 'get')
         self.fc.servers.get('1234').MultipleTimes().AndReturn(return_server)
@@ -863,7 +889,7 @@ class InstancesTest(common.HeatTestCase):
                                        None, None).AndReturn(None)
         self.m.ReplayAll()
 
-        scheduler.TaskRunner(instance.update, update_template)()
+        scheduler.TaskRunner(instance.update, after, before)()
         self.assertEqual((instance.UPDATE, instance.COMPLETE), instance.state)
         self.m.VerifyAll()
 
@@ -888,9 +914,12 @@ class InstancesTest(common.HeatTestCase):
             {'NetworkInterfaceId': 'ea29f957-cd35-4364-98fb-57ce9732c10d',
              'DeviceIndex': '2'}]
 
-        instance.t['Properties']['NetworkInterfaces'] = old_interfaces
-        update_template = copy.deepcopy(instance.t)
-        update_template['Properties']['NetworkInterfaces'] = new_interfaces
+        before_props = self.instance_props.copy()
+        before_props['NetworkInterfaces'] = old_interfaces
+        update_props = self.instance_props.copy()
+        update_props['NetworkInterfaces'] = new_interfaces
+        after = instance.t.freeze(properties=update_props)
+        before = instance.t.freeze(properties=before_props)
 
         self.m.StubOutWithMock(self.fc.servers, 'get')
         self.fc.servers.get('1234').MultipleTimes().AndReturn(return_server)
@@ -900,7 +929,7 @@ class InstancesTest(common.HeatTestCase):
 
         self.m.ReplayAll()
 
-        scheduler.TaskRunner(instance.update, update_template)()
+        scheduler.TaskRunner(instance.update, after, before)()
         self.assertEqual((instance.UPDATE, instance.COMPLETE), instance.state)
 
     def test_instance_update_network_interfaces_new_include_old(self):
@@ -924,9 +953,12 @@ class InstancesTest(common.HeatTestCase):
             {'NetworkInterfaceId': 'd1e9c73c-04fe-4e9e-983c-d5ef94cd1a46',
              'DeviceIndex': '1'}]
 
-        instance.t['Properties']['NetworkInterfaces'] = old_interfaces
-        update_template = copy.deepcopy(instance.t)
-        update_template['Properties']['NetworkInterfaces'] = new_interfaces
+        before_props = self.instance_props.copy()
+        before_props['NetworkInterfaces'] = old_interfaces
+        update_props = self.instance_props.copy()
+        update_props['NetworkInterfaces'] = new_interfaces
+        after = instance.t.freeze(properties=update_props)
+        before = instance.t.freeze(properties=before_props)
 
         self.m.StubOutWithMock(self.fc.servers, 'get')
         self.fc.servers.get('1234').MultipleTimes().AndReturn(return_server)
@@ -936,7 +968,7 @@ class InstancesTest(common.HeatTestCase):
 
         self.m.ReplayAll()
 
-        scheduler.TaskRunner(instance.update, update_template)()
+        scheduler.TaskRunner(instance.update, after, before)()
         self.assertEqual((instance.UPDATE, instance.COMPLETE), instance.state)
 
     def test_instance_update_network_interfaces_new_old_all_different(self):
@@ -960,9 +992,12 @@ class InstancesTest(common.HeatTestCase):
             {'NetworkInterfaceId': 'd1e9c73c-04fe-4e9e-983c-d5ef94cd1a46',
              'DeviceIndex': '1'}]
 
-        instance.t['Properties']['NetworkInterfaces'] = old_interfaces
-        update_template = copy.deepcopy(instance.t)
-        update_template['Properties']['NetworkInterfaces'] = new_interfaces
+        before_props = self.instance_props.copy()
+        before_props['NetworkInterfaces'] = old_interfaces
+        update_props = self.instance_props.copy()
+        update_props['NetworkInterfaces'] = new_interfaces
+        after = instance.t.freeze(properties=update_props)
+        before = instance.t.freeze(properties=before_props)
 
         self.m.StubOutWithMock(self.fc.servers, 'get')
         self.fc.servers.get('1234').MultipleTimes().AndReturn(return_server)
@@ -977,7 +1012,7 @@ class InstancesTest(common.HeatTestCase):
 
         self.m.ReplayAll()
 
-        scheduler.TaskRunner(instance.update, update_template)()
+        scheduler.TaskRunner(instance.update, after, before)()
         self.assertEqual((instance.UPDATE, instance.COMPLETE), instance.state)
 
     def test_instance_update_network_interfaces_no_old(self):
@@ -1000,8 +1035,9 @@ class InstancesTest(common.HeatTestCase):
                                        'c4485ba1-283a-4f5f-8868-0cd46cdda52f',
                                        '10.0.0.4')
 
-        update_template = copy.deepcopy(instance.t)
-        update_template['Properties']['NetworkInterfaces'] = new_interfaces
+        update_props = self.instance_props.copy()
+        update_props['NetworkInterfaces'] = new_interfaces
+        update_template = instance.t.freeze(properties=update_props)
 
         self.m.StubOutWithMock(self.fc.servers, 'get')
         self.fc.servers.get('1234').MultipleTimes().AndReturn(return_server)
@@ -1036,8 +1072,9 @@ class InstancesTest(common.HeatTestCase):
                                        'c4485ba1-283a-4f5f-8868-0cd46cdda52f',
                                        '10.0.0.4')
 
-        update_template = copy.deepcopy(instance.t)
-        update_template['Properties']['NetworkInterfaces'] = []
+        update_props = self.instance_props.copy()
+        update_props['NetworkInterfaces'] = []
+        update_template = instance.t.freeze(properties=update_props)
 
         self.m.StubOutWithMock(self.fc.servers, 'get')
         self.fc.servers.get('1234').MultipleTimes().AndReturn(return_server)
@@ -1068,12 +1105,20 @@ class InstancesTest(common.HeatTestCase):
                                        '10.0.0.4')
         subnet_id = '8c1aaddf-e49e-4f28-93ea-ca9f0b3c6240'
         nics = [{'port-id': 'ea29f957-cd35-4364-98fb-57ce9732c10d'}]
+
+        before_props = self.instance_props.copy()
         if old_interfaces is not None:
-            instance.t['Properties']['NetworkInterfaces'] = old_interfaces
-        update_template = copy.deepcopy(instance.t)
+            before_props['NetworkInterfaces'] = old_interfaces
+        update_props = copy.deepcopy(before_props)
+
         if new_interfaces is not None:
-            update_template['Properties']['NetworkInterfaces'] = new_interfaces
-        update_template['Properties']['SubnetId'] = subnet_id
+            update_props['NetworkInterfaces'] = new_interfaces
+        update_props['SubnetId'] = subnet_id
+
+        after = instance.t.freeze(properties=update_props)
+        before = instance.t.freeze(properties=before_props)
+
+        instance.reparse()
 
         self.m.StubOutWithMock(self.fc.servers, 'get')
 
@@ -1098,7 +1143,7 @@ class InstancesTest(common.HeatTestCase):
 
         self.m.ReplayAll()
 
-        scheduler.TaskRunner(instance.update, update_template)()
+        scheduler.TaskRunner(instance.update, after, before)()
         self.assertEqual((instance.UPDATE, instance.COMPLETE), instance.state)
         self.m.VerifyAll()
 
@@ -1132,10 +1177,11 @@ class InstancesTest(common.HeatTestCase):
         self.stub_ImageConstraint_validate()
         self.m.ReplayAll()
 
-        update_template = copy.deepcopy(instance.t)
-        update_template['Properties']['ImageId'] = 'mustreplace'
+        update_props = self.instance_props.copy()
+        update_props['ImageId'] = 'mustreplace'
+        update_template = instance.t.freeze(properties=update_props)
         updater = scheduler.TaskRunner(instance.update, update_template)
-        self.assertRaises(exception.UpdateReplace, updater)
+        self.assertRaises(resource.UpdateReplace, updater)
 
         self.m.VerifyAll()
 
