@@ -12,7 +12,6 @@
 #    under the License.
 
 import json
-import mock
 import uuid
 
 from keystoneauth1 import access as ks_access
@@ -30,7 +29,6 @@ import six
 
 from heat.common import config
 from heat.common import exception
-from heat.common import password_gen
 from heat.engine.clients.os.keystone import heat_keystoneclient
 from heat.tests import common
 from heat.tests import utils
@@ -57,13 +55,10 @@ class KeystoneClientTest(common.HeatTestCase):
         self.m.StubOutWithMock(ks_loading, 'load_auth_from_conf_options')
 
         cfg.CONF.set_override('auth_uri', 'http://server.test:5000/v2.0',
-                              group='keystone_authtoken', enforce_type=True)
-        cfg.CONF.set_override('stack_user_domain_id', 'adomain123',
-                              enforce_type=True)
-        cfg.CONF.set_override('stack_domain_admin', 'adminuser123',
-                              enforce_type=True)
-        cfg.CONF.set_override('stack_domain_admin_password', 'adminsecret',
-                              enforce_type=True)
+                              group='keystone_authtoken')
+        cfg.CONF.set_override('stack_user_domain_id', 'adomain123')
+        cfg.CONF.set_override('stack_domain_admin', 'adminuser123')
+        cfg.CONF.set_override('stack_domain_admin_password', 'adminsecret')
 
         self.addCleanup(self.m.VerifyAll)
 
@@ -109,7 +104,6 @@ class KeystoneClientTest(common.HeatTestCase):
         mock_auth_ref = self.m.CreateMockAnything()
         mock_ks_auth = self.m.CreateMockAnything()
 
-        self.patchobject(mock_ks_auth, 'get_auth_ref')
         if method == 'token':
             p = ks_token_endpoint.Token(token='abcd1234',
                                         endpoint='http://server.test:5000/v3')
@@ -159,22 +153,22 @@ class KeystoneClientTest(common.HeatTestCase):
         return mock_ks_auth, mock_auth_ref
 
     def test_username_length(self):
-        """Test that user names >64 characters are properly truncated."""
+        """Test that user names >255 characters are properly truncated."""
 
         self._stubs_auth()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
 
-        # a >64 character user name and the expected version
-        long_user_name = 'U' * 64 + 'S'
-        good_user_name = long_user_name[-64:]
+        # a >255 character user name and the expected version
+        long_user_name = 'U' * 255 + 'S'
+        good_user_name = 'U' * 254 + 'S'
         # mock keystone client user functions
         self.mock_ks_v3_client.users = self.m.CreateMockAnything()
         mock_user = self.m.CreateMockAnything()
         mock_user.id = 'auser123'
         # when keystone is called, the name should have been truncated
-        # to the last 64 characters of the long name
+        # to the last 255 characters of the long name
         self.mock_ks_v3_client.users.create(name=good_user_name,
                                             password='password',
                                             default_project=ctx.tenant_id
@@ -477,6 +471,7 @@ class KeystoneClientTest(common.HeatTestCase):
 
         ctx = utils.dummy_context()
         ctx.auth_token = None
+        ctx.password = 'password'
         ctx.trust_id = None
         ctx.user_domain = 'adomain123'
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
@@ -501,8 +496,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test create_trust_context with existing trust_id."""
 
         self._stubs_auth(method='trust')
-        cfg.CONF.set_override('deferred_auth_method', 'trusts',
-                              enforce_type=True)
+        cfg.CONF.set_override('deferred_auth_method', 'trusts')
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
@@ -532,11 +526,9 @@ class KeystoneClientTest(common.HeatTestCase):
                                                        project_id='42',
                                                        stub_trust_context=True)
 
-        cfg.CONF.set_override('deferred_auth_method', 'trusts',
-                              enforce_type=True)
+        cfg.CONF.set_override('deferred_auth_method', 'trusts')
         if delegate_roles:
-            cfg.CONF.set_override('trusts_delegated_roles', delegate_roles,
-                                  enforce_type=True)
+            cfg.CONF.set_override('trusts_delegated_roles', delegate_roles)
 
         trustor_roles = ['heat_stack_owner', 'admin', '__member__']
         trustee_roles = delegate_roles or trustor_roles
@@ -573,10 +565,8 @@ class KeystoneClientTest(common.HeatTestCase):
                                                     project_id='42',
                                                     stub_trust_context=True)
 
-        cfg.CONF.set_override('deferred_auth_method', 'trusts',
-                              enforce_type=True)
-        cfg.CONF.set_override('trusts_delegated_roles', ['heat_stack_owner'],
-                              enforce_type=True)
+        cfg.CONF.set_override('deferred_auth_method', 'trusts')
+        cfg.CONF.set_override('trusts_delegated_roles', ['heat_stack_owner'])
 
         self.mock_ks_v3_client.trusts = self.m.CreateMockAnything()
         self.mock_ks_v3_client.trusts.create(
@@ -630,8 +620,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test consuming a trust when initializing."""
 
         self._stubs_auth(method='trust')
-        cfg.CONF.set_override('deferred_auth_method', 'trusts',
-                              enforce_type=True)
+        cfg.CONF.set_override('deferred_auth_method', 'trusts')
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
@@ -649,8 +638,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test consuming a trust when initializing, error scoping."""
 
         self._stubs_auth(method='trust', trust_scoped=False)
-        cfg.CONF.set_override('deferred_auth_method', 'trusts',
-                              enforce_type=True)
+        cfg.CONF.set_override('deferred_auth_method', 'trusts')
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
@@ -667,8 +655,7 @@ class KeystoneClientTest(common.HeatTestCase):
         """Test consuming a trust when initializing, impersonation error."""
 
         self._stubs_auth(method='trust', user_id='wrong_user_id')
-        cfg.CONF.set_override('deferred_auth_method', 'trusts',
-                              enforce_type=True)
+        cfg.CONF.set_override('deferred_auth_method', 'trusts')
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
@@ -711,8 +698,7 @@ class KeystoneClientTest(common.HeatTestCase):
 
     def _test_delete_trust(self, raise_ext=None):
         self._stubs_auth()
-        cfg.CONF.set_override('deferred_auth_method', 'trusts',
-                              enforce_type=True)
+        cfg.CONF.set_override('deferred_auth_method', 'trusts')
         self.mock_ks_v3_client.trusts = self.m.CreateMockAnything()
         if raise_ext is None:
             self.mock_ks_v3_client.trusts.delete('atrust123').AndReturn(None)
@@ -997,13 +983,14 @@ class KeystoneClientTest(common.HeatTestCase):
                           user_id='duser123', project_id='aproject',
                           credential_id='acredentialid')
 
-    def _stub_gen_creds(self, access, secret):
+    def _stub_uuid(self, values=None):
         # stub UUID.hex to return the values specified
-        mock_access_uuid = mock.Mock()
-        mock_access_uuid.hex = access
-        self.patchobject(uuid, 'uuid4', return_value=mock_access_uuid)
-        self.patchobject(password_gen, 'generate_openstack_password',
-                         return_value=secret)
+        values = values or []
+        self.m.StubOutWithMock(uuid, 'uuid4')
+        for v in values:
+            mock_uuid = self.m.CreateMockAnything()
+            mock_uuid.hex = v
+            uuid.uuid4().AndReturn(mock_uuid)
 
     def test_create_ec2_keypair(self):
 
@@ -1019,7 +1006,7 @@ class KeystoneClientTest(common.HeatTestCase):
         ex_data_json = json.dumps(ex_data)
 
         # stub UUID.hex to match ex_data
-        self._stub_gen_creds('dummy_access', 'dummy_secret')
+        self._stub_uuid(['dummy_access', 'dummy_secret'])
 
         # mock keystone client credentials functions
         self.mock_ks_v3_client.credentials = self.m.CreateMockAnything()
@@ -1055,7 +1042,7 @@ class KeystoneClientTest(common.HeatTestCase):
         ex_data_json = json.dumps(ex_data)
 
         # stub UUID.hex to match ex_data
-        self._stub_gen_creds('dummy_access2', 'dummy_secret2')
+        self._stub_uuid(['dummy_access2', 'dummy_secret2'])
 
         # mock keystone client credentials functions
         self.mock_admin_client.credentials = self.m.CreateMockAnything()
@@ -1092,7 +1079,7 @@ class KeystoneClientTest(common.HeatTestCase):
         ex_data_json = json.dumps(ex_data)
 
         # stub UUID.hex to match ex_data
-        self._stub_gen_creds('dummy_access2', 'dummy_secret2')
+        self._stub_uuid(['dummy_access2', 'dummy_secret2'])
 
         # mock keystone client credentials functions
         self.mock_ks_v3_client.credentials = self.m.CreateMockAnything()
@@ -1422,88 +1409,10 @@ class KeystoneClientTest(common.HeatTestCase):
         self.assertIsNone(heat_ks_client.delete_stack_domain_project(
             project_id='aprojectid'))
 
-    def _test_url_for(self, service_url, expected_kwargs, ctx=None, **kwargs):
-        """Testing url_for depending on different ways to pass region name."""
-        mock_ks_auth, mock_auth_ref = self._stubs_auth(client=False)
-        mock_ks_auth.get_endpoint(mox.IsA(ks_session.Session),
-                                  **expected_kwargs).AndReturn(service_url)
-
-        self.m.ReplayAll()
-        ctx = ctx or utils.dummy_context()
-        heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
-        self.assertEqual(service_url, heat_ks_client.url_for(**kwargs))
-        self.m.VerifyAll()
-
-    def test_url_for(self):
-        """Test that None value is passed as region name.
-
-        Test that None value is passed as region name if region name is not
-        specified in the config file or as one of the arguments.
-        """
-        cfg.CONF.set_override('region_name_for_services', None,
-                              enforce_type=True)
-        service_url = 'http://example.com:1234/v1'
-        kwargs = {
-            'region_name': None
-        }
-        self._test_url_for(service_url, kwargs)
-
-    def test_url_for_with_region(self):
-        """Test that region name passed as argument.
-
-        Test that region name passed as argument is not override by region name
-        specified in the config file.
-        """
-        cfg.CONF.set_override('region_name_for_services', 'RegionOne',
-                              enforce_type=True)
-        service_url = 'http://regiontwo.example.com:1234/v1'
-        kwargs = {
-            'region_name': 'RegionTwo'
-        }
-        self._test_url_for(service_url, kwargs, None, **kwargs)
-
-    def test_url_for_with_region_name_from_config(self):
-        """Test default region name for services from config file is passed.
-
-        Test that default region name for services from config file is passed
-        if region name is not specified in arguments.
-        """
-        region_name_for_services = 'RegionOne'
-        cfg.CONF.set_override('region_name_for_services',
-                              region_name_for_services, enforce_type=True)
-        kwargs = {
-            'region_name': region_name_for_services
-        }
-        service_url = 'http://regionone.example.com:1234/v1'
-        self._test_url_for(service_url, kwargs)
-
-    def test_url_for_with_region_name_from_context(self):
-        """Test that default region name for services from context is passed.
-
-        Test that default region name for services from context is passed
-        if region name is not specified in arguments.
-        """
-        cfg.CONF.set_override('region_name_for_services', 'RegionOne',
-                              enforce_type=True)
-        service_url = 'http://regiontwo.example.com:1234/v1'
-        region_name_for_services = 'RegionTwo'
-        expected_kwargs = {
-            'region_name': region_name_for_services
-        }
-        ctx = utils.dummy_context('test_username',
-                                  'test_tenant_id',
-                                  'password',
-                                  None,
-                                  None,
-                                  None,
-                                  region_name_for_services)
-        self._test_url_for(service_url, expected_kwargs, ctx)
-
 
 class KeystoneClientTestDomainName(KeystoneClientTest):
     def setUp(self):
-        cfg.CONF.set_override('stack_user_domain_name', 'fake_domain_name',
-                              enforce_type=True)
+        cfg.CONF.set_override('stack_user_domain_name', 'fake_domain_name')
         super(KeystoneClientTestDomainName, self).setUp()
         cfg.CONF.clear_override('stack_user_domain_id')
 
