@@ -32,7 +32,7 @@ class NeutronClientPluginTestCase(common.HeatTestCase):
         con = utils.dummy_context()
         c = con.clients
         self.neutron_plugin = c.client_plugin('neutron')
-        self.neutron_plugin._client = self.neutron_client
+        self.neutron_plugin.client = lambda: self.neutron_client
 
 
 class NeutronClientPluginTest(NeutronClientPluginTestCase):
@@ -42,42 +42,6 @@ class NeutronClientPluginTest(NeutronClientPluginTestCase):
                                           'find_resourceid_by_name_or_id')
         self.mock_find.return_value = 42
 
-    def test_resolve_network(self):
-        props = {'net': 'test_network'}
-
-        res = self.neutron_plugin.resolve_network(props, 'net', 'net_id')
-        self.assertEqual(42, res)
-        self.mock_find.assert_called_once_with(self.neutron_client, 'network',
-                                               'test_network',
-                                               cmd_resource=None)
-
-        # check resolve if was send id instead of name
-        props = {'net_id': 77}
-        res = self.neutron_plugin.resolve_network(props, 'net', 'net_id')
-        self.assertEqual(77, res)
-        # in this case find_resourceid_by_name_or_id is not called
-        self.mock_find.assert_called_once_with(self.neutron_client, 'network',
-                                               'test_network',
-                                               cmd_resource=None)
-
-    def test_resolve_subnet(self):
-        props = {'snet': 'test_subnet'}
-
-        res = self.neutron_plugin.resolve_subnet(props, 'snet', 'snet_id')
-        self.assertEqual(42, res)
-        self.mock_find.assert_called_once_with(self.neutron_client, 'subnet',
-                                               'test_subnet',
-                                               cmd_resource=None)
-
-        # check resolve if was send id instead of name
-        props = {'snet_id': 77}
-        res = self.neutron_plugin.resolve_subnet(props, 'snet', 'snet_id')
-        self.assertEqual(77, res)
-        # in this case find_resourceid_by_name_or_id is not called
-        self.mock_find.assert_called_once_with(self.neutron_client, 'subnet',
-                                               'test_subnet',
-                                               cmd_resource=None)
-
     def test_get_secgroup_uuids(self):
         # test get from uuids
         sgs_uuid = ['b62c3079-6946-44f5-a67b-6b9091884d4f',
@@ -167,44 +131,37 @@ class NeutronConstraintsValidate(common.HeatTestCase):
     scenarios = [
         ('validate_network',
             dict(constraint_class=nc.NetworkConstraint,
-                 resource_type='network',
-                 cmd_resource=None)),
+                 resource_type='network')),
         ('validate_port',
             dict(constraint_class=nc.PortConstraint,
-                 resource_type='port',
-                 cmd_resource=None)),
+                 resource_type='port')),
         ('validate_router',
             dict(constraint_class=nc.RouterConstraint,
-                 resource_type='router',
-                 cmd_resource=None)),
+                 resource_type='router')),
         ('validate_subnet',
             dict(constraint_class=nc.SubnetConstraint,
-                 resource_type='subnet',
-                 cmd_resource=None)),
+                 resource_type='subnet')),
         ('validate_subnetpool',
             dict(constraint_class=nc.SubnetPoolConstraint,
-                 resource_type='subnetpool',
-                 cmd_resource=None)),
+                 resource_type='subnetpool')),
         ('validate_address_scope',
             dict(constraint_class=nc.AddressScopeConstraint,
-                 resource_type='address_scope',
-                 cmd_resource=None)),
+                 resource_type='address_scope')),
         ('validate_loadbalancer',
             dict(constraint_class=lc.LoadbalancerConstraint,
-                 resource_type='loadbalancer',
-                 cmd_resource='lbaas_loadbalancer')),
+                 resource_type='loadbalancer')),
         ('validate_listener',
             dict(constraint_class=lc.ListenerConstraint,
-                 resource_type='listener',
-                 cmd_resource=None)),
+                 resource_type='listener')),
         ('validate_pool',
             dict(constraint_class=lc.PoolConstraint,
-                 resource_type='pool',
-                 cmd_resource='lbaas_pool')),
+                 resource_type='pool')),
         ('validate_qos_policy',
             dict(constraint_class=nc.QoSPolicyConstraint,
-                 resource_type='policy',
-                 cmd_resource='qos_policy'))
+                 resource_type='policy')),
+        ('validate_security_group',
+            dict(constraint_class=nc.SecurityGroupConstraint,
+                 resource_type='security_group'))
     ]
 
     def test_validate(self):
@@ -238,10 +195,8 @@ class NeutronConstraintsValidate(common.HeatTestCase):
         self.assertTrue(constraint.validate("foo", ctx))
         self.assertFalse(constraint.validate("bar", ctx))
         mock_find.assert_has_calls(
-            [mock.call(self.resource_type, 'foo',
-                       cmd_resource=self.cmd_resource),
-             mock.call(self.resource_type, 'bar',
-                       cmd_resource=self.cmd_resource)])
+            [mock.call(self.resource_type, 'foo'),
+             mock.call(self.resource_type, 'bar')])
 
 
 class NeutronProviderConstraintsValidate(common.HeatTestCase):
