@@ -11,12 +11,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from heat.common import exception
 from heat.common.i18n import _
 from heat.engine import attributes
 from heat.engine import properties
 from heat.engine import resource
 
+from oslo_log import log as logging
+
+LOG = logging.getLogger(__name__)
+
 
 class TestResource(resource.Resource):
 
@@ -77,15 +80,19 @@ class TestResource(resource.Resource):
     }
 
     def handle_create(self):
+        LOG.info('Creating resource %s with properties %s',
+                 self.name, dict(self.properties))
         for prop in self.properties.props.keys():
             self.data_set(prop, self.properties.get(prop), redact=False)
 
         self.resource_id_set(self.physical_resource_name())
 
     def handle_update(self, json_snippet=None, tmpl_diff=None, prop_diff=None):
+        LOG.info('Updating resource %s with prop_diff %s',
+                 self.name, prop_diff)
         for prop in prop_diff:
             if '!' in prop:
-                raise exception.UpdateReplace(self.name)
+                raise resource.UpdateReplace(self.name)
             self.data_set(prop, prop_diff.get(prop), redact=False)
 
     def _resolve_attribute(self, name):
