@@ -16,6 +16,7 @@ import functools
 import inspect
 
 from oslo_log import log as logging
+from oslo_messaging import rpc
 
 LOG = logging.getLogger(__name__)
 
@@ -36,6 +37,11 @@ def asynchronous(function):
         if len(args) == 1 and not kwargs and isinstance(args[0], MessageData):
             try:
                 return function(processor, **args[0]._asdict())
+            except rpc.dispatcher.ExpectedException as exc:
+                LOG.error('[%s] Exception in "%s": %s',
+                          processor.name, function.__name__, exc.exc_info[1],
+                          exc_info=exc.exc_info)
+                raise
             except Exception as exc:
                 LOG.exception('[%s] Exception in "%s": %s',
                               processor.name, function.__name__, exc)
@@ -60,7 +66,7 @@ class MessageProcessor(object):
     def __call__(self):
         message = self.queue.get()
         if message is None:
-            LOG.debug('[%s] No messages' % self.name)
+            LOG.debug('[%s] No messages', self.name)
             return False
 
         try:
