@@ -14,6 +14,7 @@
 import mock
 import six
 
+from glanceclient import exc
 from heat.common import exception
 from heat.common import template_format
 from heat.engine import stack as parser
@@ -37,6 +38,12 @@ resources:
       min_ram: 512
       protected: False
       location: https://launchpad.net/cirros/cirros-0.3.0-x86_64-disk.img
+      location: https://launchpad.net/cirros/cirros-0.3.0-x86_64-disk.img
+      architecture: test_architecture
+      kernel_id: 12345678-1234-1234-1234-123456789012
+      os_distro: test_distro
+      owner: test_owner
+      ramdisk_id: 12345678-1234-1234-1234-123456789012
 '''
 
 image_template_validate = '''
@@ -57,9 +64,7 @@ class GlanceImageTest(common.HeatTestCase):
     def setUp(self):
         super(GlanceImageTest, self).setUp()
 
-        utils.setup_dummy_db()
         self.ctx = utils.dummy_context()
-
         tpl = template_format.parse(image_template)
         self.stack = parser.Stack(
             self.ctx, 'glance_image_test_stack',
@@ -72,6 +77,7 @@ class GlanceImageTest(common.HeatTestCase):
         self.my_image.client = glance
         glance.return_value = self.glanceclient
         self.images = self.glanceclient.images
+        self.image_tags = self.glanceclient.image_tags
 
     def _test_validate(self, resource, error_msg):
         exc = self.assertRaises(exception.StackValidationFailed,
@@ -86,7 +92,10 @@ class GlanceImageTest(common.HeatTestCase):
             template.Template(tpl)
         )
         image = stack['image']
-        image.t['Properties']['min_disk'] = -1
+        props = stack.t.t['resources']['image']['properties'].copy()
+        props['min_disk'] = -1
+        image.t = image.t.freeze(properties=props)
+        image.reparse()
         error_msg = ('Property error: resources.image.properties.min_disk: '
                      '-1 is out of range (min: 0, max: None)')
         self._test_validate(image, error_msg)
@@ -99,7 +108,10 @@ class GlanceImageTest(common.HeatTestCase):
             template.Template(tpl)
         )
         image = stack['image']
-        image.t['Properties']['min_ram'] = -1
+        props = stack.t.t['resources']['image']['properties'].copy()
+        props['min_ram'] = -1
+        image.t = image.t.freeze(properties=props)
+        image.reparse()
         error_msg = ('Property error: resources.image.properties.min_ram: '
                      '-1 is out of range (min: 0, max: None)')
         self._test_validate(image, error_msg)
@@ -112,7 +124,10 @@ class GlanceImageTest(common.HeatTestCase):
             template.Template(tpl)
         )
         image = stack['image']
-        image.t['Properties'].pop('disk_format')
+        props = stack.t.t['resources']['image']['properties'].copy()
+        del props['disk_format']
+        image.t = image.t.freeze(properties=props)
+        image.reparse()
         error_msg = 'Property disk_format not assigned'
         self._test_validate(image, error_msg)
 
@@ -124,7 +139,10 @@ class GlanceImageTest(common.HeatTestCase):
             template.Template(tpl)
         )
         image = stack['image']
-        image.t['Properties']['disk_format'] = 'incorrect_format'
+        props = stack.t.t['resources']['image']['properties'].copy()
+        props['disk_format'] = 'incorrect_format'
+        image.t = image.t.freeze(properties=props)
+        image.reparse()
         error_msg = ('Property error: '
                      'resources.image.properties.disk_format: '
                      '"incorrect_format" is not an allowed value '
@@ -139,7 +157,10 @@ class GlanceImageTest(common.HeatTestCase):
             template.Template(tpl)
         )
         image = stack['image']
-        image.t['Properties'].pop('container_format')
+        props = stack.t.t['resources']['image']['properties'].copy()
+        del props['container_format']
+        image.t = image.t.freeze(properties=props)
+        image.reparse()
         error_msg = 'Property container_format not assigned'
         self._test_validate(image, error_msg)
 
@@ -151,7 +172,10 @@ class GlanceImageTest(common.HeatTestCase):
             template.Template(tpl)
         )
         image = stack['image']
-        image.t['Properties']['container_format'] = 'incorrect_format'
+        props = stack.t.t['resources']['image']['properties'].copy()
+        props['container_format'] = 'incorrect_format'
+        image.t = image.t.freeze(properties=props)
+        image.reparse()
         error_msg = ('Property error: '
                      'resources.image.properties.container_format: '
                      '"incorrect_format" is not an allowed value '
@@ -166,7 +190,10 @@ class GlanceImageTest(common.HeatTestCase):
             template.Template(tpl)
         )
         image = stack['image']
-        image.t['Properties'].pop('location')
+        props = stack.t.t['resources']['image']['properties'].copy()
+        del props['location']
+        image.t = image.t.freeze(properties=props)
+        image.reparse()
         error_msg = 'Property location not assigned'
         self._test_validate(image, error_msg)
 
@@ -177,8 +204,11 @@ class GlanceImageTest(common.HeatTestCase):
             template.Template(tpl)
         )
         image = stack['image']
-        image.t['Properties']['disk_format'] = 'raw'
-        image.t['Properties']['container_format'] = 'ari'
+        props = stack.t.t['resources']['image']['properties'].copy()
+        props['disk_format'] = 'raw'
+        props['container_format'] = 'ari'
+        image.t = image.t.freeze(properties=props)
+        image.reparse()
         error_msg = ("Invalid mix of disk and container formats. When "
                      "setting a disk or container format to one of 'aki', "
                      "'ari', or 'ami', the container and disk formats must "
@@ -190,8 +220,122 @@ class GlanceImageTest(common.HeatTestCase):
         image_id = '41f0e60c-ebb4-4375-a2b4-845ae8b9c995'
         value.id = image_id
         self.images.create.return_value = value
+        self.image_tags.update.return_value = None
+        props = self.stack.t.t['resources']['my_image']['properties'].copy()
+        props['tags'] = ['tag1']
+        self.my_image.t = self.my_image.t.freeze(properties=props)
+        self.my_image.reparse()
         self.my_image.handle_create()
+
         self.assertEqual(image_id, self.my_image.resource_id)
+        # assert that no tags pass when image create
+        self.images.create.assert_called_once_with(
+            container_format=u'bare',
+            disk_format=u'qcow2',
+            id=u'41f0e60c-ebb4-4375-a2b4-845ae8b9c995',
+            is_public=True,
+            location=u'https://launchpad.net/cirros/'
+                     u'cirros-0.3.0-x86_64-disk.img',
+            min_disk=10,
+            min_ram=512,
+            name=u'cirros_image',
+            protected=False,
+            owner=u'test_owner',
+            properties={}
+        )
+        self.image_tags.update.assert_called_once_with(
+            self.my_image.resource_id,
+            'tag1')
+        calls = [mock.call(self.my_image.resource_id,
+                           architecture='test_architecture'),
+                 mock.call(self.my_image.resource_id,
+                           kernel_id='12345678-1234-1234-1234-123456789012'),
+                 mock.call(self.my_image.resource_id,
+                           os_distro='test_distro'),
+                 mock.call(self.my_image.resource_id,
+                           ramdisk_id='12345678-1234-1234-1234-123456789012'),
+                 ]
+        self.images.update.assert_has_calls(calls)
+
+    def _handle_update_tags(self, prop_diff):
+        self.my_image.handle_update(json_snippet=None,
+                                    tmpl_diff=None,
+                                    prop_diff=prop_diff)
+
+        self.image_tags.update.assert_called_once_with(
+            self.my_image.resource_id,
+            'tag2'
+        )
+        self.image_tags.delete.assert_called_once_with(
+            self.my_image.resource_id,
+            'tag1'
+        )
+
+    def test_image_handle_update(self):
+        self.my_image.resource_id = '477e8273-60a7-4c41-b683-fdb0bc7cd151'
+        prop_diff = {
+            'architecture': 'test_architecture',
+            'kernel_id': '12345678-1234-1234-1234-123456789012',
+            'os_distro': 'test_distro',
+            'owner': 'test_owner',
+            'ramdisk_id': '12345678-1234-1234-1234-123456789012',
+            'extra_properties': {'key1': 'value1', 'key2': 'value2'}}
+
+        self.my_image.handle_update(json_snippet=None,
+                                    tmpl_diff=None,
+                                    prop_diff=prop_diff)
+        self.images.update.assert_called_once_with(
+            self.my_image.resource_id,
+            [],
+            architecture='test_architecture',
+            kernel_id='12345678-1234-1234-1234-123456789012',
+            os_distro='test_distro',
+            owner='test_owner',
+            ramdisk_id='12345678-1234-1234-1234-123456789012',
+            key1='value1', key2='value2'
+        )
+
+    def test_image_handle_update_tags(self):
+        self.my_image.resource_id = '477e8273-60a7-4c41-b683-fdb0bc7cd151'
+
+        props = self.stack.t.t['resources']['my_image']['properties'].copy()
+        props['tags'] = ['tag1']
+        self.my_image.t = self.my_image.t.freeze(properties=props)
+        self.my_image.reparse()
+        prop_diff = {'tags': ['tag2']}
+
+        self._handle_update_tags(prop_diff)
+
+    def test_image_handle_update_remove_tags(self):
+        self.my_image.resource_id = '477e8273-60a7-4c41-b683-fdb0bc7cd151'
+
+        props = self.stack.t.t['resources']['my_image']['properties'].copy()
+        props['tags'] = ['tag1']
+        self.my_image.t = self.my_image.t.freeze(properties=props)
+        self.my_image.reparse()
+        prop_diff = {'tags': None}
+
+        self.my_image.handle_update(json_snippet=None,
+                                    tmpl_diff=None,
+                                    prop_diff=prop_diff)
+
+        self.image_tags.delete.assert_called_once_with(
+            self.my_image.resource_id,
+            'tag1'
+        )
+
+    def test_image_handle_update_tags_delete_not_found(self):
+        self.my_image.resource_id = '477e8273-60a7-4c41-b683-fdb0bc7cd151'
+
+        props = self.stack.t.t['resources']['my_image']['properties'].copy()
+        props['tags'] = ['tag1']
+        self.my_image.t = self.my_image.t.freeze(properties=props)
+        self.my_image.reparse()
+        prop_diff = {'tags': ['tag2']}
+
+        self.image_tags.delete.side_effect = exc.HTTPNotFound()
+
+        self._handle_update_tags(prop_diff)
 
     def test_image_show_resource_v1(self):
         self.glanceclient.version = 1.0
@@ -233,7 +377,14 @@ class GlanceImageTest(common.HeatTestCase):
             'is_public': False,
             'min_disk': 0,
             'min_ram': 0,
-            'id': '41f0e60c-ebb4-4375-a2b4-845ae8b9c995'
+            'id': '41f0e60c-ebb4-4375-a2b4-845ae8b9c995',
+            'tags': [],
+            'extra_properties': {},
+            'architecture': 'test_architecture',
+            'kernel_id': '12345678-1234-1234-1234-123456789012',
+            'os_distro': 'test_distro',
+            'owner': 'test_owner',
+            'ramdisk_id': '12345678-1234-1234-1234-123456789012'
         }
         if version == 1.0:
             image = mock.MagicMock()
@@ -255,7 +406,14 @@ class GlanceImageTest(common.HeatTestCase):
             'is_public': False,
             'min_disk': 0,
             'min_ram': 0,
-            'id': '41f0e60c-ebb4-4375-a2b4-845ae8b9c995'
+            'id': '41f0e60c-ebb4-4375-a2b4-845ae8b9c995',
+            'tags': [],
+            'extra_properties': {},
+            'architecture': 'test_architecture',
+            'kernel_id': '12345678-1234-1234-1234-123456789012',
+            'os_distro': 'test_distro',
+            'owner': 'test_owner',
+            'ramdisk_id': '12345678-1234-1234-1234-123456789012'
         }
         if version == 1.0:
             expected.update({'location': 'stub'})
