@@ -116,7 +116,7 @@ Resources:
         self.assert_min('[0-9]', random_string, 1)
         self.assert_min('[A-Z]', random_string, 1)
         self.assert_min('[a-z]', random_string, 20)
-        self.assert_min('[(),\[\]{}]', random_string, 1)
+        self.assert_min(r'[(),\[\]{}]', random_string, 1)
         self.assert_min('[$_]', random_string, 2)
         self.assert_min('@', random_string, 5)
         self.assertEqual(secret3.FnGetRefId(), random_string)
@@ -132,7 +132,7 @@ Resources:
         secret5 = stack['secret5']
         random_string = secret5.FnGetAtt('value')
         self.assertEqual(10, len(random_string))
-        self.assert_min('[(),\[\]{}]', random_string, 1)
+        self.assert_min(r'[(),\[\]{}]', random_string, 1)
         self.assert_min('[$_]', random_string, 2)
         self.assert_min('@', random_string, 5)
         self.assertEqual(secret5.FnGetRefId(), random_string)
@@ -259,7 +259,7 @@ Resources:
         return stack
 
     # test was saved to test backward compatibility with old behavior
-    def test_generate_random_string_backward_compatible(self):
+    def test_generate_random_string_backward_compatiable(self):
         stack = self.parse_stack(template_format.parse(self.template_rs))
         secret = stack['secret']
         char_classes = secret.properties['character_classes']
@@ -268,125 +268,8 @@ Resources:
         # run each test multiple times to confirm random generator
         # doesn't generate a matching pattern by chance
         for i in range(1, 32):
-            r = secret._generate_random_string([], char_classes, self.length)
+            r = secret._generate_random_string(None, char_classes, self.length)
 
             self.assertThat(r, matchers.HasLength(self.length))
             regex = '%s{%s}' % (self.pattern, self.length)
             self.assertThat(r, matchers.MatchesRegex(regex))
-
-
-class TestGenerateRandomStringDistribution(common.HeatTestCase):
-    def run_test(self, tmpl, iterations=5):
-        stack = utils.parse_stack(template_format.parse(tmpl))
-        secret = stack['secret']
-        secret.data_set = mock.Mock()
-
-        for i in range(iterations):
-            secret.handle_create()
-
-        return [call[1][1] for call in secret.data_set.mock_calls]
-
-    def char_counts(self, random_strings, char):
-        return [rs.count(char) for rs in random_strings]
-
-    def check_stats(self, char_counts, expected_mean, allowed_variance,
-                    expected_minimum=0):
-        mean = float(sum(char_counts)) / len(char_counts)
-        self.assertLess(mean, expected_mean + allowed_variance)
-        self.assertGreater(mean, max(0, expected_mean - allowed_variance))
-        if expected_minimum:
-            self.assertGreaterEqual(min(char_counts), expected_minimum)
-
-    def test_class_uniformity(self):
-        template_rs = '''
-HeatTemplateFormatVersion: '2012-12-12'
-Resources:
-  secret:
-    Type: OS::Heat::RandomString
-    Properties:
-      length: 66
-      character_classes:
-        - class: lettersdigits
-      character_sequences:
-        - sequence: "*$"
-'''
-
-        results = self.run_test(template_rs, 10)
-        for char in '$*':
-            self.check_stats(self.char_counts(results, char), 1.5, 2)
-
-    def test_repeated_sequence(self):
-        template_rs = '''
-HeatTemplateFormatVersion: '2012-12-12'
-Resources:
-  secret:
-    Type: OS::Heat::RandomString
-    Properties:
-      length: 40
-      character_classes: []
-      character_sequences:
-        - sequence: "**********$*****************************"
-'''
-
-        results = self.run_test(template_rs)
-        for char in '$*':
-            self.check_stats(self.char_counts(results, char), 20, 6)
-
-    def test_overlapping_classes(self):
-        template_rs = '''
-HeatTemplateFormatVersion: '2012-12-12'
-Resources:
-  secret:
-    Type: OS::Heat::RandomString
-    Properties:
-      length: 624
-      character_classes:
-        - class: lettersdigits
-        - class: digits
-        - class: octdigits
-        - class: hexdigits
-'''
-
-        results = self.run_test(template_rs, 20)
-        self.check_stats(self.char_counts(results, '0'), 10.3, 2.5)
-
-    def test_overlapping_sequences(self):
-        template_rs = '''
-HeatTemplateFormatVersion: '2012-12-12'
-Resources:
-  secret:
-    Type: OS::Heat::RandomString
-    Properties:
-      length: 60
-      character_classes: []
-      character_sequences:
-        - sequence: "01"
-        - sequence: "02"
-        - sequence: "03"
-        - sequence: "04"
-        - sequence: "05"
-        - sequence: "06"
-        - sequence: "07"
-        - sequence: "08"
-        - sequence: "09"
-'''
-
-        results = self.run_test(template_rs)
-        self.check_stats(self.char_counts(results, '0'), 10, 5)
-
-    def test_overlapping_class_sequence(self):
-        template_rs = '''
-HeatTemplateFormatVersion: '2012-12-12'
-Resources:
-  secret:
-    Type: OS::Heat::RandomString
-    Properties:
-      length: 402
-      character_classes:
-        - class: octdigits
-      character_sequences:
-        - sequence: "0"
-'''
-
-        results = self.run_test(template_rs, 10)
-        self.check_stats(self.char_counts(results, '0'), 51.125, 8, 1)
