@@ -14,17 +14,18 @@
 import datetime
 import uuid
 
+import mock
 import mox
 from oslo_serialization import jsonutils as json
 from oslo_utils import timeutils
 import six
 
-from heat.common import exception
 from heat.common import identifier
 from heat.common import template_format
 from heat.engine.clients.os import heat_plugin
 from heat.engine.clients.os import swift as swift_plugin
 from heat.engine import environment
+from heat.engine import resource
 from heat.engine.resources.openstack.heat import wait_condition_handle as h_wch
 from heat.engine import stack as parser
 from heat.engine import template as tmpl
@@ -107,7 +108,7 @@ resources:
         type: OS::Heat::UpdateWaitConditionHandle
 '''
 
-test_template_bad_waithandle = '''
+test_template_waithandle_bad_type = '''
 heat_template_version: 2013-05-23
 resources:
     wait_condition:
@@ -119,6 +120,20 @@ resources:
         type: OS::Heat::RandomString
 '''
 
+test_template_waithandle_bad_reference = '''
+heat_template_version: pike
+resources:
+    wait_condition:
+        type: OS::Heat::WaitCondition
+        properties:
+            handle: wait_handel
+            timeout: 5
+    wait_handle:
+        type: OS::Heat::WaitConditionHandle
+        properties:
+            signal_transport: NO_SIGNAL
+'''
+
 
 class HeatWaitConditionTest(common.HeatTestCase):
 
@@ -177,7 +192,7 @@ class HeatWaitConditionTest(common.HeatTestCase):
                          rsrc.state)
 
         r = resource_objects.Resource.get_by_name_and_stack(
-            None, 'wait_handle', self.stack.id)
+            self.stack.context, 'wait_handle', self.stack.id)
         self.assertEqual('wait_handle', r.name)
         self.m.VerifyAll()
 
@@ -201,21 +216,28 @@ class HeatWaitConditionTest(common.HeatTestCase):
         self.assertTrue(reason.startswith('WaitConditionFailure:'))
 
         r = resource_objects.Resource.get_by_name_and_stack(
-            None, 'wait_handle', self.stack.id)
+            self.stack.context, 'wait_handle', self.stack.id)
         self.assertEqual('wait_handle', r.name)
         self.m.VerifyAll()
 
-    def test_bad_wait_handle(self):
-        self.stack = self.create_stack(
-            template=test_template_bad_waithandle)
+    def _test_wait_handle_invalid(self, tmpl, handle_name):
+        self.stack = self.create_stack(template=tmpl)
         self.m.ReplayAll()
         self.stack.create()
         rsrc = self.stack['wait_condition']
         self.assertEqual((rsrc.CREATE, rsrc.FAILED), rsrc.state)
         reason = rsrc.status_reason
-        self.assertEqual(reason, 'ValueError: resources.wait_condition: '
-                                 'wait_handle is not a valid wait condition '
-                                 'handle.')
+        error_msg = ('ValueError: resources.wait_condition: '
+                     '%s is not a valid wait condition handle.') % handle_name
+        self.assertEqual(reason, error_msg)
+
+    def test_wait_handle_bad_type(self):
+        self._test_wait_handle_invalid(test_template_waithandle_bad_type,
+                                       'wait_handle')
+
+    def test_wait_handle_bad_reference(self):
+        self._test_wait_handle_invalid(
+            test_template_waithandle_bad_reference, 'wait_handel')
 
     def test_timeout(self):
         self.stack = self.create_stack()
@@ -480,7 +502,8 @@ class HeatWaitConditionTest(common.HeatTestCase):
         signal = json.loads(handle.FnGetAtt('signal'))
         self.assertIn('alarm_url', signal)
 
-    def test_getatt_signal_zaqar(self):
+    @mock.patch('zaqarclient.queues.v2.queues.Queue.signed_url')
+    def test_getatt_signal_zaqar(self, mock_signed_url):
         handle = self._create_heat_handle(
             template=test_template_heat_waithandle_zaqar)
         self.assertIsNone(handle.FnGetAtt('token'))
@@ -511,4 +534,4 @@ class HeatWaitConditionTest(common.HeatTestCase):
         handle = self.stack['update_wait_handle']
         self.assertEqual((handle.CREATE, handle.COMPLETE), handle.state)
         self.assertRaises(
-            exception.UpdateReplace, handle.update, None, None)
+            resource.UpdateReplace, handle.update, None, None)
