@@ -15,13 +15,13 @@ import copy
 import mock
 
 from heat.common import exception
+from heat.engine.clients.os.keystone import fake_keystoneclient as fake_ks
 from heat.engine import properties
 from heat.engine import resource
 from heat.engine.resources.openstack.keystone import role_assignments
 from heat.engine import stack
 from heat.engine import template
 from heat.tests import common
-from heat.tests import fakes
 from heat.tests import generic_resource
 from heat.tests import utils
 
@@ -96,6 +96,16 @@ class KeystoneRoleAssignmentMixinTest(common.HeatTestCase):
         (self.test_role_assignment.client_plugin.
          return_value) = self.keystone_client_plugin
 
+        self.parse_assgmnts = self.test_role_assignment.parse_list_assignments
+        self.test_role_assignment.parse_list_assignments = mock.MagicMock()
+        self.test_role_assignment.parse_list_assignments.return_value = [
+            {'role': 'role_1',
+             'domain': 'domain_1',
+             'project': None},
+            {'role': 'role_1',
+             'project': 'project_1',
+             'domain': None}]
+
     def test_properties_title(self):
         property_title_map = {MixinClass.ROLES: 'roles'}
 
@@ -189,19 +199,6 @@ class KeystoneRoleAssignmentMixinTest(common.HeatTestCase):
             project='project_1')
 
     def test_role_assignment_update_user(self):
-        self.test_role_assignment._stored_properties_data = {
-            'roles': [
-                {
-                    'role': 'role_1',
-                    'project': 'project_1'
-                },
-                {
-                    'role': 'role_1',
-                    'domain': 'domain_1'
-                }
-            ]
-        }
-
         prop_diff = {
             MixinClass.ROLES: [
                 {
@@ -246,19 +243,6 @@ class KeystoneRoleAssignmentMixinTest(common.HeatTestCase):
             project='project_1')
 
     def test_role_assignment_update_group(self):
-        self.test_role_assignment._stored_properties_data = {
-            'roles': [
-                {
-                    'role': 'role_1',
-                    'project': 'project_1'
-                },
-                {
-                    'role': 'role_1',
-                    'domain': 'domain_1'
-                }
-            ]
-        }
-
         prop_diff = {
             MixinClass.ROLES: [
                 {
@@ -317,18 +301,6 @@ class KeystoneRoleAssignmentMixinTest(common.HeatTestCase):
         self.assertEqual(0, self.roles.revoke.call_count)
 
     def test_role_assignment_delete_user(self):
-        self.test_role_assignment._stored_properties_data = {
-            'roles': [
-                {
-                    'role': 'role_1',
-                    'project': 'project_1'
-                },
-                {
-                    'role': 'role_1',
-                    'domain': 'domain_1'
-                }
-            ]
-        }
         self.assertIsNone(self.test_role_assignment.delete_assignment(
             user_id='user_1'))
 
@@ -346,18 +318,6 @@ class KeystoneRoleAssignmentMixinTest(common.HeatTestCase):
             project='project_1')
 
     def test_role_assignment_delete_group(self):
-        self.test_role_assignment._stored_properties_data = {
-            'roles': [
-                {
-                    'role': 'role_1',
-                    'project': 'project_1'
-                },
-                {
-                    'role': 'role_1',
-                    'domain': 'domain_1'
-                }
-            ]
-        }
         self.assertIsNone(self.test_role_assignment.delete_assignment(
             group_id='group_1'
         ))
@@ -375,6 +335,21 @@ class KeystoneRoleAssignmentMixinTest(common.HeatTestCase):
             group='group_1',
             project='project_1')
 
+    def test_role_assignment_delete_removed(self):
+        self.test_role_assignment.parse_list_assignments.return_value = [
+            {'role': 'role_1',
+             'domain': 'domain_1',
+             'project': None}]
+
+        self.assertIsNone(self.test_role_assignment.delete_assignment(
+            user_id='user_1'))
+
+        expected = [
+            ({'role': 'role_1', 'user': 'user_1', 'domain': 'domain_1'},)
+        ]
+
+        self.assertItemsEqual(expected, self.roles.revoke.call_args_list)
+
     def test_validate_1(self):
         self.test_role_assignment.properties = mock.MagicMock()
 
@@ -396,6 +371,42 @@ class KeystoneRoleAssignmentMixinTest(common.HeatTestCase):
         self.assertRaises(exception.ResourcePropertyConflict,
                           self.test_role_assignment.validate)
 
+    def test_empty_parse_list_assignments(self):
+        self.test_role_assignment.parse_list_assignments = self.parse_assgmnts
+        self.assertEqual([],
+                         self.test_role_assignment.parse_list_assignments())
+
+    def test_user_parse_list_assignments(self):
+        self._test_parse_list_assignments('user')
+
+    def test_group_parse_list_assignments(self):
+        self._test_parse_list_assignments('group')
+
+    def _test_parse_list_assignments(self, entity=None):
+        self.test_role_assignment.parse_list_assignments = self.parse_assgmnts
+        dict_obj = mock.MagicMock()
+        dict_obj.to_dict.side_effect = [{'scope': {
+            'project': {'id': 'fc0fe982401643368ff2eb11d9ca70f1'}},
+            'role': {'id': '3b8b253648f44256a457a5073b78021d'},
+            entity: {'id': '4147558a763046cfb68fb870d58ef4cf'}},
+            {'role': {'id': '3b8b253648f44258021d6a457a5073b7'},
+             entity: {'id': '4147558a763046cfb68fb870d58ef4cf'}}]
+        self.keystoneclient.role_assignments.list.return_value = [dict_obj,
+                                                                  dict_obj]
+
+        kwargs = {'%s_id' % entity: '4147558a763046cfb68fb870d58ef4cf'}
+        list_assignments = self.test_role_assignment.parse_list_assignments(
+            **kwargs)
+        expected = [
+            {'role': '3b8b253648f44256a457a5073b78021d',
+             'project': 'fc0fe982401643368ff2eb11d9ca70f1',
+             'domain': None},
+            {'role': '3b8b253648f44258021d6a457a5073b7',
+             'project': None,
+             'domain': None},
+        ]
+        self.assertEqual(expected, list_assignments)
+
 
 class KeystoneUserRoleAssignmentTest(common.HeatTestCase):
 
@@ -418,7 +429,7 @@ class KeystoneUserRoleAssignmentTest(common.HeatTestCase):
         # Mock client
         self.keystoneclient = mock.Mock()
         self.patchobject(resource.Resource, 'client',
-                         return_value=fakes.FakeKeystoneClient(
+                         return_value=fake_ks.FakeKeystoneClient(
                              client=self.keystoneclient))
         self.roles = self.keystoneclient.roles
 
@@ -435,6 +446,15 @@ class KeystoneUserRoleAssignmentTest(common.HeatTestCase):
         (self.test_role_assignment.client_plugin.
          return_value) = self.keystone_client_plugin
 
+        self.test_role_assignment.parse_list_assignments = mock.MagicMock()
+        self.test_role_assignment.parse_list_assignments.return_value = [
+            {'role': 'role_1',
+             'domain': 'domain_1',
+             'project': None},
+            {'role': 'role_1',
+             'project': 'project_1',
+             'domain': None}]
+
     def test_user_role_assignment_handle_create(self):
         self.test_role_assignment.handle_create()
 
@@ -451,19 +471,6 @@ class KeystoneUserRoleAssignmentTest(common.HeatTestCase):
             project='project_1')
 
     def test_user_role_assignment_handle_update(self):
-        self.test_role_assignment._stored_properties_data = {
-            'roles': [
-                {
-                    'role': 'role_1',
-                    'project': 'project_1'
-                },
-                {
-                    'role': 'role_1',
-                    'domain': 'domain_1'
-                }
-            ]
-        }
-
         prop_diff = {
             MixinClass.ROLES: [
                 {
@@ -508,18 +515,6 @@ class KeystoneUserRoleAssignmentTest(common.HeatTestCase):
             project='project_1')
 
     def test_user_role_assignment_handle_delete(self):
-        self.test_role_assignment._stored_properties_data = {
-            'roles': [
-                {
-                    'role': 'role_1',
-                    'project': 'project_1'
-                },
-                {
-                    'role': 'role_1',
-                    'domain': 'domain_1'
-                }
-            ]
-        }
         self.assertIsNone(self.test_role_assignment.handle_delete())
 
         # Remove role1-project1-domain1
@@ -536,7 +531,6 @@ class KeystoneUserRoleAssignmentTest(common.HeatTestCase):
             project='project_1')
 
     def test_user_role_assignment_delete_user_not_found(self):
-        self.test_role_assignment._stored_properties_data = {}
         self.keystone_client_plugin.get_user_id.side_effect = [
             exception.EntityNotFound]
         self.assertIsNone(self.test_role_assignment.handle_delete())
@@ -564,7 +558,7 @@ class KeystoneGroupRoleAssignmentTest(common.HeatTestCase):
         # Mock client
         self.keystoneclient = mock.Mock()
         self.patchobject(resource.Resource, 'client',
-                         return_value=fakes.FakeKeystoneClient(
+                         return_value=fake_ks.FakeKeystoneClient(
                              client=self.keystoneclient))
         self.roles = self.keystoneclient.roles
 
@@ -581,6 +575,15 @@ class KeystoneGroupRoleAssignmentTest(common.HeatTestCase):
         (self.test_role_assignment.client_plugin.
          return_value) = self.keystone_client_plugin
 
+        self.test_role_assignment.parse_list_assignments = mock.MagicMock()
+        self.test_role_assignment.parse_list_assignments.return_value = [
+            {'role': 'role_1',
+             'domain': 'domain_1',
+             'project': None},
+            {'role': 'role_1',
+             'project': 'project_1',
+             'domain': None}]
+
     def test_group_role_assignment_handle_create(self):
         self.test_role_assignment.handle_create()
 
@@ -597,19 +600,6 @@ class KeystoneGroupRoleAssignmentTest(common.HeatTestCase):
             project='project_1')
 
     def test_group_role_assignment_handle_update(self):
-        self.test_role_assignment._stored_properties_data = {
-            'roles': [
-                {
-                    'role': 'role_1',
-                    'project': 'project_1'
-                },
-                {
-                    'role': 'role_1',
-                    'domain': 'domain_1'
-                }
-            ]
-        }
-
         prop_diff = {
             MixinClass.ROLES: [
                 {
@@ -654,18 +644,6 @@ class KeystoneGroupRoleAssignmentTest(common.HeatTestCase):
             project='project_1')
 
     def test_group_role_assignment_handle_delete(self):
-        self.test_role_assignment._stored_properties_data = {
-            'roles': [
-                {
-                    'role': 'role_1',
-                    'project': 'project_1'
-                },
-                {
-                    'role': 'role_1',
-                    'domain': 'domain_1'
-                }
-            ]
-        }
         self.assertIsNone(self.test_role_assignment.handle_delete())
 
         # Remove role1-project1-domain1
@@ -682,7 +660,6 @@ class KeystoneGroupRoleAssignmentTest(common.HeatTestCase):
             project='project_1')
 
     def test_group_role_assignment_delete_group_not_found(self):
-        self.test_role_assignment._stored_properties_data = {}
         self.keystone_client_plugin.get_group_id.side_effect = [
             exception.EntityNotFound]
         self.assertIsNone(self.test_role_assignment.handle_delete())
