@@ -44,7 +44,21 @@ resources:
           publish:
             result: <% $.hello %>
 """
-
+workflow_template_with_tags = """
+heat_template_version: queens
+resources:
+  workflow:
+    type: OS::Mistral::Workflow
+    properties:
+      type: direct
+      tags:
+        - tagged
+      tasks:
+        - name: hello
+          action: std.echo output='Good morning!'
+          publish:
+            result: <% $.hello %>
+"""
 workflow_template_with_params = """
 heat_template_version: 2013-05-23
 resources:
@@ -673,11 +687,11 @@ class TestMistralWorkflow(common.HeatTestCase):
         self.patchobject(exec_manager, '_create', return_value=execution)
         scheduler.TaskRunner(wf.signal, details)()
         call_args = self.mistral.executions.create.call_args
-        args, _ = call_args
+        args, kwargs = call_args
         expected_args = (
             '{"image": "31d8eeaf-686e-4e95-bb27-765014b9f20b", '
             '"name": "create_test_server", "flavor": "3"}')
-        self.validate_json_inputs(args[1], expected_args)
+        self.validate_json_inputs(kwargs['workflow_input'], expected_args)
         self.assertEqual({'executions': '12345'}, wf.data())
         # Updating the workflow changing "use_request_body_as_input" to
         # false and signaling again with the expected request body format.
@@ -698,11 +712,11 @@ class TestMistralWorkflow(common.HeatTestCase):
         self.patchobject(exec_manager, '_create', return_value=execution)
         scheduler.TaskRunner(wf.signal, details)()
         call_args = self.mistral.executions.create.call_args
-        args, _ = call_args
+        args, kwargs = call_args
         expected_args = (
             '{"image": "31d8eeaf-686e-4e95-bb27-765014b9f20b", '
             '"name": "create_test_server", "flavor": "4"}')
-        self.validate_json_inputs(args[1], expected_args)
+        self.validate_json_inputs(kwargs['workflow_input'], expected_args)
         self.assertEqual({'executions': '54321,12345', 'name':
                          'test_stack-workflow-b5fiekdsa355'}, wf.data())
         scheduler.TaskRunner(wf.delete)()
@@ -747,6 +761,22 @@ class TestMistralWorkflow(common.HeatTestCase):
             lambda *args, **kw: self.verify_params(*args, **kw))
         scheduler.TaskRunner(wf.signal, details)()
 
+    def test_workflow_tags(self):
+        tmpl = template_format.parse(workflow_template_with_tags)
+        stack = utils.parse_stack(tmpl)
+        rsrc_defns = stack.t.resource_definitions(stack)['workflow']
+        wf = workflow.Workflow('workflow', rsrc_defns, stack)
+        self.mistral.workflows.create.return_value = [
+            FakeWorkflow('workflow')]
+        scheduler.TaskRunner(wf.create)()
+        details = {'tags': ['mytag'],
+                   'params': {'test': 'param_value', 'test1': 'param_value_1'}}
+        execution = mock.Mock()
+        execution.id = '12345'
+        self.mistral.executions.create.side_effect = (
+            lambda *args, **kw: self.verify_params(*args, **kw))
+        scheduler.TaskRunner(wf.signal, details)()
+
     def test_workflow_params_merge(self):
         tmpl = template_format.parse(workflow_template_with_params)
         stack = utils.parse_stack(tmpl)
