@@ -50,6 +50,9 @@ class ListenerTest(common.HeatTestCase):
             return_value=self.neutron_client)
 
     def test_validate_terminated_https(self):
+        self.patchobject(listener.Listener, 'is_service_available',
+                         return_value=(True, None))
+
         tmpl = yaml.load(inline_templates.LISTENER_TEMPLATE)
         props = tmpl['resources']['listener']['properties']
         props['protocol'] = 'TERMINATED_HTTPS'
@@ -75,6 +78,7 @@ class ListenerTest(common.HeatTestCase):
                 'protocol_port': 80,
                 'protocol': 'TCP',
                 'loadbalancer_id': '123',
+                'default_pool_id': 'my_pool',
                 'name': 'my_listener',
                 'description': 'my listener',
                 'admin_state_up': True,
@@ -94,6 +98,21 @@ class ListenerTest(common.HeatTestCase):
         self.assertFalse(self.listener.check_create_complete(props))
         self.assertTrue(self.listener.check_create_complete(props))
 
+    @mock.patch('heat.engine.clients.os.neutron.'
+                'NeutronClientPlugin.has_extension', return_value=True)
+    def test_create_missing_properties(self, ext_func):
+        for prop in ('protocol', 'protocol_port', 'loadbalancer'):
+            tmpl = yaml.load(inline_templates.LISTENER_TEMPLATE)
+            del tmpl['resources']['listener']['properties'][prop]
+            del tmpl['resources']['listener']['properties']['default_pool']
+            self._create_stack(tmpl=yaml.dump(tmpl))
+            if prop == 'loadbalancer':
+                self.assertRaises(exception.PropertyUnspecifiedError,
+                                  self.listener.validate)
+            else:
+                self.assertRaises(exception.StackValidationFailed,
+                                  self.listener.validate)
+
     def test_show_resource(self):
         self._create_stack()
         self.listener.resource_id_set('1234')
@@ -120,7 +139,8 @@ class ListenerTest(common.HeatTestCase):
             'name': 'your_listener',
         }
 
-        prop_diff = self.listener.handle_update(None, None, prop_diff)
+        prop_diff = self.listener.handle_update(self.listener.t,
+                                                None, prop_diff)
 
         self.assertFalse(self.listener.check_update_complete(prop_diff))
         self.assertFalse(self.listener._update_called)
