@@ -60,8 +60,8 @@ class PoolTest(common.HeatTestCase):
                 'session_persistence type is set to APP_COOKIE.')
         with mock.patch('heat.engine.clients.os.neutron.NeutronClientPlugin.'
                         'has_extension', return_value=True):
-            self.assertRaisesRegexp(exception.StackValidationFailed,
-                                    msg, self.pool.validate)
+            self.assertRaisesRegex(exception.StackValidationFailed,
+                                   msg, self.pool.validate)
 
     def test_validate_source_ip_cookie_name(self):
         tmpl = yaml.load(inline_templates.POOL_TEMPLATE)
@@ -74,8 +74,8 @@ class PoolTest(common.HeatTestCase):
                 'session_persistence type is set to SOURCE_IP.')
         with mock.patch('heat.engine.clients.os.neutron.NeutronClientPlugin.'
                         'has_extension', return_value=True):
-            self.assertRaisesRegexp(exception.StackValidationFailed,
-                                    msg, self.pool.validate)
+            self.assertRaisesRegex(exception.StackValidationFailed,
+                                   msg, self.pool.validate)
 
     def test_create(self):
         self._create_stack()
@@ -97,6 +97,7 @@ class PoolTest(common.HeatTestCase):
                 },
                 'lb_algorithm': 'ROUND_ROBIN',
                 'listener_id': '123',
+                'loadbalancer_id': 'my_lb',
                 'protocol': 'HTTP',
                 'admin_state_up': True
             }
@@ -112,13 +113,20 @@ class PoolTest(common.HeatTestCase):
         self.assertTrue(self.pool.check_create_complete(props))
 
     def test_create_missing_properties(self):
+        self.patchobject(pool.Pool, 'is_service_available',
+                         return_value=(True, None))
+
         for prop in ('lb_algorithm', 'listener', 'protocol'):
             tmpl = yaml.load(inline_templates.POOL_TEMPLATE)
+            del tmpl['resources']['pool']['properties']['loadbalancer']
             del tmpl['resources']['pool']['properties'][prop]
             self._create_stack(tmpl=yaml.dump(tmpl))
-
-            self.assertRaises(exception.StackValidationFailed,
-                              self.pool.validate)
+            if prop == 'listener':
+                self.assertRaises(exception.PropertyUnspecifiedError,
+                                  self.pool.validate)
+            else:
+                self.assertRaises(exception.StackValidationFailed,
+                                  self.pool.validate)
 
     def test_show_resource(self):
         self._create_stack()
