@@ -362,7 +362,7 @@ class PoolTest(common.HeatTestCase):
                 'subnet',
                 'sub9999',
                 cmd_resource=None,
-            ).AndReturn('sub9999')
+            ).MultipleTimes().AndReturn('sub9999')
             neutronclient.Client.create_vip(stvipvsn
                                             ).AndReturn({'vip': {'id': 'xyz'}})
         else:
@@ -386,11 +386,7 @@ class PoolTest(common.HeatTestCase):
         self.m.VerifyAll()
 
     def test_create_with_vip_subnet(self):
-        rsrc = self.create_pool(with_vip_subnet=True)
-        self.m.ReplayAll()
-        scheduler.TaskRunner(rsrc.create)()
-        self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
-        self.m.VerifyAll()
+        self._test_create(with_vip_subnet=True)
 
     def test_create_pending(self):
         snippet = template_format.parse(pool_template)
@@ -433,7 +429,7 @@ class PoolTest(common.HeatTestCase):
         self.m.VerifyAll()
 
     def test_create_failed_error_status(self):
-        cfg.CONF.set_override('action_retry_limit', 0, enforce_type=True)
+        cfg.CONF.set_override('action_retry_limit', 0)
 
         snippet = template_format.parse(pool_template)
         self.stack = utils.parse_stack(snippet)
