@@ -14,9 +14,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from keystoneclient.auth.identity import v3 as ks_v3_auth
-from keystoneclient import exceptions as keystone_exc
-from keystoneclient import session as ks_session
+from keystoneauth1 import exceptions as keystone_exc
+from keystoneauth1.identity import generic as ks_auth
+from keystoneauth1 import session as ks_session
 import mox
 from oslo_config import cfg
 import six
@@ -26,7 +26,8 @@ from heat.common import auth_password
 from heat.tests import common
 
 
-cfg.CONF.import_opt('keystone_backend', 'heat.common.heat_keystoneclient')
+cfg.CONF.import_opt('keystone_backend',
+                    'heat.engine.clients.os.keystone.heat_keystoneclient')
 
 
 EXPECTED_ENV_RESPONSE = {
@@ -39,54 +40,35 @@ EXPECTED_ENV_RESPONSE = {
     'HTTP_X_AUTH_TOKEN': 'lalalalalala',
 }
 
-
-TOKEN_V2_RESPONSE = {
-    'token': {
-        'id': 'lalalalalala',
-        'expires': '2020-01-01T00:00:10.000123Z',
-        'tenant': {
-            'id': 'tenant_id1',
-            'name': 'tenant_name1',
-        },
-    },
-    'user': {
-        'id': 'user_id1',
-        'name': 'user_name1',
-        'roles': [
-            {'name': 'role1'},
-            {'name': 'role2'},
-        ],
-    },
-    'serviceCatalog': {}
-}
-
-
 TOKEN_V3_RESPONSE = {
     'version': 'v3',
-    'project': {
-        'id': 'tenant_id1',
-        'name': 'tenant_name1',
-    },
-    'token': {
-        'id': 'lalalalalala',
-        'expires': '2020-01-01T00:00:10.000123Z',
-        'tenant': {
-            'id': 'tenant_id1',
-            'name': 'tenant_name1',
-        },
-        'methods': ['password'],
-    },
-    'user': {
-        'id': 'user_id1',
-        'name': 'user_name1',
-    },
-    'roles': [
-        {'name': 'role1'},
-        {'name': 'role2'},
-    ],
-    'auth_token': 'lalalalalala'
+    'project_id': 'tenant_id1',
+    'project_name': 'tenant_name1',
+    'user_id': 'user_id1',
+    'username': 'user_name1',
+    'service_catalog': None,
+    'role_names': ['role1', 'role2'],
+    'auth_token': 'lalalalalala',
+    'user_domain_id': 'domain1'
 }
 
+TOKEN_V2_RESPONSE = {
+    'version': 'v2',
+    'tenant_id': 'tenant_id1',
+    'tenant_name': 'tenant_name1',
+    'user_id': 'user_id1',
+    'service_catalog': None,
+    'username': 'user_name1',
+    'role_names': ['role1', 'role2'],
+    'auth_token': 'lalalalalala',
+    'user_domain_id': 'domain1'
+}
+
+
+class FakeAccessInfo(object):
+    def __init__(self, **args):
+        self.__dict__.update(args)
+
 
 class FakeApp(object):
     """This represents a WSGI app protected by our auth middleware."""
@@ -119,10 +101,10 @@ class KeystonePasswordAuthProtocolTest(common.HeatTestCase):
         self.response_headers = dict(headers)
 
     def test_valid_v2_request(self):
-        mock_auth = self.m.CreateMock(ks_v3_auth.Password)
-        self.m.StubOutWithMock(ks_v3_auth, 'Password')
+        mock_auth = self.m.CreateMock(ks_auth.Password)
+        self.m.StubOutWithMock(ks_auth, 'Password')
 
-        ks_v3_auth.Password(
+        ks_auth.Password(
             auth_url=self.config['auth_uri'],
             password='goodpassword',
             project_id='tenant_id1',
@@ -130,9 +112,8 @@ class KeystonePasswordAuthProtocolTest(common.HeatTestCase):
             username='user_name1').AndReturn(mock_auth)
 
         m = mock_auth.get_access(mox.IsA(ks_session.Session))
-        m.AndReturn(TOKEN_V2_RESPONSE)
+        m.AndReturn(FakeAccessInfo(**TOKEN_V2_RESPONSE))
 
-        self.app.expected_env['keystone.token_info'] = TOKEN_V2_RESPONSE
         self.m.ReplayAll()
         req = webob.Request.blank('/tenant_id1/')
         req.headers['X_AUTH_USER'] = 'user_name1'
@@ -143,21 +124,18 @@ class KeystonePasswordAuthProtocolTest(common.HeatTestCase):
         self.m.VerifyAll()
 
     def test_valid_v3_request(self):
-        mock_auth = self.m.CreateMock(ks_v3_auth.Password)
-        self.m.StubOutWithMock(ks_v3_auth, 'Password')
+        mock_auth = self.m.CreateMock(ks_auth.Password)
+        self.m.StubOutWithMock(ks_auth, 'Password')
 
-        ks_v3_auth.Password(auth_url=self.config['auth_uri'],
-                            password='goodpassword',
-                            project_id='tenant_id1',
-                            user_domain_id='domain1',
-                            username='user_name1').AndReturn(mock_auth)
+        ks_auth.Password(auth_url=self.config['auth_uri'],
+                         password='goodpassword',
+                         project_id='tenant_id1',
+                         user_domain_id='domain1',
+                         username='user_name1').AndReturn(mock_auth)
 
         m = mock_auth.get_access(mox.IsA(ks_session.Session))
-        m.AndReturn(TOKEN_V3_RESPONSE)
+        m.AndReturn(FakeAccessInfo(**TOKEN_V3_RESPONSE))
 
-        self.app.expected_env['keystone.token_info'] = {
-            'token': TOKEN_V3_RESPONSE
-        }
         self.m.ReplayAll()
         req = webob.Request.blank('/tenant_id1/')
         req.headers['X_AUTH_USER'] = 'user_name1'
@@ -168,13 +146,13 @@ class KeystonePasswordAuthProtocolTest(common.HeatTestCase):
         self.m.VerifyAll()
 
     def test_request_with_bad_credentials(self):
-        self.m.StubOutWithMock(ks_v3_auth, 'Password')
+        self.m.StubOutWithMock(ks_auth, 'Password')
 
-        m = ks_v3_auth.Password(auth_url=self.config['auth_uri'],
-                                password='badpassword',
-                                project_id='tenant_id1',
-                                user_domain_id='domain1',
-                                username='user_name1')
+        m = ks_auth.Password(auth_url=self.config['auth_uri'],
+                             password='badpassword',
+                             project_id='tenant_id1',
+                             user_domain_id='domain1',
+                             username='user_name1')
         m.AndRaise(keystone_exc.Unauthorized(401))
 
         self.m.ReplayAll()
