@@ -40,6 +40,7 @@ class TestRequestContext(common.HeatTestCase):
                     'user': 'mick',
                     'password': 'foo',
                     'trust_id': None,
+                    'global_request_id': None,
                     'show_deleted': False,
                     'roles': ['arole', 'notadmin'],
                     'tenant_id': '456tenant',
@@ -49,8 +50,8 @@ class TestRequestContext(common.HeatTestCase):
                     'aws_creds': 'blah',
                     'region_name': 'RegionOne',
                     'user_identity': 'fooUser 456tenant',
-                    'user_domain_id': None,
-                    'project_domain_id': None}
+                    'user_domain': None,
+                    'project_domain': None}
 
         super(TestRequestContext, self).setUp()
 
@@ -128,7 +129,7 @@ class TestRequestContext(common.HeatTestCase):
     def test_keystone_v3_endpoint_in_context(self):
         """Ensure that the context is the preferred source for the auth_uri."""
         cfg.CONF.set_override('auth_uri', 'http://xyz',
-                              group='clients_keystone', enforce_type=True)
+                              group='clients_keystone')
         policy_check = 'heat.common.policy.Enforcer.check_is_admin'
         with mock.patch(policy_check) as pc:
             pc.return_value = False
@@ -144,7 +145,7 @@ class TestRequestContext(common.HeatTestCase):
         the preferred source when the context does not have the auth_uri.
         """
         cfg.CONF.set_override('auth_uri', 'http://xyz',
-                              group='clients_keystone', enforce_type=True)
+                              group='clients_keystone')
         policy_check = 'heat.common.policy.Enforcer.check_is_admin'
         with mock.patch(policy_check) as pc:
             pc.return_value = False
@@ -166,7 +167,7 @@ class TestRequestContext(common.HeatTestCase):
         """
         importutils.import_module('keystonemiddleware.auth_token')
         cfg.CONF.set_override('auth_uri', 'http://abc/v2.0',
-                              group='keystone_authtoken', enforce_type=True)
+                              group='keystone_authtoken')
         policy_check = 'heat.common.policy.Enforcer.check_is_admin'
         with mock.patch(policy_check) as pc:
             pc.return_value = False
