@@ -49,14 +49,12 @@ class TestPolicyEnforcer(common.HeatTestCase):
         return policy_path + filename
 
     def test_policy_cfn_default(self):
-        enforcer = policy.Enforcer(
-            scope='cloudformation',
-            policy_file=self.get_policy_file('deny_stack_user.json'))
+        enforcer = policy.Enforcer(scope='cloudformation')
 
         ctx = utils.dummy_context(roles=[])
         for action in self.cfn_actions:
             # Everything should be allowed
-            enforcer.enforce(ctx, action)
+            enforcer.enforce(ctx, action, is_registered_policy=True)
 
     def test_policy_cfn_notallowed(self):
         enforcer = policy.Enforcer(
@@ -67,100 +65,70 @@ class TestPolicyEnforcer(common.HeatTestCase):
         for action in self.cfn_actions:
             # Everything should raise the default exception.Forbidden
             self.assertRaises(exception.Forbidden, enforcer.enforce, ctx,
-                              action, {})
+                              action, {}, is_registered_policy=True)
 
     def test_policy_cfn_deny_stack_user(self):
-        enforcer = policy.Enforcer(
-            scope='cloudformation',
-            policy_file=self.get_policy_file('deny_stack_user.json'))
+        enforcer = policy.Enforcer(scope='cloudformation')
 
         ctx = utils.dummy_context(roles=['heat_stack_user'])
         for action in self.cfn_actions:
             # Everything apart from DescribeStackResource should be Forbidden
             if action == "DescribeStackResource":
-                enforcer.enforce(ctx, action)
+                enforcer.enforce(ctx, action, is_registered_policy=True)
             else:
                 self.assertRaises(exception.Forbidden, enforcer.enforce, ctx,
-                                  action, {})
+                                  action, {}, is_registered_policy=True)
 
     def test_policy_cfn_allow_non_stack_user(self):
-        enforcer = policy.Enforcer(
-            scope='cloudformation',
-            policy_file=self.get_policy_file('deny_stack_user.json'))
+        enforcer = policy.Enforcer(scope='cloudformation')
 
         ctx = utils.dummy_context(roles=['not_a_stack_user'])
         for action in self.cfn_actions:
             # Everything should be allowed
-            enforcer.enforce(ctx, action)
-
-    def test_policy_cw_deny_stack_user(self):
-        enforcer = policy.Enforcer(
-            scope='cloudwatch',
-            policy_file=self.get_policy_file('deny_stack_user.json'))
-
-        ctx = utils.dummy_context(roles=['heat_stack_user'])
-        for action in self.cw_actions:
-            # Everything apart from PutMetricData should be Forbidden
-            if action == "PutMetricData":
-                enforcer.enforce(ctx, action)
-            else:
-                self.assertRaises(exception.Forbidden, enforcer.enforce, ctx,
-                                  action, {})
-
-    def test_policy_cw_allow_non_stack_user(self):
-        enforcer = policy.Enforcer(
-            scope='cloudwatch',
-            policy_file=self.get_policy_file('deny_stack_user.json'))
-
-        ctx = utils.dummy_context(roles=['not_a_stack_user'])
-        for action in self.cw_actions:
-            # Everything should be allowed
-            enforcer.enforce(ctx, action)
+            enforcer.enforce(ctx, action, is_registered_policy=True)
 
     def test_set_rules_overwrite_true(self):
-        enforcer = policy.Enforcer(
-            policy_file=self.get_policy_file('deny_stack_user.json'))
+        enforcer = policy.Enforcer()
         enforcer.load_rules(True)
         enforcer.set_rules({'test_heat_rule': 1}, True)
         self.assertEqual({'test_heat_rule': 1}, enforcer.enforcer.rules)
 
     def test_set_rules_overwrite_false(self):
-        enforcer = policy.Enforcer(
-            policy_file=self.get_policy_file('deny_stack_user.json'))
+        enforcer = policy.Enforcer()
         enforcer.load_rules(True)
         enforcer.load_rules(True)
         enforcer.set_rules({'test_heat_rule': 1}, False)
         self.assertIn('test_heat_rule', enforcer.enforcer.rules)
 
     def test_load_rules_force_reload_true(self):
-        enforcer = policy.Enforcer(
-            policy_file=self.get_policy_file('deny_stack_user.json'))
+        enforcer = policy.Enforcer()
         enforcer.load_rules(True)
         enforcer.set_rules({'test_heat_rule': 'test'})
         enforcer.load_rules(True)
         self.assertNotIn({'test_heat_rule': 'test'}, enforcer.enforcer.rules)
 
     def test_load_rules_force_reload_false(self):
-        enforcer = policy.Enforcer(
-            policy_file=self.get_policy_file('deny_stack_user.json'))
+        enforcer = policy.Enforcer()
         enforcer.load_rules(True)
         enforcer.load_rules(True)
         enforcer.set_rules({'test_heat_rule': 'test'})
         enforcer.load_rules(False)
         self.assertIn('test_heat_rule', enforcer.enforcer.rules)
 
-    def test_default_rule(self):
+    def test_no_such_action(self):
         ctx = utils.dummy_context(roles=['not_a_stack_user'])
-        enforcer = policy.Enforcer(
-            scope='cloudformation',
-            policy_file=self.get_policy_file('deny_stack_user.json'),
-            exc=None, default_rule='!')
+        enforcer = policy.Enforcer(scope='cloudformation')
         action = 'no_such_action'
-        self.assertFalse(enforcer.enforce(ctx, action))
+        msg = 'cloudformation:no_such_action has not been registered'
+        self.assertRaisesRegex(base_policy.PolicyNotRegistered,
+                               msg,
+                               enforcer.enforce,
+                               ctx, action,
+                               None, None,
+                               True)
 
     def test_check_admin(self):
-        enforcer = policy.Enforcer(
-            policy_file=self.get_policy_file('check_admin.json'))
+        enforcer = policy.Enforcer()
 
         ctx = utils.dummy_context(roles=[])
         self.assertFalse(enforcer.check_is_admin(ctx))
@@ -174,64 +142,74 @@ class TestPolicyEnforcer(common.HeatTestCase):
     def test_enforce_creds(self):
         enforcer = policy.Enforcer()
         ctx = utils.dummy_context(roles=['admin'])
-        self.m.StubOutWithMock(base_policy.Enforcer, 'enforce')
-        base_policy.Enforcer.enforce('context_is_admin', {},
-                                     ctx.to_policy_values(),
-                                     False, exc=None).AndReturn(True)
-        self.m.ReplayAll()
         self.assertTrue(enforcer.check_is_admin(ctx))
 
     def test_resource_default_rule(self):
         context = utils.dummy_context(roles=['non-admin'])
-        enforcer = policy.ResourceEnforcer(
-            policy_file=self.get_policy_file('resources.json'))
+        enforcer = policy.ResourceEnforcer()
         res_type = "OS::Test::NotInPolicy"
-        self.assertTrue(enforcer.enforce(context, res_type))
+        self.assertTrue(enforcer.enforce(context, res_type,
+                                         is_registered_policy=True))
 
     def test_resource_enforce_success(self):
         context = utils.dummy_context(roles=['admin'])
-        enforcer = policy.ResourceEnforcer(
-            policy_file=self.get_policy_file('resources.json'))
-        res_type = "OS::Test::AdminOnly"
-        self.assertTrue(enforcer.enforce(context, res_type))
+        enforcer = policy.ResourceEnforcer()
+        res_type = "OS::Keystone::User"
+        self.assertTrue(enforcer.enforce(context, res_type,
+                                         is_registered_policy=True))
 
     def test_resource_enforce_fail(self):
         context = utils.dummy_context(roles=['non-admin'])
-        enforcer = policy.ResourceEnforcer(
-            policy_file=self.get_policy_file('resources.json'))
-        res_type = "OS::Test::AdminOnly"
+        enforcer = policy.ResourceEnforcer()
+        res_type = "OS::Nova::Quota"
         ex = self.assertRaises(exception.Forbidden,
                                enforcer.enforce,
-                               context, res_type)
+                               context, res_type,
+                               None, None,
+                               True)
         self.assertIn(res_type, ex.message)
 
     def test_resource_wildcard_enforce_fail(self):
         context = utils.dummy_context(roles=['non-admin'])
-        enforcer = policy.ResourceEnforcer(
-            policy_file=self.get_policy_file('resources.json'))
+        enforcer = policy.ResourceEnforcer()
         res_type = "OS::Keystone::User"
         ex = self.assertRaises(exception.Forbidden,
                                enforcer.enforce,
-                               context, res_type)
+                               context, res_type,
+                               None, None,
+                               True)
+
         self.assertIn(res_type.split("::", 1)[0], ex.message)
 
     def test_resource_enforce_returns_false(self):
         context = utils.dummy_context(roles=['non-admin'])
-        enforcer = policy.ResourceEnforcer(
-            policy_file=self.get_policy_file('resources.json'),
-            exc=None)
-        res_type = "OS::Test::AdminOnly"
-        self.assertFalse(enforcer.enforce(context, res_type))
-        self.assertIsNotNone(enforcer.enforce(context, res_type))
+        enforcer = policy.ResourceEnforcer(exc=None)
+        res_type = "OS::Keystone::User"
+        self.assertFalse(enforcer.enforce(context, res_type,
+                                          is_registered_policy=True))
+        self.assertIsNotNone(enforcer.enforce(context, res_type,
+                                              is_registered_policy=True))
 
     def test_resource_enforce_exc_on_false(self):
         context = utils.dummy_context(roles=['non-admin'])
-        enforcer = policy.ResourceEnforcer(
-            policy_file=self.get_policy_file('resources.json'))
-        res_type = "OS::Test::AdminOnly"
-        self.patchobject(base_policy.Enforcer, 'enforce',
-                         return_value=False)
+        enforcer = policy.ResourceEnforcer()
+        res_type = "OS::Keystone::User"
         ex = self.assertRaises(exception.Forbidden,
                                enforcer.enforce,
-                               context, res_type)
+                               context, res_type,
+                               None, None,
+                               True)
+
+        self.assertIn(res_type, ex.message)
+
+    def test_resource_enforce_override_deny_admin(self):
+        context = utils.dummy_context(roles=['admin'])
+        enforcer = policy.ResourceEnforcer(
+            policy_file=self.get_policy_file('resources.json'))
+        res_type = "OS::Cinder::Quota"
+        ex = self.assertRaises(exception.Forbidden,
+                               enforcer.enforce,
+                               context, res_type,
+                               None, None,
+                               True)
         self.assertIn(res_type, ex.message)
