@@ -23,6 +23,7 @@ from heat.common import identifier
 from heat.common import template_format
 from heat.common import urlfetch
 from heat.engine import api
+from heat.engine import node_data
 from heat.engine import resource
 from heat.engine.resources.aws.cfn import stack as stack_res
 from heat.engine import rsrc_defn
@@ -249,7 +250,8 @@ Resources:
         with mock.patch('heat.common.template_format.parse') as mock_parse:
             mock_parse.return_value = 'child_template'
             self.assertEqual('child_template', nested_stack.child_template())
-            mock_parse.assert_called_once_with('template_file')
+            mock_parse.assert_called_once_with(
+                'template_file', 'https://server.test/the.template')
 
     def test_child_template_when_fetching_file_fails(self):
         urlfetch.get.side_effect = exceptions.RequestException()
@@ -276,15 +278,15 @@ Resources:
         t = template_format.parse(self.test_template)
         tmpl = template.Template(t)
         ctx = utils.dummy_context()
-        cache_data = {'the_nested': {
+        cache_data = {'the_nested': node_data.NodeData.from_dict({
             'uuid': mock.ANY,
             'id': mock.ANY,
             'action': 'CREATE',
             'status': 'COMPLETE',
             'reference_id': 'the_nested_convg_mock'
-        }}
+        })}
         stack = parser.Stack(ctx, 'test_stack', tmpl, cache_data=cache_data)
-        nested_stack = stack['the_nested']
+        nested_stack = stack.defn['the_nested']
         self.assertEqual('the_nested_convg_mock', nested_stack.FnGetRefId())
 
     def test_get_attribute(self):
