@@ -784,8 +784,9 @@ class PropertyTest(common.HeatTestCase):
         # python 3.4.3 returns another error message
         # try to handle this by regexp
         self.assertRaisesRegex(
-            TypeError, "int\(\) argument must be a string(, a bytes-like "
-                       "object)? or a number, not 'list'", p.get_value, [1])
+            TypeError, r"int\(\) argument must be a string"
+                       "(, a bytes-like object)?"
+                       " or a number, not 'list'", p.get_value, [1])
 
     def test_str_from_int(self):
         schema = {'Type': 'String'}
@@ -1686,15 +1687,15 @@ class PropertiesValidationTest(common.HeatTestCase):
         schema = {'foo': {'Type': 'String'}}
         props = properties.Properties(schema, {'foo': ['foo', 'bar']})
         ex = self.assertRaises(exception.StackValidationFailed, props.validate)
-        self.assertEqual('Property error: foo: Value must be a string',
-                         six.text_type(ex))
+        self.assertIn('Property error: foo: Value must be a string',
+                      six.text_type(ex))
 
     def test_dict_instead_string(self):
         schema = {'foo': {'Type': 'String'}}
         props = properties.Properties(schema, {'foo': {'foo': 'bar'}})
         ex = self.assertRaises(exception.StackValidationFailed, props.validate)
-        self.assertEqual('Property error: foo: Value must be a string',
-                         six.text_type(ex))
+        self.assertIn('Property error: foo: Value must be a string',
+                      six.text_type(ex))
 
     def test_none_string(self):
         schema = {'foo': {'Type': 'String'}}
