@@ -207,6 +207,19 @@ test_template_findinmap_invalid = '''
 }
 '''
 
+test_template_bad_yaql_metadata = '''
+heat_template_version: 2016-10-14
+parameters:
+resources:
+  my_instance:
+    type: OS::Heat::TestResource
+    metadata:
+      test:
+        yaql:
+          expression: {'foo': 'bar'}
+          data: "$.data"
+'''
+
 test_template_invalid_resources = '''
 {
   "AWSTemplateFormatVersion" : "2010-09-09",
@@ -880,6 +893,10 @@ parameters:
     type: string
     description: Name of private network to be created
 
+  merged_param:
+    type: comma_delimited_list
+    description: A merged list of values
+
 resources:
   private_net:
     type: OS::Neutron::Net
@@ -924,6 +941,46 @@ resources:
     external_id: foobar
 '''
 
+test_template_hot_parameter_tags_older = '''
+heat_template_version: 2013-05-23
+
+parameters:
+  KeyName:
+    type: string
+    description: Name of an existing key pair to use for the instance
+    label: Nova KeyPair Name
+    tags:
+      - feature1
+      - feature2
+
+'''
+
+test_template_hot_parameter_tags_pass = '''
+heat_template_version: 2018-03-02
+
+parameters:
+  KeyName:
+    type: string
+    description: Name of an existing key pair to use for the instance
+    label: Nova KeyPair Name
+    tags:
+      - feature1
+      - feature2
+
+'''
+
+test_template_hot_parameter_tags_fail = '''
+heat_template_version: 2018-03-02
+
+parameters:
+  KeyName:
+    type: string
+    description: Name of an existing key pair to use for the instance
+    label: Nova KeyPair Name
+    tags: feature
+
+'''
+
 
 class ValidateTest(common.HeatTestCase):
     def setUp(self):
@@ -939,6 +996,11 @@ class ValidateTest(common.HeatTestCase):
         self.mock_is_service_available = self.mock_isa.start()
         self.addCleanup(self.mock_isa.stop)
         self.engine = service.EngineService('a', 't')
+        self.empty_environment = {
+            'event_sinks': [],
+            'parameter_defaults': {},
+            'parameters': {},
+            'resource_registry': {'resources': {}}}
 
     def _mock_get_image_id_success(self, imageId):
         self.patchobject(glance.GlanceClientPlugin, 'find_image_by_name_or_id',
@@ -1005,6 +1067,12 @@ class ValidateTest(common.HeatTestCase):
         res = dict(self.engine.validate_template(self.ctx, t, {}))
         self.assertNotEqual(res['Description'], 'Successfully validated')
 
+    def test_validate_bad_yaql_metadata(self):
+        t = template_format.parse(test_template_bad_yaql_metadata)
+        res = dict(self.engine.validate_template(self.ctx, t, {}))
+        self.assertIn('Error', res)
+        self.assertIn('yaql', res['Error'])
+
     def test_validate_parameters(self):
         t = template_format.parse(test_template_ref % 'WikiDatabase')
         res = dict(self.engine.validate_template(self.ctx, t, {}))
@@ -1043,7 +1111,8 @@ class ValidateTest(common.HeatTestCase):
         other_template = test_template_no_default.replace(
             'net_name', 'net_name2')
 
-        files = {'env1': 'parameter_defaults:\n  net_name: net1',
+        files = {'env1': 'parameter_defaults:'
+                         '\n  net_name: net1',
                  'env2': 'parameter_defaults:'
                          '\n  net_name: net2'
                          '\n  net_name2: net3',
@@ -1051,12 +1120,72 @@ class ValidateTest(common.HeatTestCase):
                  'tmpl2.yaml': other_template}
         params = {'parameters': {}, 'parameter_defaults': {}}
 
-        self.engine.validate_template(
+        ret = self.engine.validate_template(
             self.ctx, t,
             params=params,
             files=files, environment_files=['env1', 'env2'])
         self.assertEqual('net2', params['parameter_defaults']['net_name'])
         self.assertEqual('net3', params['parameter_defaults']['net_name2'])
+        expected = {
+            'Description': 'No description',
+            'Parameters': {
+                'size': {'AllowedValues': [1, 4, 8],
+                         'Description': '',
+                         'Label': u'size',
+                         'NoEcho': 'false',
+                         'Type': 'Number'}},
+            'Environment': {
+                'event_sinks': [],
+                'parameter_defaults': {
+                    'net_name': u'net2',
+                    'net_name2': u'net3'},
+                'parameters': {},
+                'resource_registry': {'resources': {}}}}
+
+        self.assertEqual(expected, ret)
+
+    def test_validate_parameters_merged_env(self):
+        t = template_format.parse(test_template_allowed_integers)
+
+        other_template = test_template_no_default.replace(
+            'net_name', 'net_name2')
+
+        files = {'env1': 'parameter_defaults:'
+                         '\n  net_name: net1'
+                         '\n  merged_param: [net1, net2]'
+                         '\nparameter_merge_strategies:'
+                         '\n  merged_param: merge',
+                 'env2': 'parameter_defaults:'
+                         '\n  net_name: net2'
+                         '\n  net_name2: net3'
+                         '\n  merged_param: [net3, net4]'
+                         '\nparameter_merge_strategies:'
+                         '\n  merged_param: merge',
+                 'tmpl1.yaml': test_template_no_default,
+                 'tmpl2.yaml': other_template}
+        params = {'parameters': {}, 'parameter_defaults': {}}
+
+        expected = {
+            'Description': 'No description',
+            'Parameters': {
+                'size': {'AllowedValues': [1, 4, 8],
+                         'Description': '',
+                         'Label': u'size',
+                         'NoEcho': 'false',
+                         'Type': 'Number'}},
+            'Environment': {
+                'event_sinks': [],
+                'parameter_defaults': {
+                    'net_name': u'net2',
+                    'net_name2': u'net3',
+                    'merged_param': ['net1', 'net2', 'net3', 'net4']},
+                'parameters': {},
+                'resource_registry': {'resources': {}}}}
+        ret = self.engine.validate_template(
+            self.ctx, t,
+            params=params,
+            files=files, environment_files=['env1', 'env2'])
+        self.assertEqual(expected, ret)
 
     def test_validate_hot_empty_parameters_valid(self):
         t = template_format.parse(
@@ -1149,7 +1278,8 @@ class ValidateTest(common.HeatTestCase):
 
         res = dict(self.engine.validate_template(self.ctx, t, {}))
         expected = {"Description": "test.",
-                    "Parameters": {}}
+                    "Parameters": {},
+                    "Environment": self.empty_environment}
         self.assertEqual(expected, res)
 
     def test_validate_hot_empty_outputs_valid(self):
@@ -1162,7 +1292,8 @@ class ValidateTest(common.HeatTestCase):
 
         res = dict(self.engine.validate_template(self.ctx, t, {}))
         expected = {"Description": "test.",
-                    "Parameters": {}}
+                    "Parameters": {},
+                    "Environment": self.empty_environment}
         self.assertEqual(expected, res)
 
     def test_validate_properties(self):
@@ -1232,14 +1363,18 @@ class ValidateTest(common.HeatTestCase):
         t = template_format.parse(test_template_snapshot_deletion_policy)
 
         res = dict(self.engine.validate_template(self.ctx, t, {}))
-        self.assertEqual(
-            {'Error': '"Snapshot" deletion policy not supported'}, res)
+        self.assertEqual({'Error': 'Resources.WikiDatabase.DeletionPolicy: '
+                                   '"Snapshot" deletion policy '
+                                   'not supported'},
+                         res)
 
     def test_volume_snapshot_deletion_policy(self):
         t = template_format.parse(test_template_volume_snapshot)
 
         res = dict(self.engine.validate_template(self.ctx, t, {}))
-        self.assertEqual({'Description': u'test.', 'Parameters': {}}, res)
+        expected = {'Description': u'test.', 'Parameters': {},
+                    'Environment': self.empty_environment}
+        self.assertEqual(expected, res)
 
     def test_validate_template_without_resources(self):
         hot_tpl = template_format.parse('''
@@ -1247,7 +1382,8 @@ class ValidateTest(common.HeatTestCase):
         ''')
 
         res = dict(self.engine.validate_template(self.ctx, hot_tpl, {}))
-        expected = {'Description': 'No description', 'Parameters': {}}
+        expected = {'Description': 'No description', 'Parameters': {},
+                    'Environment': self.empty_environment}
         self.assertEqual(expected, res)
 
     def test_validate_template_with_invalid_resource_type(self):
@@ -1755,7 +1891,8 @@ class ValidateTest(common.HeatTestCase):
             t,
             {},
             ignorable_errors=[exception.ResourceTypeUnavailable.error_code]))
-        expected = {'Description': 'No description', 'Parameters': {}}
+        expected = {'Description': 'No description', 'Parameters': {},
+                    'Environment': self.empty_environment}
         self.assertEqual(expected, res)
 
     def test_validate_with_ignorable_errors_invalid_error_code(self):
@@ -1835,7 +1972,14 @@ parameter_groups:
                             'NoEcho': 'false',
                             'Type': 'String'}},
                     'Type': 'OS::Test::TestResource'}},
-        }
+            'Environment': {
+                'event_sinks': [],
+                'parameter_defaults': {},
+                'parameters': {},
+                'resource_registry': {
+                    'OS::Test::TestResource':
+                    'https://server.test/nested.template',
+                    'resources': {}}}}
         self.assertEqual(expected, res)
 
     def test_validate_allowed_external_rsrc(self):
@@ -1859,3 +2003,39 @@ parameter_groups:
                                 self.ctx, t, {})
         self.assertEqual(dependencies.CircularDependencyException,
                          exc.exc_info[0])
+
+    def test_validate_hot_parameter_tags_older(self):
+        t = template_format.parse(test_template_hot_parameter_tags_older)
+
+        exc = self.assertRaises(dispatcher.ExpectedException,
+                                self.engine.validate_template,
+                                self.ctx, t, {})
+        self.assertEqual(exception.InvalidSchemaError,
+                         exc.exc_info[0])
+
+    def test_validate_hot_parameter_tags_pass(self):
+        t = template_format.parse(test_template_hot_parameter_tags_pass)
+
+        res = dict(self.engine.validate_template(self.ctx, t, {}))
+        parameters = res['Parameters']
+
+        expected = {'KeyName': {
+            'Description': 'Name of an existing key pair to use for the '
+                           'instance',
+            'NoEcho': 'false',
+            'Label': 'Nova KeyPair Name',
+            'Type': 'String',
+            'Tags': [
+                'feature1',
+                'feature2'
+            ]}}
+        self.assertEqual(expected, parameters)
+
+    def test_validate_hot_parameter_tags_fail(self):
+        t = template_format.parse(test_template_hot_parameter_tags_fail)
+
+        exc = self.assertRaises(dispatcher.ExpectedException,
+                                self.engine.validate_template,
+                                self.ctx, t, {})
+        self.assertEqual(exception.InvalidSchemaError,
+                         exc.exc_info[0])
