@@ -19,7 +19,6 @@ import time
 import fixtures
 from heatclient import exc as heat_exceptions
 from keystoneauth1 import exceptions as kc_exceptions
-from neutronclient.common import exceptions as network_exceptions
 from oslo_log import log as logging
 from oslo_utils import timeutils
 import six
@@ -30,7 +29,6 @@ import testtools
 from heat_integrationtests.common import clients
 from heat_integrationtests.common import config
 from heat_integrationtests.common import exceptions
-from heat_integrationtests.common import remote_client
 
 LOG = logging.getLogger(__name__)
 _LOG_FORMAT = "%(levelname)8s [%(name)s] %(message)s"
@@ -68,13 +66,25 @@ def rand_name(name=''):
         return randbits
 
 
+def requires_convergence(test_method):
+    '''Decorator for convergence-only tests.
+
+    The decorated test will be skipped when convergence is disabled.
+    '''
+    convergence_enabled = config.init_conf(
+    ).heat_plugin.convergence_engine_enabled
+    skipper = testtools.skipUnless(convergence_enabled,
+                                   "Convergence-only tests are disabled")
+    return skipper(test_method)
+
+
 class HeatIntegrationTest(testscenarios.WithScenarios,
                           testtools.TestCase):
 
     def setUp(self):
         super(HeatIntegrationTest, self).setUp()
 
-        self.conf = config.CONF.heat_plugin
+        self.conf = config.init_conf().heat_plugin
 
         self.assertIsNotNone(self.conf.auth_url,
                              'No auth_url configured')
@@ -98,32 +108,12 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
         self.network_client = self.manager.network_client
         self.volume_client = self.manager.volume_client
         self.object_client = self.manager.object_client
-        self.metering_client = self.manager.metering_client
 
         self.client = self.orchestration_client
 
     def setup_clients_for_admin(self):
         self.setup_clients(self.conf, True)
 
-    def get_remote_client(self, server_or_ip, username, private_key=None):
-        if isinstance(server_or_ip, six.string_types):
-            ip = server_or_ip
-        else:
-            network_name_for_ssh = self.conf.network_for_ssh
-            ip = server_or_ip.networks[network_name_for_ssh][0]
-        if private_key is None:
-            private_key = self.keypair.private_key
-        linux_client = remote_client.RemoteClient(ip, username,
-                                                  pkey=private_key,
-                                                  conf=self.conf)
-        try:
-            linux_client.validate_authentication()
-        except exceptions.SSHTimeout:
-            LOG.exception('ssh connection to %s failed', ip)
-            raise
-
-        return linux_client
-
     def check_connectivity(self, check_ip):
         def try_connect(ip):
             try:
@@ -188,13 +178,6 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
             if net['name'] == net_name:
                 return net
 
-    def is_network_extension_supported(self, extension_alias):
-        try:
-            self.network_client.show_extension(extension_alias)
-        except network_exceptions.NeutronClientException:
-            return False
-        return True
-
     def is_service_available(self, service_type):
         try:
             self.identity_client.get_endpoint_url(
@@ -286,11 +269,15 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
             return False
         return res.resource_status == status
 
-    def _verify_status(self, stack, stack_identifier, status, fail_regexp):
+    def _verify_status(self, stack, stack_identifier, status,
+                       fail_regexp, is_action_cancelled=False):
         if stack.stack_status == status:
             # Handle UPDATE_COMPLETE/FAILED case: Make sure we don't
             # wait for a stale UPDATE_COMPLETE/FAILED status.
             if status in ('UPDATE_FAILED', 'UPDATE_COMPLETE'):
+                if is_action_cancelled:
+                    return True
+
                 if self.updated_time.get(
                         stack_identifier) != stack.updated_time:
                     self.updated_time[stack_identifier] = stack.updated_time
@@ -324,7 +311,8 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
                                failure_pattern=None,
                                success_on_not_found=False,
                                signal_required=False,
-                               resources_to_signal=None):
+                               resources_to_signal=None,
+                               is_action_cancelled=False):
         """Waits for a Stack to reach a given status.
 
         Note this compares the full $action_$status, e.g
@@ -350,11 +338,16 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
             except heat_exceptions.HTTPNotFound:
                 if success_on_not_found:
                     return
+                elif not any(s in status for s in ['CREATE', 'ADOPT']):
+                    # raise exception when stack not found, if it's not
+                    # in create or adopt (which should be the only two possible
+                    # reason that stack may not have been created yet)
+                    raise
                 # ignore this, as the resource may not have
                 # been created yet
             else:
                 if self._verify_status(stack, stack_identifier, status,
-                                       fail_regexp):
+                                       fail_regexp, is_action_cancelled):
                     return
             if signal_required:
                 self.signal_resources(resources_to_signal)
@@ -425,7 +418,7 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
 
         self._wait_for_stack_status(**kwargs)
 
-    def cancel_update_stack(self, stack_identifier,
+    def cancel_update_stack(self, stack_identifier, rollback=True,
                             expected_status='ROLLBACK_COMPLETE'):
 
         stack_name = stack_identifier.split('/')[0]
@@ -433,10 +426,16 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
         self.updated_time[stack_identifier] = self.client.stacks.get(
             stack_identifier, resolve_outputs=False).updated_time
 
-        self.client.actions.cancel_update(stack_name)
+        if rollback:
+            self.client.actions.cancel_update(stack_name)
+        else:
+            self.client.actions.cancel_without_rollback(stack_name)
 
         kwargs = {'stack_identifier': stack_identifier,
                   'status': expected_status}
+        if expected_status == 'UPDATE_FAILED':
+            kwargs['is_action_cancelled'] = True
+
         if expected_status in ['ROLLBACK_COMPLETE']:
             # To trigger rollback you would intentionally fail the stack
             # Hence check for rollback failures
@@ -583,7 +582,7 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
             environment_files=environment_files,
             timeout_mins=timeout_mins
         )
-        if expected_status not in ['ROLLBACK_COMPLETE'] and enable_cleanup:
+        if enable_cleanup:
             self.addCleanup(self._stack_delete, name)
 
         stack = self.client.stacks.get(name, resolve_outputs=False)
@@ -683,13 +682,13 @@ class HeatIntegrationTest(testscenarios.WithScenarios,
             time.sleep(build_interval)
 
     def check_autoscale_complete(self, stack_id, expected_num, parent_stack,
-                                 policy):
+                                 group_name):
         res_list = self.client.resources.list(stack_id)
         all_res_complete = all(res.resource_status in ('UPDATE_COMPLETE',
                                                        'CREATE_COMPLETE')
                                for res in res_list)
         all_res = len(res_list) == expected_num
         if all_res and all_res_complete:
-            metadata = self.client.resources.metadata(parent_stack, policy)
+            metadata = self.client.resources.metadata(parent_stack, group_name)
             return not metadata.get('scaling_in_progress')
         return False
